#include "Agent.hpp"
#include "WorldEngine.hpp"
#include <cassert>
#include <iostream>
#include "LuaFunctions.hpp"
#include "GridManager.hpp"

Agent::Agent(Vec2 pos, const char* brain):
  TreeNode(pos),
  m_brainFunctionName(brain)
{
    m_lua = lua_newthread(WorldEngine::get_instance().get_lua_state());
    assert(m_lua && "lua thread initialization failed");
    m_luaRef = luaL_ref(WorldEngine::get_instance().get_lua_state(), LUA_REGISTRYINDEX);
    lua_getglobal(m_lua, m_brainFunctionName.c_str());
//     WorldEngine::get_instance().set_current_agent_id(get_id());
    set_script_name(m_brainFunctionName);
    int s = lua_resume(m_lua, 0);
    if(s != LUA_YIELD){
      throw LuaScript::Exception("Could not spawn agent due to an error in its script");
    }
//       assert(s == LUA_YIELD && " brain could not yield...");
   
   // DebugStream << "Created Agent " << get_id() << std::endl;
   set_motion_model(MotionModel::MT_LINEAR);
   set_property("Visibility",50.0f);   
   set_property("Title",std::string(brain)); 
   set_property("Velocity",Vec2(0,0));
}





void Agent::tick(double dT)
{
    //DebugStream << "Agent tick"<<std::endl;
    TreeNode::tick(dT);
    if(!get_activity_state())return;
    
    // DebugStream << "Enter Tick() : " << m_id << std::endl;
    WorldEngine::get_instance().set_current_agent_id(get_id());
    try {
      int s = lua_resume(m_lua, 0);
      if(s != LUA_YIELD && s != 0) {
 	  DebugStream << "exception in agent " << get_id()  <<std::endl;
	  set_active_state(false);
// 	  kill();
// 	  throw AgentException();
// 	  remove_treenode_from_existence();
      }
      }catch(LuaScript::Exception& e){
	DebugStream << "Error found "<<std::endl;
	set_active_state(false);
	add_error(e.what());
      }catch(Property::Exception& e){
	DebugStream << "Property Error found "<<std::endl;
	set_active_state(false);
	add_error(e.what());
	
      }
}

TreeNode::TreeNodeType Agent::is_type() const
{
    return TN_AGENT;
}


std::string Agent::get_brain_function_name()
{
  return m_brainFunctionName;
}



