#ifndef GRIDMANAGER_HPP
#define GRIDMANAGER_HPP
#include <iostream>
#include <algorithm>
#include <vector>
#include <boost/shared_ptr.hpp>
#include <lo/lo_osc_types.h>
#include "Utilities.hpp"


class Scale{
public:
  typedef std::vector<float> ScaleValues;
  enum Direction{
      SD_VERTICAL =0,
      SD_HORIZONTAL
  };
private:
  ScaleValues m_values;
  Direction m_direction;
  bool m_active;
  
  
  void sort(){
      std::sort(m_values.begin(),m_values.end());
  }
public:
  Scale(Direction dir = SD_VERTICAL):m_direction(dir),m_active(true)
  {
  }
  void add_value(float value){
    m_values.push_back(value); 
    sort();
  }
  ScaleValues::iterator begin(){
      return m_values.begin();
  }
  ScaleValues::iterator end(){
      return m_values.end();
  }
  void set_activity(bool state){
      m_active = state;
  }
  //If not active it will return the current value
  float get_closest(float value){
   if(m_active){
      if(!m_values.empty()){ 
	ScaleValues::iterator lower;
	lower = std::lower_bound(m_values.begin(),m_values.end(),value);
	if(lower == m_values.end())return m_values.back();
	return (*lower);
      }  
    }else{
	return value;
    }
  }
  void print_scale(){
   for(int n =0;n<m_values.size();++n){
    DebugStream << "scale index = " <<n << " value = "<<m_values.at(n)<<std::endl; 
   }
  }
  void set_direction(Direction dir){
      m_direction = dir;
  }
  Direction get_direction(){
      return m_direction;
  }
  bool is_active(){
    return m_active;
  }
};

typedef boost::shared_ptr<Scale> ScalePtr;
typedef std::vector<ScalePtr> ScaleVector;







class GridManager
{
  static ScaleVector m_grids;
  GridManager();
public:
  static GridManager& get_instance();
  
  void add_scale(ScalePtr scale);
  ScalePtr get_scale(int index);
  float get_closest_value(int index, float value);
  
  static ScaleVector::iterator begin();
  static ScaleVector::iterator end();
  
};

#endif // GRIDMANAGER_HPP
