#include "LuaFunctions.hpp"
#include "WorldEngine.hpp"
#include <iostream>
#include "Agent.hpp"
#include "lo/lo.h"
#include "Vec2.hpp"
#include "lua/lobject.h"
#include "lua/llex.h"
#include <fstream>
#include "PathManager.hpp"
#include "Utilities.hpp"
#include <sstream>
#include <cstdarg>
#include <stdio.h>
#include <stdlib.h>

#include "TestOutput.hpp"

//#include "mainwindow.hpp"


float SafeGetNumberLua(lua_State* L, int index){
  if(lua_isnumber(L,index)){
    return lua_tonumber(L,index);
  } else {
   throw LuaScript::NumberException("Cannot retrieve number");
  }
}

std::string SafeGetStringLua(lua_State* L, int index){
  if(lua_isstring(L,index)){
    return lua_tostring(L,index);
  } else {
   throw LuaScript::StringException("Cannot retrieve string");
  }
}

void add_table_property(lua_State* L,int index, TreeNode::Ptr node){
     // DebugStream <<" Is table? " << lua_istable(L,index) << std::endl;
      //if(!lua_istable(L,index))return;
	lua_pushnil(L);  /* first key */
	PropertyValue val;
	std::string id ;

     while (lua_next(L, 1) != 0) {

       if(!lua_isstring(L,-2)  ){
	  return;
       }
       id = lua_tostring(L,-2);
       //DebugStream << "id = " << id<< std::endl;

	if( lua_isnumber(L,-1)){
	  val = float(lua_tonumber(L,-1));
	}else if (lua_isstring(L,-1)){
	  val = std::string(lua_tostring(L,-1));
	}else if(lua_isuserdata(L,-1)){
	  try{
	    val = *LuaScript::lvec2_check(L,-1);
	  }catch(...){
	    val = new Vec2(0,0);
	  }
	}else{
	  val = 0;
	}


	//DebugStream << "value = "<< val<<std::endl;
	if(!node->property_exists(id)){
	 // DebugStream << "property doesnt exist!"<<std::endl;
	  node->add_property(id,Property( val,Property::PF_READ | Property::PF_WRITE | Property::PF_SEND ));
	}else{
	  Property& prop = node->get_property(id);
	  prop.set_flags(Property::PF_READ | Property::PF_WRITE | Property::PF_SEND );
	  node->set_property(id,val);
	}

       lua_pop(L, 1);
     }

    return;


}







int LuaScript::initai(lua_State* L){
// 	DebugStream << "Initializing brain"<<std::endl;
	int size = lua_gettop(L);
	if(size > 0){
// 	  int id = WorldEngine::get_instance().get_current_agent_id();
// 	  TreeNode::Ptr node = TreeNode::get_treenode_by_id(id).lock();
// 	  DebugStream << "ID "<< id<<std::endl;
	  //wrong id is emmitted here.
	}
	return lua_yield(L,0);
}

int LuaScript::initproperties(lua_State* L)
{
	int id  =  lua_tointeger(L,-1);
// 	DebugStream << "init prop id = "<<id << " ________"<<std::endl;

	try{
	TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
	if(!lua_istable(L,-2))throw LuaScript::Exception("Cannot initialize properties without providing a table of properties");
	add_table_property(L,-2,activeAgent);

	}catch(TreeNode::IdException& e){
	  char buffer[255];
	  sprintf(buffer,"Could not initialize properties for Agent with ID %i as that Agent does not exist",id);
	  throw LuaScript::Exception(buffer);
	}
	return lua_yield(L,0);
}


int LuaScript::getdt(lua_State* L){
      try{
	lua_pushnumber(L, WorldEngine::get_instance().get_delta() );
	return 1;
      }catch(...){
	 char buffer[255];
	 sprintf(buffer,"Could not obtain the delta from the world");
	 throw LuaScript::Exception(buffer);
	 return 0;
      }
}

int LuaScript::getid(lua_State* L){
    try{
	lua_pushnumber(L, WorldEngine::get_instance().get_current_agent_id() );
	return 1;

    }catch(...){
        char buffer[255];
	sprintf(buffer,"Could not obtain ID of the current Agent!");
	throw LuaScript::Exception(buffer);
	return 0;
    }
}

int LuaScript::get_parentid(lua_State* L)
{
  int id = lua_tointeger(L,-1);
  try{
    TreeNode::Ptr t = TreeNode::get_treenode_by_id(id).lock();

    lua_pushinteger(L,t->get_parent_id());
    return 1;

  }catch(TreeNode::IdException& e){
    char buffer[255];
    sprintf(buffer,"Could not obtain the parent for the Agent with the ID: %i as an Agent with that ID does not exist",id);
    throw LuaScript::Exception(buffer);
  }


  return 0;

}







int LuaScript::kill(lua_State* L){
	int id = lua_tointeger(L,-1);
	try{
	  if(TreeNode::Ptr t = TreeNode::get_treenode_by_id(id).lock()){
	      t->kill();
	  }
	  lua_yield(L,0);
	  return 1;

	}catch(TreeNode::IdException& e){
	  char buffer[255];
	  sprintf(buffer,"Could Agent with ID %i as it does not exist",id);
	  throw LuaScript::Exception(buffer);
	}
}

int LuaScript::kill_children(lua_State* L)
{
	int id = lua_tointeger(L,-1);
	try{
	  if(TreeNode::Ptr t = TreeNode::get_treenode_by_id(id).lock()){
	      t->kill_children();
	  }
	  return lua_yield(L,0);
	}catch(TreeNode::IdException& e){
	  char buffer[255];
	  sprintf(buffer,"Could not kill the children for Agent with ID %i as that Agent does not exist",id);
	  throw LuaScript::Exception(buffer);
	}
}


int LuaScript::get_childId(lua_State* L)
{
  int id = lua_tointeger(L,-1);
  try{
  TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();

    int localChildId = lua_tointeger(L,-2);
    int childId = activeAgent->get_child_at(localChildId).get_id();
    lua_pushinteger(L,childId);
    return 1;
  }catch(TreeNode::IdException& e){
	  char buffer[255];
	  sprintf(buffer,"Could not initialize properties for Agent with ID %i as that Agent does not exist",id);
	  throw LuaScript::Exception(buffer);
	  return 0;
  }
}







int LuaScript::spawn(lua_State* L)
{
	int argNos = lua_gettop(L);
// 	DebugStream << "Spawn args "<< argNos<<std::endl;

	int parentid;
	TreeNode::TreeNodeType type;
	std::string brain;
	float posx;
	float posy;
	Vec2 pos;
	try{
	  if(argNos == 5){

	    parentid = lua_tointeger(L,-5);
	    type = TreeNode::TreeNodeType( lua_tointeger(L,-4));
	    brain = lua_tostring(L,-3);
	    float posx = lua_tonumber(L,-2);
	    float posy = lua_tonumber(L,-1);
	    pos = Vec2(posx,posy);

	  }else if(argNos ==4){
	    parentid = lua_tointeger(L,-4);
	    type = TreeNode::TreeNodeType( lua_tointeger(L,-3));
	    brain = lua_tostring(L,-2);
	    pos = *lvec2_check(L,-1);
	  }else{
	    char buffer[512];
	    //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	    sprintf(buffer,"Incorrect number of arguments provided for spawn function.\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),0 (nodeType),string (agentName),vec2 pos");
	    throw LuaScript::Exception(buffer);
	    return lua_error(L);
	  }
	}catch(LuaScript::Exception& e){
	      throw e;
	}catch(LuaScript::Vec2Exception& e){
	  char buffer[512];
	  //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	  sprintf(buffer,"Could not obtain Vec2 in spawn function.\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),0 (nodeType),string (agentName),vec2 pos");
	  throw LuaScript::Exception(buffer);
	}catch(...){
	  char buffer[512];
	  //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	  sprintf(buffer,"Incorrect arguments provided for spawn function.\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),0 (nodeType),string (agentName),vec2 pos");
	  throw LuaScript::Exception(buffer);

	}
	if(parentid==0) parentid = WorldEngine::get_instance().get_world()->get_id();


	TreeNode::Ptr parent;
	try{
	  parent = TreeNode::get_treenode_by_id(parentid).lock();
	}catch(TreeNode::IdException& e){
	  char buffer[255];
	  sprintf(buffer,"In spawn function, could not obtain parent Agent with ID %i as that Agent does not exist",parentid);
	  throw LuaScript::Exception(buffer);
	}


	TreeNode::Ptr child;
	std::string t(brain.c_str());
	if(t.find("ai_")==std::string::npos){
	  char buffer[255];
	  sprintf(buffer,"In spawn function, could not create Agent with brain function %s, check that that brain function has been loaded",brain.c_str());
	  throw LuaScript::Exception(buffer);
	}

	child = parent->add_child(TreeNode::factory(type,pos,brain.c_str()));


	lua_pushnumber(L,child->get_id());
	lua_yield(L,0);
	return 1;
}



int LuaScript::spawnArgs(lua_State* L)
{


	int argNos = lua_gettop(L);
// 	DebugStream << "Spawn args "<< argNos<<std::endl;



	int parentid;
	TreeNode::TreeNodeType type;
	std::string brain;
	float posx;
	float posy;
	Vec2 pos;
	try{
	if(argNos == 6){
	    parentid = lua_tointeger(L,-5);
	    type = TreeNode::TreeNodeType( lua_tointeger(L,-4));
	    brain = lua_tostring(L,-3);
    	    posx = lua_tonumber(L,-2);
	    posy = lua_tonumber(L,-1);
	    pos = Vec2(posx,posy);
	}else if (argNos == 5){
	   parentid = lua_tointeger(L,-4);
	    type = TreeNode::TreeNodeType( lua_tointeger(L,-3));
	    brain = lua_tostring(L,-2);
    	    pos = *lvec2_check(L,-1);
	}else{
	   char buffer[512];
	    //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	    sprintf(buffer,"Incorrect number of arguments provided for spawnP function.\nTry \n\t table (key value pairs of property name and value),int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t table (key value pairs of property name and value),int (parentID),int (nodeType),string (agentName),vec2 (position)");
	    throw LuaScript::Exception(buffer);
	    return lua_error(L);
	}
	}catch(LuaScript::Exception& e){
	    throw e;
	}catch(LuaScript::Vec2Exception& e){
	    char buffer[512];
	    //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	    sprintf(buffer,"Unable to obtain Vec2 in spawnP function.\nTry \n\t table (key value pairs of property name and value),int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t table (key value pairs of property name and value),int (parentID),int (nodeType),string (agentName),vec2 (position)");
	    throw LuaScript::Exception(buffer);
	    return lua_error(L);
	}catch(...){
	    char buffer[512];
	    //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	    sprintf(buffer,"Incorrect arguments provided for spawnP function.\nTry \n\t table (key value pairs of property name and value),int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t table (key value pairs of property name and value),int (parentID),int (nodeType),string (agentName),vec2 (position)");
	    throw LuaScript::Exception(buffer);
	    return lua_error(L);

	}
	if(parentid==0) parentid = WorldEngine::get_instance().get_world()->get_id();
	TreeNode::Ptr parent;
	try{
	  parent = TreeNode::get_treenode_by_id(parentid).lock();
	}catch(TreeNode::IdException& e){
	  char buffer[255];
	  sprintf(buffer,"In spawnp function, could not obtain parent Agent with ID %i as that Agent does not exist",parentid);
	  throw LuaScript::Exception(buffer);
	}

	TreeNode::Ptr child;
	std::string t(brain.c_str());
	if(t.find("ai_")==std::string::npos){
	  char buffer[255];
	  sprintf(buffer,"In spawnp function, could not create Agent with brain function %s, check that that brain function has been loaded",brain.c_str());
	  throw LuaScript::Exception(buffer);
	}

	child =   parent->add_child(TreeNode::factory(type,pos,brain.c_str()));

// 	argNos = lua_gettop(L);
// 	DebugStream << "Spawn args "<< argNos<<std::endl;
	int tablePos = -6;
	if(argNos == 6){
	  tablePos = -6;
	}else{
	    tablePos = -5;
	}
 	add_table_property(L,tablePos,child);


	lua_pushnumber(L,child->get_id());
	lua_yield(L,0);
	return 1;
}




int LuaScript::spawnrelative(lua_State* L)
{
	int argNos = lua_gettop(L);
// 	DebugStream << "Spawn args "<< argNos<<std::endl;


	int parentid;
	TreeNode::TreeNodeType type;
	std::string brain;
	float posx;
	float posy;
	Vec2 pos;
	try{

	  if(argNos == 5){
	    parentid = lua_tointeger(L,-5);
	    type = TreeNode::TreeNodeType( lua_tointeger(L,-4));
	    brain = SafeGetStringLua(L,-3);
	    posx = SafeGetNumberLua(L,-2);
	    posy = SafeGetNumberLua(L,-1);
	    pos = Vec2(posx,posy);

	  }else if(argNos == 4){

	    parentid = lua_tointeger(L,-4);
	    type = TreeNode::TreeNodeType( lua_tointeger(L,-3));
	    brain = SafeGetStringLua(L,-2);
	    pos = *lvec2_check(L,-1);
	  }else{
	      char buffer[512];
	      //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	      sprintf(buffer,"Incorrect number of arguments provided for spawnRelative function.\nTry \n\t int (parentID),int (nodeType),string (agentName),int (relative x position),int (relative y position) \n\t or \n\t int (parentID),int (nodeType),string (agentName),Vec2 (relative position)");
	      throw LuaScript::Exception(buffer);

	      return lua_error(L);
	  }
	}catch(LuaScript::Exception& e){
	  throw e;
	}catch(LuaScript::Vec2Exception& e){
	       char buffer[512];
	      //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	      sprintf(buffer,"Unable to obtain Vec2 in spawnRelative function.\nTry \n\t int (parentID),int (nodeType),string (agentName),int (relative x position),int (relative y position) \n\t or \n\t int (parentID),int (nodeType),string (agentName),Vec2 (relative position)");
	      throw LuaScript::Exception(buffer);
	}catch(...){
	      char buffer[512];
	      //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	      sprintf(buffer,"Incorrect arguments provided for spawnRelative function.\nTry \n\t int (parentID),int (nodeType),string (agentName),int (relative x position),int (relative y position) \n\t or \n\t int (parentID),int (nodeType),string (agentName),Vec2 (relative position)");
	      throw LuaScript::Exception(buffer);

	}

	if(parentid==0) parentid = WorldEngine::get_instance().get_world()->get_id();
	TreeNode::Ptr parent;
	try{
	 parent=  TreeNode::get_treenode_by_id(parentid).lock();
	}catch(TreeNode::IdException& e){
	  char buffer[255];
	  sprintf(buffer,"In spawnrelative function, could not obtain parent Agent with ID %i as that Agent does not exist",parentid);
	  throw LuaScript::Exception(buffer);
	}

	Vec2 parentPos = parent->get_pos();

	TreeNode::Ptr child;
	std::string t(brain.c_str());
	if(t.find("ai_")==std::string::npos){
	  char buffer[255];
	  sprintf(buffer,"In spawnrelative function, could not create Agent with brain function %s, check that that brain function has been loaded",brain.c_str());
	  throw LuaScript::Exception(buffer);
	}

	child =   parent->add_child(TreeNode::factory(type,parentPos+ pos,brain.c_str()));
// 	if(child)DebugStream << "relative shild spwned"<<std::endl;
	lua_pushnumber(L,child->get_id());
	lua_yield(L,0);
	return 1;
}

int LuaScript::spawnrelativeArgs(lua_State* L)
{
	int argNos = lua_gettop(L);
// 	DebugStream << "Spawn args "<< argNos<<std::endl;

	int parentid;
	TreeNode::TreeNodeType type;
	std::string brain;
	float posx;
	float posy;
	Vec2 pos;
	try{
	  if(argNos ==6 ){

	  parentid = lua_tointeger(L,-5);
	  type = TreeNode::TreeNodeType( lua_tointeger(L,-4));
	  brain = SafeGetStringLua(L,-3);
	  posx = SafeGetNumberLua(L,-2);
	  posy = SafeGetNumberLua(L,-1);
	  pos = Vec2(posx,posy);

	  }else if(argNos == 5){
	    parentid = lua_tointeger(L,-4);
	    type = TreeNode::TreeNodeType( lua_tointeger(L,-3));
	    brain = SafeGetStringLua(L,-2);
	    pos = *lvec2_check(L,-1);
	  }else{
	      char buffer[512];
	      //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	      sprintf(buffer,"Incorrect number of arguments provided for spawnRelativeP function.\nTry \n\t table (key value pairs of property name and value),int (parentID),int (nodeType),string (agentName),int (relative x position),int (relative y position) \n\t or \n\t table (key value pairs of property name and value), int (parentID),int (nodeType),string (agentName),Vec2 (relative position)");
	      throw LuaScript::Exception(buffer);

	      return lua_error(L);
	  }
	}catch(LuaScript::Exception& e){
	  throw e;
	}catch(LuaScript::Vec2Exception& e){
	    char buffer[512];
	    //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	    sprintf(buffer,"Unable to obtain Vec2 in spawnRelativeP function.\nTry \n\t table (key value pairs of property name and value),int (parentID),int (nodeType),string (agentName),int (relative x position),int (relative y position) \n\t or \n\t table (key value pairs of property name and value), int (parentID),int (nodeType),string (agentName),Vec2 (relative position)");
	    throw LuaScript::Exception(buffer);
	}

	catch(...){
	    char buffer[512];
	    //"\nTry \n\t int (parentID),int (nodeType),string (agentName),int (xPos),int (yPos) \n\t or \n\t int (parentID),int (nodeType),string (agentName),vec2 pos"
	    sprintf(buffer,"Incorrect arguments provided for spawnRelativeP function.\nTry \n\t table (key value pairs of property name and value),int (parentID),int (nodeType),string (agentName),int (relative x position),int (relative y position) \n\t or \n\t table (key value pairs of property name and value), int (parentID),int (nodeType),string (agentName),Vec2 (relative position)");
	    throw LuaScript::Exception(buffer);

	    return lua_error(L);
	}

	if(parentid==0) parentid = WorldEngine::get_instance().get_world()->get_id();
	TreeNode::Ptr parent;
	try{
	    parent =  TreeNode::get_treenode_by_id(parentid).lock();
	}catch(TreeNode::IdException& e){
	  char buffer[255];
	  sprintf(buffer,"In spawnRelativeP function, could not obtain parent Agent with ID %i as that Agent does not exist",parentid);
	  throw LuaScript::Exception(buffer);
	}

	Vec2 parentPos = parent->get_pos();

	TreeNode::Ptr child;
// 	if(child)DebugStream << "relative shild spwned"<<std::endl;

	std::string t(brain.c_str());
	if(t.find("ai_")==std::string::npos){
	  char buffer[255];
	  sprintf(buffer,"In spawnRelativeP function, could not create Agent with brain function %s, check that that brain function has been loaded",brain.c_str());
	  throw LuaScript::Exception(buffer);
	}

	child =   parent->add_child(TreeNode::factory(type,parentPos + pos,brain.c_str()));


	int tablePos = -6;
	if(argNos ==5)tablePos = -5;
	add_table_property(L,tablePos,child);

	lua_pushnumber(L,child->get_id());
	lua_yield(L,0);
	return 1;
}



int LuaScript::parseTable(lua_State* L)
{

//     DebugStream <<" Is table? " << lua_istable(L,-1) << std::endl;
     lua_pushnil(L);  /* first key */
     PropertyValue val;
     std::string id ;

     while (lua_next(L, 1) != 0) {

       if(!lua_isstring(L,-2)  ){
	  return 0;
       }
       id = lua_tostring(L,-2);
//        DebugStream << "id = " << id<< std::endl;

	if( lua_isnumber(L,-1)){
	  val = float(lua_tonumber(L,-1));
	}else if (lua_isstring(L,-1)){
	  val = std::string(lua_tostring(L,-1));
	}else if(lua_isuserdata(L,-1)){
	  try{
	  val = *lvec2_check(L,-1);
	  }catch(...){
	    val = new Vec2(10,10);
	  }
	}else{
	  val = 0;
	}
// 	DebugStream << "value = "<< val<<std::endl;

       lua_pop(L, 1);
     }

    return 1;

}



int LuaScript::addpathpoint(lua_State* L)
{
  int argNos = lua_gettop(L);
  int pathIndex = -3;
  int pathId = 0;
  float posxIndex = -2;
  Vec2 pos;
  //if a vec2 has been passed
  try{
      if(argNos == 2){
	pathIndex +=1;
	pos = *(lvec2_check(L,-1));
      }else if(argNos ==3 ){
	  float x = SafeGetNumberLua(L,posxIndex);
	  float y = SafeGetNumberLua(L,posxIndex+1);
	  pos = Vec2(x,y);

      }else{
	char buffer[255];
	sprintf(buffer,"Incorrect number of arguments provided for addPoint function, \nTry \n\t int (path id), int (x position), int (y position) \n\t or \n\t int (path id), Vec2 (point)");
	throw LuaScript::Exception(buffer);

	return lua_error(L);
      }
      pathId = SafeGetNumberLua(L,pathIndex);
  }catch(LuaScript::Exception& e){
    throw e;
  }catch(...){
      	char buffer[255];
	sprintf(buffer,"Incorrect arguments provided for addPoint function, \nTry \n\t int (path id), int (x position), int (y position) \n\t or \n\t int (path id), Vec2 (point)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
  }



  int noOfPaths = PathManager::get_instance().get_paths_size() ;
  //if path doesnt exist create it
  if(pathId > noOfPaths-1){
      PathPtr newPath = PathPtr(new Path);
      newPath->add_point(pos);
      PathManager::get_instance().add_path(newPath);
  }else{
      PathManager::get_instance().get_path(pathId)->add_point(pos);
  }
  return 1;

}


int LuaScript::removepathpoint(lua_State* L)
{
  int pathIndex,pointIndex;

  try{
      pathIndex = SafeGetNumberLua(L,-2);
      pointIndex = SafeGetNumberLua(L,-1);
      PathManager::get_instance().get_path(pathIndex)->remove_point(pointIndex);
      return 1;
  }catch(...){
      char buffer[255];
      sprintf(buffer,"Incorrect arguments provided for removePoint function (%d,%d), \nTry \n\t int (path index), int (point index) ",pathIndex,pointIndex);
      throw LuaScript::Exception(buffer);
      return lua_error(L);
  }
}


int LuaScript::movepathpoint(lua_State* L)
{
  int noArgs = lua_gettop(L);
  int pathIndex;
  int pointIndex;
  float x,y;
  Vec2 pos;

  try{

    if(noArgs == 3){
      pathIndex = SafeGetNumberLua(L,-3);
      pointIndex = SafeGetNumberLua(L,-2);
      pos = *(lvec2_check(L,-1));
    }else if(noArgs == 4){
      pathIndex = SafeGetNumberLua(L,-4);
      pointIndex = SafeGetNumberLua(L,-3);
      x = SafeGetNumberLua(L,-2);
      y = SafeGetNumberLua(L,-1);
      pos = Vec2(x,y);
    }else{
      char buffer[255];
      sprintf(buffer,"Incorrect number of arguments provided for movePoint function (%d), \nTry \n\t int (path index), int (point index) , int (x position), int ( y position)  \n\t or \n\t int (path index), int (point index) , Vec2 (point position)",noArgs);
      throw LuaScript::Exception(buffer);
      return lua_error(L);
    }

  }catch(LuaScript::Exception& e){
    throw e;
  }catch(...){
      char buffer[255];
      sprintf(buffer,"Incorrect arguments provided for movePoint function (%d), \nTry \n\t int (path index), int (point index) , int (x position), int ( y position)  \n\t or \n\t int (path index), int (point index) , Vec2 (point position)",noArgs);
      throw LuaScript::Exception(buffer);
      return lua_error(L);
  }


  int noPaths = PathManager::get_instance().get_paths_size() -1;
  if(pathIndex > noPaths){
    char buffer[255];
    sprintf(buffer,"Cannot move point as path %d does not exist",pathIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }
  PathManager::get_instance().get_path(pathIndex)->set_point(pointIndex,pos);


  return 1;

}

int LuaScript::add_path(lua_State* L)
{
  int argNos = lua_gettop(L);
  float posxIndex = -2;
    Vec2 pos;

    try{
	//if a vec2 has been passed
	if(argNos == 1){
	  pos = *(lvec2_check(L,-1));
	}else if(argNos ==2 ){
	    float x = SafeGetNumberLua(L,posxIndex);
	    float y = SafeGetNumberLua(L,posxIndex+1);
	    pos = Vec2(x,y);
	}else{
	  throw LuaScript::Exception("Incorrect number of arguments for addPath, you have to provide a point.\nTry \n\t int (x position), int ( y position)  \n\t or \n\t Vec2 (point position) ");
	  return lua_error(L);
	}


    }catch(LuaScript::Exception& e){
      throw e;
    }catch(LuaScript::Vec2Exception& e){
      throw LuaScript::Exception("Could not obtain a valid Vec2 for addPath");
      return lua_error(L);

    }catch(...){
        throw LuaScript::Exception("Incorrect arguments for addPath, you have to provide a point.\nTry \n\t int (x position), int ( y position)  \n\t or \n\t Vec2 (point position) ");
	return lua_error(L);
    }

    PathPtr p = PathPtr(new Path);
    p->add_point(pos);
    int id = PathManager::get_instance().add_path(p);
    lua_pushnumber(L,id);
    return 1;

}
int LuaScript::remove_path(lua_State* L)
{
  int index;
  try{
    index = SafeGetNumberLua(L,-1);
    PathManager::get_instance().remove_path(index);
    return 1;
  }catch(...){
    throw LuaScript::Exception("Incorrect number of arguments for removePath. You need to provide a path index. \nTry \n\t int (path index)");
  }
}

int LuaScript::movepath(lua_State* L)
{
  int argNos = lua_gettop(L);
  float x,y;
  Vec2 pos;
  int pathIndex;

  try{

  if(argNos == 2){
    pathIndex = SafeGetNumberLua(L,-2);
    pos = * (lvec2_check(L,-1));
  }else if(argNos == 3){
    pathIndex = SafeGetNumberLua(L,-3);
    x = SafeGetNumberLua(L,-2);
    y = SafeGetNumberLua(L,-1);
    pos = Vec2(x,y);
  }else{
      throw LuaScript::Exception("Incorrect number of arguments for movePath.\nTry \n\t int (path index) , int (x delta),int (y delta)\n\t or \n\t int (path index) , Vec2 (delta) ");

      return lua_error(L);
  }
  }catch(LuaScript::Exception& e){
    throw e;
    return lua_error(L);
  }catch(LuaScript::Vec2Exception& e ){
    throw LuaScript::Exception("Incorrect Vec2 provided for movePath.");
    return lua_error(L);

  }catch(...){
      throw LuaScript::Exception("Incorrect arguments for movePath.\nTry \n\t int (path index) , int (x delta),int (y delta)\n\t or \n\t int (path index) , Vec2 (delta) ");
      return lua_error(L);
  }

  if(pathIndex > PathManager::get_instance().get_paths_size()){
    char buffer[255];
    sprintf(buffer,"Cannot movepath of index %d  as it does not exist",pathIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }
  PathManager::get_instance().get_path(pathIndex)->move(pos);
  return 1;

}





int LuaScript::move(lua_State* L)
{
  int argNos = lua_gettop(L);


  int id;// = lua_tointeger(L,-1);
  float posx;// = lua_tonumber(L,-3);
  float posy;// = lua_tonumber(L,-2);
  Vec2 pos;
  try{
    if(argNos == 3){
      id = SafeGetNumberLua(L,-1);
      posx = SafeGetNumberLua(L,-3);
      posy = SafeGetNumberLua(L,-2);
      pos = Vec2(posx,posy);
    }else if(argNos ==2){
      id = SafeGetNumberLua(L,-1);
      pos = *(lvec2_check(L,-2));
    }else{
	       char buffer[1024];
	       sprintf(buffer,"Incorrect number of arguments (%d) for move/moveID \nTry \n\t move( int xDelta,int yDelta,int agentID)\n\t or \n\t moveID(Vec2 delta,int agentID)",argNos);
	       throw LuaScript::Exception(buffer);
         return lua_error(L);
    }

  }catch(LuaScript::Exception& e){
    throw e;
  }catch(LuaScript::Vec2Exception& e){
    char buffer[1024];
    sprintf(buffer,"unable to obtain Vec2  in move/moveID \nTry \n\t move(int xDelta,int yDelta)\n\t or \n\t move(Vec2 delta)\n\t or \n\t moveID(int agentID, int xDelta,int yDelta)\n\t or \n\t moveID(int agentID,Vec2 delta)");
    throw LuaScript::Exception(buffer);

  }catch(...){
    char buffer[1024];
    sprintf(buffer,"Incorrect arguments for move/moveID \nTry \n\t move(int xDelta,int yDelta)\n\t or \n\t move(Vec2 delta)\n\t or \n\t moveID(int agentID, int xDelta,int yDelta)\n\t or \n\t moveID(int agentID,Vec2 delta)");
    throw LuaScript::Exception(buffer);

    return lua_error(L);
  }
  TreeNode::Ptr self =  TreeNode::get_treenode_by_id(id).lock();

  Vec2 selfPos = self->get_pos();
  self->set_pos(selfPos + pos);
  return lua_yield(L,0);
}




void lua_pushPoint(lua_State* L, const Vec2& p){
	lua_newtable(L);

	lua_pushstring(L, "x");
	lua_pushnumber(L, p.m_x);
	lua_settable(L,-3);

	lua_pushstring(L,"y");
	lua_pushnumber(L, p.m_y);
	lua_settable(L,-3);
}

int LuaScript::changemotionmodel(lua_State* L)
{

    TreeNode::Ptr tptr;
    int id = lua_tointeger(L,-1);
    try{
	tptr=  TreeNode::get_treenode_by_id( id  ).lock();


	std::string type(lua_tostring(L,-2));
	if(type == "Linear" || type == "linear"){
	    tptr->set_motion_model(MotionModel::MT_LINEAR);
	} else if(type == "Physics" || type == "physics"){
	    tptr->set_motion_model(MotionModel::MT_PHYSICS);
	}else if (type == "Static" || type == "static"){
	  tptr->set_motion_model(MotionModel::MT_STATIC);
	}else if(type == "Path" || type == "path"){
	    tptr->set_motion_model(MotionModel::MT_PATH);
	}else{
	    char buffer[255];
	    sprintf(buffer,"Could not change the motion model to %s as that motion model doesn't exist",type.c_str());
	    throw LuaScript::Exception(buffer);


	    tptr->set_motion_model(MotionModel::MT_STATIC);
	}

    }catch(TreeNode::IdException& e){
       char buffer[255];
	sprintf(buffer,"Could not change the motion model of the Agent with ID %d as it does not exist",id);
	throw LuaScript::Exception(buffer);


    }

}




int LuaScript::getposition(lua_State* L){
	int id  =  SafeGetNumberLua(L,-1);
	try{


	  TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
  // 	lua_pushPoint(L, activeAgent->get_pos() );
	  lua_pushVec2(L,activeAgent->get_pos());
	  return 1;

	}catch(... ){
	  char buffer[255];
	  sprintf(buffer,"Could not get the position for Agent with ID %i as that Agent does not exist",id);
	  throw LuaScript::Exception(buffer);

	  return lua_error(L);
	}
}

int LuaScript::setposition(lua_State* L){

	int argNos = lua_gettop(L);
	int id;
	try{

	id  = lua_tointeger(L,-1);

	TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();

	if(argNos ==3){

	    int x = (int)SafeGetNumberLua(L,-3);
	    int y = (int)SafeGetNumberLua(L,-2);
	    lua_pop(L,2);
	    activeAgent->set_pos(Vec2(x,y));
	      //DebugStream << "c++ x = "<< x << " y = " << y<<std::endl;
	    //this could be lua yield
	    return 1;
	}else if(argNos == 2){
	    Vec2 pos = *(lvec2_check(L,-2));
	    activeAgent->set_pos(pos);
	    return 1;
	}else{
	    char buffer[1024];
	    sprintf(buffer,"Incorrect number of arguments to setPosition/setPositionID (%d) \nTry \n\t setPosition(int x,int y)\n\t or \n\t setPosition(Vec2 position)\n\t or \n\t setPositionID(int x,int y,int agentID)\n\t or \n\t setPositionID(Vec2 position, int agentID)",argNos);
	    throw LuaScript::Exception(buffer);
	    return lua_error(L);
	}

	}catch(TreeNode::IdException& e){
	  char buffer[255];
	  sprintf(buffer,"Could not set the position for Agent with ID %i as that Agent does not exist",id);
	  throw LuaScript::Exception(buffer);
	}catch(LuaScript::Exception& e){
	  throw e;
	}catch(LuaScript::Vec2Exception& e){
    char buffer[255];
    sprintf(buffer,"Could obtain Vec2 in setPosition/setPostionID");
    throw LuaScript::Exception(buffer);

  }catch(...){
	    char buffer[1024];
	    sprintf(buffer,"Incorrect arguments to setPosition/setPositionID (%d) \nTry \n\t setPosition(int x,int y)\n\t or \n\t setPosition(Vec2 position)\n\t or \n\t setPositionID(int x,int y,int agentID)\n\t or \n\t setPositionID(Vec2 position,int agentID)",argNos);
	    throw LuaScript::Exception(buffer);
	    return lua_error(L);
	}
}


int LuaScript::has_children(lua_State* L)
{
   int id  = lua_tointeger(L,-1);

  try{
    TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
    lua_pushboolean(L,activeAgent->has_children());
    return 1;
  }catch(TreeNode::IdException){
     char buffer[255];
     sprintf(buffer,"Could complete hasChildren function for Agent with ID %i as that Agent does not exist",id);
	  throw LuaScript::Exception(buffer);
	  return lua_error(L);
  }
}

int LuaScript::number_of_children(lua_State* L)
{
  int id  = lua_tointeger(L,-1);

  try{
    TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
    lua_pushinteger(L,activeAgent->number_of_children());
    return 1;

  }catch(TreeNode::IdException){
     char buffer[255];
     sprintf(buffer,"Could not return the number of children for Agent with ID %i as that Agent does not exist",id);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }
}





template<lua_CFunction F>
int self_call(lua_State* L){
//   DebugStream << "Self call " << int(F) << std::endl;
  lua_pushinteger(L, WorldEngine::get_instance().get_current_agent_id() );
  return F(L);
}


int LuaScript::sc_new(lua_State* L)
{
    int id,origId;

    try{
	id = SafeGetNumberLua(L,-1);
	origId = id;
	id +=SC_OFFSET;

    //     DebugStream << "SC spawn id = "<< id <<std::endl;
	std::string name = SafeGetStringLua(L,-2);
    //     DebugStream << "SC synth name = " << name<<std::endl;
	lo_address t = WorldEngine::get_instance().get_lo_address();

	lo_send(t,"/s_new","siii",name.c_str(),id,1,1);
	return 1;
    }catch(LuaScript::NumberException& e){
	  char buffer[255];
	  sprintf(buffer,"In raw scNew, could not obtain ID");
	  throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(LuaScript::StringException& e){
        char buffer[255];
	sprintf(buffer,"No Supercollider synth name provided in raw scNew");
	throw LuaScript::Exception(buffer);

	return lua_error(L);
    }
}
int LuaScript::sc_set(lua_State* L)
{
    try{

	int id = SafeGetNumberLua(L,-1);
	id +=SC_OFFSET;
	std::string name = SafeGetStringLua(L,-3);
	float val = (float)SafeGetNumberLua(L,-2);
	lo_address t = WorldEngine::get_instance().get_lo_address();
	lo_send(t,"/n_set","isf",id,name.c_str(),val);
	return 1;
    }catch(LuaScript::NumberException& e){
	  char buffer[255];
	  sprintf(buffer,"In raw scSet, could not obtain ID or the value.\n\t Try \n\t scSet(\"frequency\",440)");
	  throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(LuaScript::StringException& e){
        char buffer[255];
	sprintf(buffer,"No Supercollider synth argument provided in raw scSet");
	throw LuaScript::Exception(buffer);

	return lua_error(L);
    }
}

int LuaScript::sc_free(lua_State* L)
{
    try{
     int id = SafeGetNumberLua(L,-1);
     id +=SC_OFFSET;
     lo_address t = WorldEngine::get_instance().get_lo_address();
     lo_send(t,"/n_free","i",id);
     return 1;
    }catch(LuaScript::NumberException& e){
	char buffer[255];
	sprintf(buffer,"In raw scFree, could not obtain ID of the node to free from the Agent");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }
}
int LuaScript::getlifetime(lua_State* L)
{

      int id;
      try{
	id =  SafeGetNumberLua(L,-1);


	TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
	lua_pushnumber(L,activeAgent->get_lifeTime());
	return 1;

      }catch(LuaScript::NumberException& e){
	  char buffer[255];
	  sprintf(buffer,"In getLifetime/getLifetimeID,could not obtain ID of the Agent.\n\t Try \n\t getLifetime() \n\t or \n\t getLifetimeID(int agentID)");
	  throw LuaScript::Exception(buffer);
      }catch(TreeNode::IdException&e){
	  char buffer[255];
	  sprintf(buffer,"In getLifetime/getLifetimeID,could not Agent with ID %i as it does not exist.\n\t Try \n\t getLifetime() \n\t or \n\t getLifetimeID(int agentID)",id);
	  throw LuaScript::Exception(buffer);

      }
}

int LuaScript::addproperty(lua_State* L)
{

    int totalArgs = lua_gettop(L);
    std::string name;
    int id;
    try{


	  if(totalArgs ==3){

	    id  =  SafeGetNumberLua(L,-1);
	    TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
// 	    if(!activeAgent)throw LuaScript::Exception("Could not retrieve agent by id");

	    name = SafeGetStringLua(L,-3);
	    if(name.empty())throw LuaScript::StringException("In Cannot retrieve string");

	    PropertyValue val;
	    if( lua_isnumber(L,-2)){
	      val = float(SafeGetNumberLua(L,-2));
	    }else if (lua_isstring(L,-2)){
	      val = std::string(SafeGetStringLua(L,-2));
	    }else if(lua_isuserdata(L,-2)){
	      val = *lvec2_check(L,-2);
	      DebugStream << "adding userdata "<<val <<std::endl;

	    }else{
	      char buffer[255];
	      sprintf(buffer,"Unable to identify the type of the new added property %s ",name.c_str());
	      throw LuaScript::Exception(buffer);
	    }
	//     DebugStream << id << " "<< name <<  " "<< val<< std::endl;
	    activeAgent->add_property(name ,Property(val,Property::PF_READ | Property::PF_WRITE | Property::PF_SEND));
	    lua_yield(L,0);
	    return 0;

	  }else if(totalArgs ==2){
	    int id  =  SafeGetNumberLua(L,-1);
	    TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
// 	    if(!activeAgent)throw LuaScript::Exception("Could not retrieve agent by id");
	    if(lua_istable(L,-2)){
	     add_table_property(L,-2,activeAgent);
	     lua_yield(L,0);
	     return 0;
	     std::cout << "adding table"<<std::endl;
	    }else{
	      char buffer[1024];
	      sprintf(buffer,"Incorrect arguments for addP/addPID (%s). \n\t Try \n\t addP(string propertyName,int value) \n\t addP(string propertyName,float value) \n\t addP(string propertyName,string value) \n\t addP(string propertyName,Vec2 value) \n\t addP(table propertyTable) \n\t or \n\taddPID(string propertyName,int value,int agentID) \n\t addPID(string propertyName,float value,int agentID) \n\t addPID(string propertyName,string value,int agentID) \n\t addPID(string propertyName,Vec2 value,int agentID) \n\t addPID(table propertyTable,int agentID)  ",name.c_str());
	      throw LuaScript::Exception(buffer);
	    }
	  }else{
	      char buffer[1024];
	      sprintf(buffer,"Incorrect number of arguments for addP/addPID (%s). \n\t Try \n\t addP(string propertyName,int value) \n\t addP(string propertyName,float value) \n\t addP(string propertyName,string value) \n\t addP(string propertyName,Vec2 value) \n\t addP(table propertyTable) \n\t or \n\taddPID(string propertyName,int value,int agentID) \n\t addPID(string propertyName,float value,int agentID) \n\t addPID(string propertyName,string value,int agentID) \n\t addPID(string propertyName,Vec2 value,int agentID) \n\t addPID(table propertyTable,int agentID)  ",name.c_str());
	      throw LuaScript::Exception(buffer);

	  }
    }catch(LuaScript::NumberException& e){

	char buffer[255];
	sprintf(buffer,"Unable to retrieve number when adding property (%s)",name.c_str());
	throw LuaScript::Exception(buffer);

    }catch(LuaScript::StringException& e){
	char buffer[255];
	sprintf(buffer,"Unable to retrieve property name for addP (got %s)",name.c_str());
	throw LuaScript::Exception(buffer);

    }catch(TreeNode::IdException& e){
	char buffer[255];
	sprintf(buffer,"Unable to add property to Agent with ID %i as it does not exist",id);
	throw LuaScript::Exception(buffer);
    }catch(LuaScript::Exception& e){
      throw e;
    }catch(LuaScript::Vec2Exception& e){
	char buffer[255];
	sprintf(buffer,"Unable to parse Vec2 name for property ( %s) in addP",name.c_str());
	throw LuaScript::Exception(buffer);
    }
    catch(...){
      char buffer[1024];
      sprintf(buffer,"Incorrect arguments for addP/addPID (%s). \n\t Try \n\t addP(string propertyName,int value) \n\t addP(string propertyName,float value) \n\t addP(string propertyName,string value) \n\t addP(string propertyName,Vec2 value) \n\t addP(table propertyTable) \n\t or \n\taddPID(string propertyName,int value,int agentID) \n\t addPID(string propertyName,float value,int agentID) \n\t addPID(string propertyName,string value,int agentID) \n\t addPID(string propertyName,Vec2 value,int agentID) \n\t addPID(table propertyTable,int agentID)  ",name.c_str());
      throw LuaScript::Exception(buffer);
    }

}

///////////////////////////////////////////got here with updating messages and vec2 safety functions


int LuaScript::setproperty(lua_State* L)
{
    int totalArgs = lua_gettop(L);
    int id;
    std::string name;
    try{
	  if(totalArgs ==3){
	      id  =  SafeGetNumberLua(L,-1);
	      TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
	  //     DebugStream << "set id "<<id << std::endl;
// 	      if(!activeAgent)throw LuaScript::Exception("Could not retrieve agent by id");

	      name = lua_tostring(L,-3);
	      if(name.empty())throw LuaScript::StringException("No string id for property provided");

	      PropertyValue val;
	      if( lua_isnumber(L,-2)){
		val = float(SafeGetNumberLua(L,-2));
	      }else if (lua_isstring(L,-2)){
		val = std::string(SafeGetStringLua(L,-2));
	      }else if(lua_isuserdata(L,-2)){
		val = *lvec2_check(L,-2);
	      }else{
		char buffer[255];
		sprintf(buffer,"Unable to identify the type of the new added property %s ",name.c_str());
		throw LuaScript::Exception(buffer);
	      }
	  //     DebugStream << id << " "<< name <<  " "<< val<< std::endl;
	      activeAgent->set_property(name,val);
	      return 0;
	  }else if(totalArgs ==2){
	    int id  =  SafeGetNumberLua(L,-1);
	    TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
// 	    if(!activeAgent)throw LuaScript::Exception("Could not retrieve agent by id");
	    if(lua_istable(L,-2)){
	      add_table_property(L,-2,activeAgent);
	    }else{
	      char buffer[1024];
	      sprintf(buffer,"Incorrect arguments for setP/setPID (%s). \n\t Try \n\t setP(string propertyName,int value) \n\t setP(string propertyName,float value) \n\t setP(string propertyName,string value) \n\t setP(string propertyName,Vec2 value) \n\t setP(table propertyTable) \n\t or \n\tsetPID(string propertyName,int value,int agentID) \n\t setPID(string propertyName,float value,int agentID) \n\t setPID(string propertyName,string value,int agentID) \n\t setPID(string propertyName,Vec2 value,int agentID) \n\t setPID(table propertyTable,int agentID)  ",name.c_str());
	      throw LuaScript::Exception(buffer);
	    }
	  }else{
	      char buffer[1024];
	      sprintf(buffer,"Incorrect number of arguments for setP/setPID (%s). \n\t Try \n\t setP(string propertyName,int value) \n\t setP(string propertyName,float value) \n\t setP(string propertyName,string value) \n\t setP(string propertyName,Vec2 value) \n\t setP(table propertyTable) \n\t or \n\tsetPID(string propertyName,int value,int agentID) \n\t setPID(string propertyName,float value,int agentID) \n\t setPID(string propertyName,string value,int agentID) \n\t setPID(string propertyName,Vec2 value,int agentID) \n\t setPID(table propertyTable,int agentID)  ",name.c_str());
	      throw LuaScript::Exception(buffer);
	  }

    }catch(LuaScript::NumberException& e){

	char buffer[255];
	sprintf(buffer,"Unable to retrieve number when settting the property %s",name.c_str());
	throw LuaScript::Exception(buffer);

    }catch(LuaScript::StringException& e){
	     char buffer[255];
	      sprintf(buffer,"Unable to retrieve the name of the property to set (got %s)",name.c_str());
	       throw LuaScript::Exception(buffer);

    }catch(TreeNode::IdException& e){
	char buffer[255];
	sprintf(buffer,"Unable to set property of Agent with ID %i as it does not exist",id);
	throw LuaScript::Exception(buffer);
    }catch(LuaScript::Exception& e){
      throw e;
    }catch(LuaScript::Vec2Exception& e){
      char buffer[255];
      sprintf(buffer,"Unable to retrieve the Vec2 in setP/setPID (%s)",name.c_str());
      throw LuaScript::Exception(buffer);
    }catch(...){
      char buffer[1024];
      sprintf(buffer,"Incorrect number of arguments for setP/setPID (%s). \n\t Try \n\t setP(string propertyName,int value) \n\t setP(string propertyName,float value) \n\t setP(string propertyName,string value) \n\t setP(string propertyName,Vec2 value) \n\t setP(table propertyTable) \n\t or \n\tsetPID(string propertyName,int value,int agentID) \n\t setPID(string propertyName,float value,int agentID) \n\t setPID(string propertyName,string value,int agentID) \n\t setPID(string propertyName,Vec2 value,int agentID) \n\t setPID(table propertyTable,int agentID)  ",name.c_str());
      throw LuaScript::Exception(buffer);
    }

}

int LuaScript::hasProperty(lua_State* L)
{
  std::string propName;
  int id;
  int argNos = lua_gettop(L);
  try{
    if(argNos == 2){

      propName = SafeGetStringLua(L,-2);
      id  =  SafeGetNumberLua(L,-1);
      TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
    // //   if(!activeAgent){
    // //       DebugStream << "could not get agent"<<std::endl;
    // //       throw LuaScript::Exception("Could not retrieve agent by id");
    //   }

      bool exists = activeAgent->property_exists(propName);
      lua_pushboolean(L,exists);
      return 1;
    }else{
      char buffer[255];
      sprintf(buffer,"Incorrect arguments for hasP/hasPID \n\t Try \n\t hasP(string propertyName) \n\t hasPID(string propertyName,int agendID) ",id,propName.c_str());
      throw LuaScript::Exception(buffer);
      return lua_error(L);
    }
  }catch(TreeNode::IdException& e){
    char buffer[255];
    sprintf(buffer,"Unable to obtain whether Agent with ID %i has the property %s as the Agent does not exist",id,propName.c_str());
    throw LuaScript::Exception(buffer);

    return lua_error(L);
  }catch(LuaScript::StringException& e){
    char buffer[255];
    sprintf(buffer,"Unable to obtain the name of the property to check in hasP/hasPID");
    throw LuaScript::Exception(buffer);
  }
}


////////

int LuaScript::getproperty(lua_State* L)
{
    int id;
    int argNos = lua_gettop(L);
    try{
      id =  SafeGetNumberLua(L,-1);

      if(argNos != 2){
	  char buffer[255];
	  sprintf(buffer,"Incorrect number of arguments for getP/getPID. \n\t Try \n\t getP(string propertyName)\n\t getPID(string propertyName,int agentID)");
	  throw LuaScript::Exception(buffer);
	}


	TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();

      /* if(!activeAgent){
	  DebugStream << "could not get agent"<<std::endl;
	  throw LuaScript::Exception("Could not retrieve agent by id");
	}
      */ std::string name = SafeGetStringLua(L,-2);
    //     DebugStream << "Property name "<< name<<std::endl;
	if(name.empty())throw LuaScript::Exception("No property name provided in getP/getPID");
	PropertyValue val = activeAgent->get_property(name).get_value();
	if(val.type() == typeid(int) || val.type() == typeid(float) ) {
	  lua_pushnumber(L,boost::get<float>(val));
    //       DebugStream << "Retrieved float "<<std::endl;
	}
	else if(val.type() == typeid(std::string)   ) {
	  lua_pushstring(L,boost::get<std::string>(val).c_str());
	}
	else if(val.type() == typeid(Vec2) ){
	  lua_pushVec2(L,boost::get<Vec2>(val) );
	}
	else {
	  DebugStream << "Unknown type error in get property"<<std::endl;
	  lua_pushstring(L,"ERROR UNKOWN TYPE");
	}
	return 1;
    }catch(TreeNode::IdException& e){
	char buffer[255];
	sprintf(buffer,"Unable to get property from Agent with ID %i it does not exist",id);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(LuaScript::StringException& e){
	char buffer[255];
	sprintf(buffer,"Unable to get the name of the property you wish to obtain in getP/getPID ",id);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }

}




int LuaScript::getnearest(lua_State* L)
{
  int argNos = lua_gettop(L);
  const Vec2* pos;
  int agentID;
  try{

      if(argNos == 1){
	if(lua_isnumber(L,-1)){
	   agentID = SafeGetNumberLua(L,-1);
	   TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(agentID).lock();
	   pos = &activeAgent->get_pos();
	}else{
	   pos = lvec2_check(L,-1);
	}
	
      }else if(argNos ==3){
	   int x = SafeGetNumberLua(L,-3);
	   int y = SafeGetNumberLua(L,-2);
	   pos = new Vec2(x,y);
      }else if(argNos ==2){
	   pos = lvec2_check(L,-2);

      }else{
	char buffer[255];
	sprintf(buffer,"Incorrect arguments for getNearest/getNearestID. Try \n\t getNearest() \n\t getNearest(Vec2 position) \n\t getNearest(int xPosition, int yPosition) \n\t getNearestID(int agentID))");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
      }

      int id = find_nearest_node(*pos);
      lua_pushinteger(L,id);
      return 1;

  }catch(TreeNode::IdException& e){
    	char buffer[255];
	     sprintf(buffer,"Unable to getNearest to Agent with ID %i as that Agent does not exist",agentID);
	     throw LuaScript::Exception(buffer);
	      return lua_error(L);
  }catch(LuaScript::NumberException& e){
      char buffer[255];
      sprintf(buffer,"Unable to obtain the coordinates to get nearest Agent. Try \n\t getNearest() \n\t getNearest(Vec2 position) \n\t getNearest(int xPosition, int yPosition) \n\t getNearestID(int agentID))");
      throw LuaScript::Exception(buffer);
      return lua_error(L);
  }catch(LuaScript::Vec2Exception& e){
    throw LuaScript::Exception("Unable to obtain Vec2 for getNearest");
  }catch(...){
    char buffer[255];
    sprintf(buffer,"Unable to obtain the coordinates to get nearest Agent. Try \n\t getNearest() \n\t getNearest(Vec2 position) \n\t getNearest(int xPosition, int yPosition) \n\t getNearestID(int agentID))");
    throw LuaScript::Exception(buffer);

  }
}






int LuaScript::drawrgbarelative(lua_State* L)
{
  int totalArgs = lua_gettop(L);
  DebugStream << "draw arg no = " << totalArgs<<std::endl;
  int bufferIndex;
  Vec2* pos;
  float r;
  float g;
  float b;
  RGBA* col;
  int id;

  try{

      if(totalArgs ==4){

	bufferIndex = SafeGetNumberLua(L,-4);
	pos = lvec2_check(L,-3);
	col = lrgba_check(L,-2);
	id = SafeGetNumberLua(L,-1);
	DebugStream << col->R()<<std::endl;
      }else if(totalArgs == 6){

	bufferIndex = SafeGetNumberLua(L,-6);
	pos = lvec2_check(L,-5);
	r = SafeGetNumberLua(L,-4);
	g = SafeGetNumberLua(L,-3);
	b = SafeGetNumberLua(L,-2);
	id = SafeGetNumberLua(L,-1);
	col = new RGBA(r,g,b,150);
      }else{

	  char buffer[1024];
	  sprintf(buffer,"Incorrect number of arguments to drawRelative/drawRelativeID. \n\t Try \n\t drawRelative(int imageIndex, Vec2 relativePosition, RGBA colourData)\n\tdrawRelative(int imageIndex, Vec2 relativePosition, int red,int green,blue)\n\t drawRelativeID(int imageIndex, Vec2 relativePosition, RGBA colourData, int agentID)\n\tdrawRelativeID(int imageIndex, Vec2 relativePosition, int red,int green,blue,int agentID)");
	  throw LuaScript::Exception(buffer);
	  return lua_error(L);


      }

    TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();


      if(WorldEngine::get_instance().get_surface(bufferIndex)){
	  Vec2  currentPos = activeAgent->get_pos();
	  WorldEngine::get_instance().get_surface(bufferIndex)->set_rgba_at(*pos + currentPos,*col);
      }

  }catch(TreeNode::IdException& e){
    	char buffer[255];
	sprintf(buffer,"Unable to drawRelative/drawRelativeID to Agent with ID %i as that Agent does not exist",id);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
  }catch(std::out_of_range& e){
    //cant find surface....
	char buffer[255];
	sprintf(buffer,"Unable to drawRelative/drawRelativeID to Buffer with ID %i as that it does not exist",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
}catch(LuaScript::Vec2Exception& e){
  char buffer[255];
  sprintf(buffer,"Unable to obtain Vec2 in drawRelative/drawRelativeID ",bufferIndex);
  throw LuaScript::Exception(buffer);
  return lua_error(L);
}catch(LuaScript::RGBAException& e){
  throw LuaScript::Exception("Could not obtain RGBA in drawRelative/drawRelativeID ");
}catch(...){
	char buffer[1024];
	sprintf(buffer,"Incorrect arguments for drawRelative/drawRelativeID. \n\t Try \n\t drawRelative(int imageIndex, Vec2 relativePosition, RGBA colourData)\n\tdrawRelative(int imageIndex, Vec2 relativePosition, int red,int green,blue)\n\t drawRelativeID(int imageIndex, Vec2 relativePosition, RGBA colourData, int agentID)\n\tdrawRelativeID(int imageIndex, Vec2 relativePosition, int red,int green,blue,int agentID)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);

  }

  //   delete col;
  return 0;
}


int LuaScript::draw_line(lua_State* L)
{
  int totalArgs = lua_gettop(L);
  int bufferIndex;
  Vec2* posA;
  Vec2* posB;
  float r;
  float g;
  float b;
  RGBA* col;
  try{
	    if(totalArgs == 6){
	    //order
	      bufferIndex = SafeGetNumberLua(L,-6);
	      posA = lvec2_check(L,-5);
	      posB = lvec2_check(L,-4);
	      r = SafeGetNumberLua(L,-3);
	      g = SafeGetNumberLua(L,-2);
	      b = SafeGetNumberLua(L,-1);
	      col = new RGBA(r,g,b,150);

	    }else if(totalArgs == 4 ){
	      bufferIndex = SafeGetNumberLua(L,-4);
	      posA = lvec2_check(L,-3);
	      posB = lvec2_check(L,-2);
	      col = lrgba_check(L,-1);
	    }else{
	      char buffer[1024];
	      sprintf(buffer,"Incorrect number of arguments for drawLine (%i). \n\t Try \n\t drawLine(int imageIndex, Vec2 startPoint,Vec2 endPoint, RGBA colourData)\n\tdrawLine(int imageIndex, Vec2 startPoint,Vec2 endPoint, int red,int green, int blue)",totalArgs);
	      throw LuaScript::Exception(buffer);
	      return lua_error(L);

	    }
	    if(WorldEngine::get_instance().get_surface(bufferIndex)){

		      WorldEngine::get_instance().get_surface(bufferIndex)->draw_line(*posA,*posB,*col);
	    }
  }catch(std::out_of_range& e){
    //cant find surface....
	char buffer[255];
	sprintf(buffer,"Unable to drawLine to Buffer with ID %i as that it does not exist",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
  }catch(LuaScript::Exception& e){
    throw e;
  }catch(LuaScript::Vec2Exception& e){
      throw LuaScript::Exception("Unable to obtain Vec2 in drawLine");
  }catch(LuaScript::RGBAException& e){
    throw LuaScript::Exception("Could not obtain RGBA in drawLine");
  }catch(...){
	char buffer[1024];
	sprintf(buffer,"Incorrect arguments for drawLine. \n\t Try \n\t drawLine(int imageIndex, Vec2 startPoint,Vec2 endPoint, RGBA colourData)\n\tdrawLine(int imageIndex, Vec2 startPoint,Vec2 endPoint, int red,int green, int blue)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);

  }
//   delete col;
  return 0;
}


int LuaScript::draw_rect(lua_State* L)
{

  int totalArgs = lua_gettop(L);
  int bufferIndex;
  Vec2* posA;
  Vec2* posB;
  float r;
  float g;
  float b;
  RGBA* col;
  int thickNess;

  try{

      if(totalArgs == 7){


	  bufferIndex = SafeGetNumberLua(L,-7);
	  posA = lvec2_check(L,-6);
	  posB = lvec2_check(L,-5);
	  r = SafeGetNumberLua(L,-4);
	  g = SafeGetNumberLua(L,-3);
	  b = SafeGetNumberLua(L,-2);
	  thickNess = SafeGetNumberLua(L,-1);
	  col = new RGBA(r,g,b,150);
      }else if(totalArgs == 5){
	  bufferIndex = SafeGetNumberLua(L,-5);
	  posA = lvec2_check(L,-4);
	  posB = lvec2_check(L,-3);
	  col = lrgba_check(L,-2);
	  thickNess = SafeGetNumberLua(L,-1);
	 
      }else{
	char buffer[1024];
	sprintf(buffer,"Incorrect number of arguments for drawRect (%i). \n\t Try \n\t drawRect(int imageIndex, Vec2 bottomLeft,Vec2 topRight, RGBA colourData)\n\t drawRect(int imageIndex, Vec2 bottomLeft,Vec2 topRight, int red,int green, int blue)",totalArgs);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
      }

      if(WorldEngine::get_instance().get_surface(bufferIndex)){

	  WorldEngine::get_instance().get_surface(bufferIndex)->draw_rect(*posA,*posB,*col,thickNess);
      }
      return 0;

  }catch(std::out_of_range& e){
    //cant find surface....
	char buffer[255];
	sprintf(buffer,"Unable to drawRect to Buffer with ID %i as that it does not exist",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
  }catch(LuaScript::Exception& e){
    throw e;
  }catch(LuaScript::Vec2Exception& e){
    throw LuaScript::Exception("Unable to obtain Vec2 in drawRect");
  }catch(LuaScript::RGBAException& e){
    throw LuaScript::Exception("Could not obtain RGBA in drawRect");
  }catch(...){
	char buffer[1024];
	sprintf(buffer,"Incorrect arguments for drawRect. \n\t Try \n\t drawRect(int imageIndex, Vec2 bottomLeft,Vec2 topRight, RGBA colourData)\n\t drawRect(int imageIndex, Vec2 bottomLeft,Vec2 topRight, int red,int green, int blue)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);

  }
}

int LuaScript::draw_circle(lua_State* L)
{
  int totalArgs = lua_gettop(L);
  int bufferIndex;
  Vec2* posA;
  float radius;
  float r;
  float g;
  float b;
  RGBA* col;
  int thickNess;
  try{
      if(totalArgs == 7){

	bufferIndex = SafeGetNumberLua(L,-7);
	posA = lvec2_check(L,-6);
	radius = SafeGetNumberLua(L,-5);
	r = SafeGetNumberLua(L,-4);
	g = SafeGetNumberLua(L,-3);
	b = SafeGetNumberLua(L,-2);
	thickNess = SafeGetNumberLua(L,-1);
	col = new RGBA(r,g,b,150);
      }else if(totalArgs ==5){
	  bufferIndex = SafeGetNumberLua(L,-5);
	  posA = lvec2_check(L,-4);
	  radius = SafeGetNumberLua(L,-3);
	  col = lrgba_check(L,-2);
	  thickNess = SafeGetNumberLua(L,-1);

      }else{
	    char buffer[1024];
	    sprintf(buffer,"Incorrect number of arguments for drawCircle (%i). \n\t Try \n\t drawCircle(int imageIndex, Vec2 position,int radius, RGBA colourData, int lineThickness)\n\t drawCircle(int imageIndex, Vec2 position,int radius, int red,int green, int blue, int lineThickness)",totalArgs);
	    throw LuaScript::Exception(buffer);
	    return lua_error(L);
      }

      if(WorldEngine::get_instance().get_surface(bufferIndex)){

	WorldEngine::get_instance().get_surface(bufferIndex)->draw_circle(*posA,radius,  *col,thickNess);
      }
      return 0;
  }catch(std::out_of_range& e){
    //cant find surface....
	char buffer[255];
	sprintf(buffer,"Unable to drawCircle to Buffer with ID %i as that it does not exist",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
  }catch(LuaScript::Exception& e){
    throw e;
  }catch(LuaScript::Vec2Exception& e){
    throw LuaScript::Exception("Unable to obtain Vec2 in drawCircle");
  }catch(LuaScript::RGBAException& e){
    throw LuaScript::Exception("Could not obtain RGBA in drawCircle");
  }catch(...){
	char buffer[1024];
	sprintf(buffer,"Incorrect arguments for drawCircle. \n\t Try \n\t drawCircle(int imageIndex, Vec2 position,int radius, RGBA colourData, int lineThickness)\n\t drawCircle(int imageIndex, Vec2 position,int radius, int red,int green, int blue, int lineThickness)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);

  }
}

int LuaScript::get_rgba(lua_State* L)
{
  int totalArgs = lua_gettop(L);
  int bufferIndex;
  const Vec2* pos;
  RGBA col;
  int id;
  try{
      if(totalArgs == 2){

	bufferIndex = SafeGetNumberLua(L,-2);
	if(lua_isnumber(L,-1)){
	     id = SafeGetNumberLua(L,-1);
	     TreeNode::Ptr treenode = TreeNode::get_treenode_by_id(id).lock();
	     pos = &treenode->get_pos();
	}else{
	    pos = lvec2_check(L,-1);
	}

      }else if(totalArgs == 3){
		bufferIndex = SafeGetNumberLua(L,-3);
	 if(lua_isnumber(L,-2)){
	    throw "Error";
	    return lua_error(L);
	}else{
	    pos = lvec2_check(L,-2);
	}

      } else{
	char buffer[255];
	sprintf(buffer,"Incorrect number of arguments to getrgba/getrgbaID. \n\tTry \n\t getrgba(int imageIndex) \n\t getrgba(int imageIndex,Vec2 position) \n\t getrgbaID(int imageIndex,int agentID)",totalArgs);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
      }



      if(WorldEngine::get_instance().get_surface(bufferIndex)){
	  col = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(*pos);
	  return lua_pushRGBA(L,col);
      }

  }catch(std::out_of_range& e){
    //cant find surface....
	char buffer[255];
	sprintf(buffer,"Unable to getRGBA of Buffer with ID %i as that it does not exist",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
  }catch(LuaScript::Exception& e){
    throw e;
  }catch(TreeNode::IdException& e){
	char buffer[255];
	sprintf(buffer,"Unable to getRGBA/getRGBAID of Agent with ID %i as it does not exist",id);
	throw LuaScript::Exception(buffer);
  }catch(LuaScript::Vec2Exception& e){
    throw LuaScript::Exception("Unable to obtain Vec2 in getRGBA");
  } catch(...){
	char buffer[255];
	sprintf(buffer,"Incorrect arguments to getRGBA/getRGBAID. \n\tTry \n\t getRGBA(int imageIndex) \n\t getRGBA(int imageIndex,Vec2 position) \n\t getRGBAID(int imageIndex,int agentID)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
  }

}


int LuaScript::getredfromsurface(lua_State* L)
{
  int bufferIndex;
  int totalArgs = lua_gettop(L);

  try{
      if(totalArgs != 2)throw "error";

      bufferIndex = SafeGetNumberLua(L,-2);
      const Vec2* pos = lvec2_check(L,-1);
      float val(0);
      if(WorldEngine::get_instance().get_surface(bufferIndex)){
	  val = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(*pos).R();
      }else{
	val = 0;
      }
      lua_pushnumber(L,val);
      return 1;
  }catch(LuaScript::NumberException& e){
   // getRedFromPoint
    char buffer[255];
    sprintf(buffer,"Unable to obtain image index in getRedFromPoint. \n\tTry \n\t getRedFromPoint(int imageIndex,Vec2 point)");
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }catch(std::out_of_range& e){
   // getRedFromPoint
    char buffer[255];
    sprintf(buffer,"Unable to obtain image of index %i in getRedFromPoint as it does not exist. \n\tTry \n\t getRedFromPoint(int imageIndex,Vec2 point)",bufferIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }catch(LuaScript::Vec2Exception& e){
    throw LuaScript::Exception("Unable to obtain Vec2 in getRedFromPoint");
  } catch(...){
    char buffer[255];
    sprintf(buffer,"Unable to getRedFromPoint. \n\tTry \n\t getRedFromPoint(int imageIndex,Vec2 point)",bufferIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }
}

int LuaScript::getgreenfromsurface(lua_State* L)
{
  int bufferIndex ;
  int totalArgs = lua_gettop(L);

  try{
      if(totalArgs != 2)throw "error";
	bufferIndex= SafeGetNumberLua(L,-2);
	const Vec2* pos = lvec2_check(L,-1);
	float val(0);
	if(WorldEngine::get_instance().get_surface(bufferIndex)){
	  val = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(*pos).B();
	}else{
	  val = 0;
	}

	lua_pushnumber(L,val);

	return 1;

  }catch(LuaScript::NumberException& e){
   // getredfrompoint
    char buffer[255];
    sprintf(buffer,"Unable to obtain image index in getGreenFromPoint. \n\tTry \n\t getGreenFromPoint(int imageIndex,Vec2 point)");
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }catch(std::out_of_range& e){
   // getredfrompoint
    char buffer[255];
    sprintf(buffer,"Unable to obtain image of index %i in getGreenFromPoint as it does not exist. \n\tTry \n\t getGreenFromPoint(int imageIndex,Vec2 point)",bufferIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }catch(LuaScript::Vec2Exception& e){
    throw LuaScript::Exception("Unable to obtain Vec2 in getGreenFromPoint");
  }catch(...){
    char buffer[255];
    sprintf(buffer,"Unable to getGreenFromPoint. \n\tTry \n\t getGreenFromPoint(int imageIndex,Vec2 point)",bufferIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }
}
int LuaScript::getbluefromsurface(lua_State* L)
{

  int bufferIndex;
  int totalArgs = lua_gettop(L);

  try{
      if(totalArgs != 2)throw "error";
      bufferIndex = SafeGetNumberLua(L,-2);

      const Vec2* pos = lvec2_check(L,-1);
      float val(0);
      if(WorldEngine::get_instance().get_surface(bufferIndex)){
	val = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(*pos).G();

      }else{
	val = 0;
      }
      lua_pushnumber(L,val);
      return 1;
  }catch(LuaScript::NumberException& e){
   // getredfrompoint
    char buffer[255];
    sprintf(buffer,"Unable to obtain image index in getBlueFromPoint. \n\tTry \n\t getBlueFromPoint(int imageIndex,Vec2 point)");
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }catch(std::out_of_range& e){
   // getredfrompoint
    char buffer[255];
    sprintf(buffer,"Unable to obtain image of index %i in getBlueFromPoint as it does not exist. \n\tTry \n\t getBlueFromPoint(int imageIndex,Vec2 point)",bufferIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }catch(LuaScript::Vec2Exception& e){
    throw LuaScript::Exception("Unable to obtain Vec2 in getBlueFromPoint");
  }catch(...){
    char buffer[255];
    sprintf(buffer,"Unable to getBlueFromPoint. \n\tTry \n\t getBlueFromPoint(int imageIndex,Vec2 point)",bufferIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }
}

int LuaScript::gettotalfromsurface(lua_State* L)
{
  int bufferIndex;
  int totalArgs = lua_gettop(L);

  try{
      if(totalArgs != 2)throw "error";
      bufferIndex = SafeGetNumberLua(L,-2);
      const Vec2* pos = lvec2_check(L,-1);
      float val(0),r(0),g(0),b(0);
      if(WorldEngine::get_instance().get_surface(bufferIndex)){
	r = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(*pos).R();
	g = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(*pos).G();
	b = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(*pos).B();
	val = r+g+b;
      }else{
	val = 0;
      }
      lua_pushnumber(L,val);
      return 1;
  }catch(LuaScript::NumberException& e){
   // getredfrompoint
    char buffer[255];
    sprintf(buffer,"Unable to obtain image index in getTotalFromPoint. \n\tTry \n\t getTotalFromPoint(int imageIndex,Vec2 point)");
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }catch(std::out_of_range& e){
   // getredfrompoint
    char buffer[255];
    sprintf(buffer,"Unable to obtain image of index %i in getTotalFromPoint as it does not exist. \n\tTry \n\t getTotalFromPoint(int imageIndex,Vec2 point)",bufferIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }catch(LuaScript::Vec2Exception& e){
    throw LuaScript::Exception("Unable to obtain Vec2 in getTotalFromPoint");
  }catch(...){
    char buffer[255];
    sprintf(buffer,"Unable to getTotalFromPoint. \n\tTry \n\t getTotalFromPoint(int imageIndex,Vec2 point)",bufferIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);

  }
}

int LuaScript::getmeanfromsurface(lua_State* L)
{
  int bufferIndex;

  int totalArgs = lua_gettop(L);

  try{
      if(totalArgs != 2)throw "error";

      bufferIndex= SafeGetNumberLua(L,-2);
      const Vec2* pos = lvec2_check(L,-1);
      float val(0),r(0),g(0),b(0);
      if(WorldEngine::get_instance().get_surface(bufferIndex)){
	r = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(*pos).R();
	g = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(*pos).G();
	b = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(*pos).B();
	val = (r+g+b)/3.0f;
      }else{
	val = 0;
      }
      lua_pushnumber(L,val);
      return 1;
  }catch(LuaScript::NumberException& e){
   // getredfrompoint
    char buffer[255];
    sprintf(buffer,"Unable to obtain image index in getMeanFromPoint. \n\tTry \n\t getMeanFromPoint(int imageIndex,Vec2 point)");
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }catch(std::out_of_range& e){
   // getredfrompoint
    char buffer[255];
    sprintf(buffer,"Unable to obtain image of index %i in getMeanFromPoint as it does not exist. \n\tTry \n\t getMeanFromPoint(int imageIndex,Vec2 point)",bufferIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }catch(LuaScript::Vec2Exception& e){
    throw LuaScript::Exception("Unable to obtain Vec2 in getMeanFromPoint");
  }catch(...){
    char buffer[255];
    sprintf(buffer,"Unable to getMeanFromPoint . \n\tTry \n\t getMeanFromPoint(int imageIndex,Vec2 point)",bufferIndex);
    throw LuaScript::Exception(buffer);
    return lua_error(L);

  }
}








int LuaScript::getredID(lua_State* L)
{
    int id;
    int bufferIndex;
    int totalArgs = lua_gettop(L);

      try{
      if(totalArgs != 2)throw "error";
	id= SafeGetNumberLua(L,-1);

	TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();

	bufferIndex = SafeGetNumberLua(L,-2);

	float val(0);
	if(WorldEngine::get_instance().get_surface(bufferIndex)){
	  Vec2 pos =boost::get<Vec2>(activeAgent->get_property("pos").get_value());
	  val = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(pos).R();
	}else{
	  val = 0;
	}
	lua_pushnumber(L,val);
	return 1;
    }catch(LuaScript::NumberException& e){
	char buffer[255];
	sprintf(buffer,"Unable to obtain ID or image buffer in getR/getRID. \n\tTry \n\t getR(int imageIndex)\n\t getRID(int imageIndex, int agentID)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(TreeNode::IdException& e){
      char buffer[255];
      sprintf(buffer,"Unable to obtain Agent with the ID %i in getR/getRID as it does not exist",id);
      throw LuaScript::Exception(buffer);
      return lua_error(L);
    }catch(std::out_of_range& e){
      // getredfrompoint
	char buffer[255];
	sprintf(buffer,"Unable to obtain image of index %i in getR/getRID as it does not exist. \n\tTry \n\t getR(int imageIndex)\n\t getRID(int imageIndex, int agentID)",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(...){
	char buffer[255];
	sprintf(buffer,"Unable to getR/getRID. \n\tTry \n\t getR(int imageIndex)\n\t getRID(int imageIndex, int agentID)",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }
}

int LuaScript::getgreenID(lua_State* L)
{
    int id;
    int bufferIndex;
    int totalArgs = lua_gettop(L);

    try{
	if(totalArgs != 2)throw "error";
	id= SafeGetNumberLua(L,-1);

	TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
	if(!activeAgent){
	DebugStream << "could not get agent"<<std::endl;
	throw LuaScript::Exception("Could not retrieve agent by id in getGreen function");
	}


	int bufferIndex = SafeGetNumberLua(L,-2);

	float val(0);
	if(WorldEngine::get_instance().get_surface(bufferIndex)){
	  Vec2 pos =boost::get<Vec2>(activeAgent->get_property("pos").get_value());
	  val = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(pos).G();
	}else{
	  val = 0;
	}
	lua_pushnumber(L,val);
	return 1;
    }catch(LuaScript::NumberException& e){
	char buffer[255];
	sprintf(buffer,"Unable to obtain ID or image buffer in getG/getGID. \n\tTry \n\t getG(int imageIndex)\n\t getGID(int imageIndex, int agentID)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(TreeNode::IdException& e){
      char buffer[255];
      sprintf(buffer,"Unable to obtain Agent with the ID %i in getG/getGID as it does not exist",id);
      throw LuaScript::Exception(buffer);
      return lua_error(L);
    }catch(std::out_of_range& e){
      // getredfrompoint
	char buffer[255];
	sprintf(buffer,"Unable to obtain image of index %i in getG/getGID as it does not exist. \n\tTry \n\t getG(int imageIndex)\n\t getGID(int imageIndex, int agentID)",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(...){
	char buffer[255];
	sprintf(buffer,"Unable to getG/getGID. \n\tTry \n\t getG(int imageIndex)\n\t getGID(int imageIndex, int agentID)",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }
}

int LuaScript::getblueID(lua_State* L)
{
    int id;
    int bufferIndex;
    int totalArgs = lua_gettop(L);

    try{
	  if(totalArgs != 2)throw "error";
	  id= SafeGetNumberLua(L,-1);

	  TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
	  if(!activeAgent){
	    DebugStream << "could not get agent"<<std::endl;
	    throw LuaScript::Exception("Could not retrieve agent by id in getBlue function");
	  }


	  bufferIndex = SafeGetNumberLua(L,-2);

	  float val(0);
	  if(WorldEngine::get_instance().get_surface(bufferIndex)){
	    Vec2 pos =boost::get<Vec2>(activeAgent->get_property("pos").get_value());
	    val = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(pos).B();
	  }else{
	    val = 0;
	  }
	  lua_pushnumber(L,val);
	  return 1;
    }catch(LuaScript::NumberException& e){
	char buffer[255];
	sprintf(buffer,"Unable to obtain ID or image buffer in getB/getBID. \n\tTry \n\t getB(int imageIndex)\n\t getBID(int imageIndex, int agentID)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(TreeNode::IdException& e){
      char buffer[255];
      sprintf(buffer,"Unable to obtain Agent with the ID %i in getB/getBID as it does not exist",id);
      throw LuaScript::Exception(buffer);
      return lua_error(L);
    }catch(std::out_of_range& e){
      // getredfrompoint
	char buffer[255];
	sprintf(buffer,"Unable to obtain image of index %i in getB/getBID as it does not exist. \n\tTry \n\t getB(int imageIndex)\n\t getBID(int imageIndex, int agentID)",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(...){
	char buffer[255];
	sprintf(buffer,"Unable to getB/getBID. \n\tTry \n\t getB(int imageIndex)\n\t getBID(int imageIndex, int agentID)",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }
}


int LuaScript::gettotalColID(lua_State* L)
{
    int id;
    int bufferIndex;
   int totalArgs = lua_gettop(L);

    try{
      if(totalArgs != 2)throw "error";
	id = SafeGetNumberLua(L,-1);
	DebugStream << "getting total col"<<std::endl;
	TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
// 	if(!activeAgent){
// 	  DebugStream << "could not get agent"<<std::endl;
// 	  throw LuaScript::Exception("Could not retrieve agent by id in getTotalCol function");
// 	}
//
	bufferIndex = SafeGetNumberLua(L,-2);

	float val(0),r(0),g(0),b(0);
	if(WorldEngine::get_instance().get_surface(bufferIndex)){
	  Vec2 pos =boost::get<Vec2>(activeAgent->get_property("pos").get_value());
	  r = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(pos).R();
	  g = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(pos).G();
	  b = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(pos).B();
	  val = r+g+b;
	  DebugStream << "Val " << val<<std::endl;
	  lua_pushnumber(L,val);
	  return 1;
	}else{
	  val = 0;
	   lua_pushnumber(L,val);
	  return 1;
	}
	return 0;

    }catch(LuaScript::NumberException& e){
	char buffer[255];
	sprintf(buffer,"Unable to obtain ID or image buffer in getTotalCol/getTotalColID. \n\tTry \n\t getTotalCol(int imageIndex)\n\t getTotalColID(int imageIndex, int agentID)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(TreeNode::IdException& e){
      char buffer[255];
      sprintf(buffer,"Unable to obtain Agent with the ID %i in getTotalCol/getTotalColID as it does not exist",id);
      throw LuaScript::Exception(buffer);
      return lua_error(L);
    }catch(std::out_of_range& e){
      // getredfrompoint
	char buffer[255];
	sprintf(buffer,"Unable to obtain image of index %i in getTotalCol/getTotalColID as it does not exist. \n\tTry \n\t getTotalCol(int imageIndex)\n\t gettotalColID(int imageIndex, int agentID)",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(...){
	char buffer[255];
	sprintf(buffer,"Unable to getTotalCol/getTotalColID. \n\tTry \n\t getTotalCol(int imageIndex)\n\t getTotalColID(int imageIndex, int agentID)",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }
}

int LuaScript::getmeanColID(lua_State* L)
{
    int id;
    int bufferIndex;
   int totalArgs = lua_gettop(L);

    try{
	if(totalArgs != 2)throw "error";
	id = SafeGetNumberLua(L,-1);

	TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
	if(!activeAgent){
	  DebugStream << "could not get agent"<<std::endl;
	  throw LuaScript::Exception("Could not retrieve agent by id in getMeanCol function");
	}


	const int bufferIndex = SafeGetNumberLua(L,-2);

	float val(0),r(0),g(0),b(0);
	if(WorldEngine::get_instance().get_surface(bufferIndex)){
	    Vec2 pos =boost::get<Vec2>(activeAgent->get_property("pos").get_value());
	    r = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(pos).R();
	    g = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(pos).G();
	    b = WorldEngine::get_instance().get_surface(bufferIndex)->get_rgba_at(pos).B();
	    val = float(r+g+b)/3.0f;
	}else{
	  val = 0.5;
	}
	lua_pushnumber(L,val);
	return 1;
    }catch(LuaScript::NumberException& e){
	char buffer[255];
	sprintf(buffer,"Unable to obtain ID or image buffer in getMeanCol/getMeanColID. \n\tTry \n\t getMeanCol(int imageIndex)\n\t getMeanColID(int imageIndex, int agentID)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(TreeNode::IdException& e){
      char buffer[255];
      sprintf(buffer,"Unable to obtain Agent with the ID %i in getMeanCol/getMeanColID as it does not exist",id);
      throw LuaScript::Exception(buffer);
      return lua_error(L);
    }catch(std::out_of_range& e){
      // getredfrompoint
	char buffer[255];
	sprintf(buffer,"Unable to obtain image of index %i in getMeanCol/getMeanColID as it does not exist. \n\tTry \n\t getMeanCol(int imageIndex)\n\t getMeanColID(int imageIndex, int agentID)",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(...){
	char buffer[255];
	sprintf(buffer,"Unable to getMeanCol/getTotalColID. \n\tTry \n\t getMeanCol(int imageIndex)\n\t getMeanColID(int imageIndex, int agentID)",bufferIndex);
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }
}




int LuaScript::getworldsize(lua_State* L)
{
  try{
    lua_pushVec2( L,  boost::get<Vec2>( WorldEngine::get_instance().get_world()->get_property("WorldSize").get_value()  ) );
    return 1;
  }catch(...){
   // getworldsize
    char buffer[255];
    sprintf(buffer,"Unable to get the worldsize! Please restart the software as this should never happen!");
    throw LuaScript::Exception(buffer);
    return lua_error(L);
  }
}



int LuaScript::rawoscSF(lua_State* L)
{
  try{
      int totalArgs = lua_gettop(L);
      if(totalArgs !=5)throw "error";

      std::string addr = SafeGetStringLua(L,-5);
      std::string port = SafeGetStringLua(L,-4);
      std::string path = SafeGetStringLua(L,-3);
      std::string prefix = SafeGetStringLua(L,-2);

      float value = SafeGetNumberLua(L,-1);

      lo_address t_address = lo_address_new(addr.c_str(),port.c_str());
      assert(t_address);
      if(lo_send(t_address,path.c_str(),"sf",prefix.c_str(),float(value)) == -1 ){
	  throw t_address;
      }
      lo_address_free(t_address);

      return 1;
  }catch(...){
   //oscSF
    char buffer[255];
    sprintf(buffer,"Unable to use oscSF (Advisable that you use send instead). \n\t oscSF(string ipAddress,string portNumber,string path, string text,float value)");
    throw LuaScript::Exception(buffer);
  }
}

int LuaScript::rawoscF(lua_State* L)
{

  try{
      int totalArgs = lua_gettop(L);
      if(totalArgs !=4)throw "error";
      std::string addr = SafeGetStringLua(L,-4);
      std::string port = SafeGetStringLua(L,-3);
      std::string path = SafeGetStringLua(L,-2);

      float value = SafeGetNumberLua(L,-1);

      lo_address t_address = lo_address_new(addr.c_str(),port.c_str());
      assert(t_address);
      if(lo_send(t_address,path.c_str(),"f",float(value)) == -1 ){
	  throw t_address;
      }
      lo_address_free(t_address);

      return 1;
  }catch(...){
      char buffer[255];
      sprintf(buffer,"Unable to use oscF (Advisable that you use send instead). \n\t oscF(string ipAddress,string portNumber,string path,float value)");
      throw LuaScript::Exception(buffer);
  }
}

int LuaScript::rawoscSI(lua_State* L)
{
    try{
      int totalArgs = lua_gettop(L);
      if(totalArgs !=5)throw "error";
      std::string addr = SafeGetStringLua(L,-5);
      std::string port = SafeGetStringLua(L,-4);
      std::string path = SafeGetStringLua(L,-3);
      std::string prefix = SafeGetStringLua(L,-2);

      float value = SafeGetNumberLua(L,-1);

      lo_address t_address = lo_address_new(addr.c_str(),port.c_str());
      assert(t_address);
      if(lo_send(t_address,path.c_str(),"si",prefix.c_str(),int(value)) == -1 ){
	  throw t_address;
      }
      lo_address_free(t_address);

      return 1;
    }catch(...){
      char buffer[255];
      sprintf(buffer,"Unable to use oscSI (Advisable that you use send instead). \n\t oscSI(string ipAddress,string portNumber,string path,string text,float value)");
      throw LuaScript::Exception(buffer);
  }
}

int LuaScript::rawoscI(lua_State* L)
{

    try{
      int totalArgs = lua_gettop(L);
      if(totalArgs !=4)throw "error";
      std::string addr = SafeGetStringLua(L,-4);
      std::string port = SafeGetStringLua(L,-3);
      std::string path = SafeGetStringLua(L,-2);

      float value = SafeGetNumberLua(L,-1);

      lo_address t_address = lo_address_new(addr.c_str(),port.c_str());
      assert(t_address);
      if(lo_send(t_address,path.c_str(),"i",int(value)) == -1 ){
	  throw t_address;
      }
      lo_address_free(t_address);

      return 1;
    }catch(...){
      char buffer[255];
      sprintf(buffer,"Unable to use oscI (Advisable that you use send instead). \n\t oscI(string ipAddress,string portNumber,string path,int value)");
      throw LuaScript::Exception(buffer);
  }
}


int LuaScript::rawoscSS(lua_State* L)
{
   try{
      int totalArgs = lua_gettop(L);
      if(totalArgs !=5)throw "error";

      std::string addr = SafeGetStringLua(L,-5);
      std::string port = SafeGetStringLua(L,-4);
      std::string path = SafeGetStringLua(L,-3);
      std::string prefix = SafeGetStringLua(L,-2);

      std::string value = SafeGetStringLua(L,-1);

      lo_address t_address = lo_address_new(addr.c_str(),port.c_str());
      assert(t_address);
      if(lo_send(t_address,path.c_str(),"ss",prefix.c_str(),value.c_str()) == -1 ){
	  throw t_address;
      }
      lo_address_free(t_address);

      return 1;
   }catch(...){
      char buffer[255];
      sprintf(buffer,"Unable to use oscSS (Advisable that you use send instead). \n\t oscSS(string ipAddress,string portNumber,string path,string text1,string text2)");
      throw LuaScript::Exception(buffer);
  }
}

int LuaScript::rawoscS(lua_State* L)
{
   try{
      int totalArgs = lua_gettop(L);
      if(totalArgs !=4)throw "error";
      std::string addr = SafeGetStringLua(L,-4);
      std::string port = SafeGetStringLua(L,-3);
      std::string path = SafeGetStringLua(L,-2);

      std::string value = SafeGetStringLua(L,-1);

      lo_address t_address = lo_address_new(addr.c_str(),port.c_str());
      assert(t_address);
      if(lo_send(t_address,path.c_str(),"s",value.c_str()) == -1 ){
	  throw t_address;
      }
      lo_address_free(t_address);

      return 1;
   }catch(...){
      char buffer[255];
      sprintf(buffer,"Unable to use oscS (Advisable that you use send instead). \n\t oscS(string ipAddress,string portNumber,string path,string text)");
      throw LuaScript::Exception(buffer);
  }
}


void set_intfield(lua_State* state,int index,int id){
    lua_pushinteger(state,index);
    lua_pushinteger(state,id);
    lua_settable(state,-3);
}
void set_field(lua_State* state,const char* index,int id){
    lua_pushstring(state,index);
    lua_pushinteger(state,id);
    lua_settable(state,-3);
}
int LuaScript::getNearbyID(lua_State* L)
{

  int argNos = lua_gettop(L);
  const Vec2* pos;
  int id;
  float radius;

  try{
    if(argNos ==2){

      if(lua_isnumber(L,-1)){
	id = SafeGetNumberLua(L,-1);
	TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
	pos = &activeAgent->get_pos();
      }else if(lua_isuserdata(L,-1)){
	pos = lvec2_check(L,-1);
      }else{
	throw "Error";
      }

      radius = SafeGetNumberLua(L,-2);

    }else if (argNos == 3){
    // if(lua_isnumber(L,-1)||lua_isstring(L,-1))throw "error";
      if(lua_isnumber(L,-3)){
	  	radius = SafeGetNumberLua(L,-3);
	  //throw "error";
      }else if(lua_isuserdata(L,-3)){
	  	pos = lvec2_check(L,-3);
	  	radius = SafeGetNumberLua(L,-2);
      }else{
	  throw "Error";
      }


    }
      Vec2 t = *pos;
      IdVector* ids = get_nearby_ids(t,radius );



      //   lua_newtable( L);
      // set_field(L,"size",ids->size());
	lua_createtable(L,ids->size(),0);

      for(int n = 0; n < ids->size();++n){
  //     std::cout << " Nearby Id = "<< ids->at(n)<<std::endl;
  //     set_intfield(L,n,ids->at(n));
	lua_pushinteger(L, ids->at(n));
	lua_rawseti (L, -2, n);
      }
      return 1;
    //"\n\t Try \n\t getNearby(int radius)\n\t getNearby(Vec2 position, int radius) \n\t getNearbyID(int radius, int agentID)"
  }catch(TreeNode::IdException& e){
     char buffer[255];
     sprintf(buffer,"Unable to getNearby/getNearbyID for Agent with ID %i as it does not exist. \n\t Try \n\t getNearby(int radius)\n\t getNearby(Vec2 position, int radius) \n\t getNearbyID(int radius, int agentID)",id);
     throw LuaScript::Exception(buffer);
  }catch(LuaScript::Vec2Exception& e){
      throw LuaScript::Exception("Unable to obtain Vec2 in getNearby/getNearbyID");
  }catch(...){
     char buffer[255];
     sprintf(buffer,"Unable to getNearby/getNearbyID Agents. \n\t Try \n\t getNearby(int radius)\n\t getNearby(Vec2 position, int radius) \n\t getNearbyID(int radius, int agentID)",id);
     throw LuaScript::Exception(buffer);
  }

}

//is temperamentall....
int LuaScript::get_type(lua_State* L)
{

  int id;
  int argNos = lua_gettop(L);
  try{
    if(argNos !=1 )throw "Error" ;
    id = SafeGetNumberLua(L,-1);
    TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();

    std::string name = activeAgent->get_script_name();
    lua_pushstring(L,name.c_str());
    return 1;
  }catch(LuaScript::NumberException& e){
     char buffer[255];
     sprintf(buffer,"Could not obtain the ID of the Agent in getType/getTypeID. \n\t Try \n\t getType()\n\t getTypeID(int agentID)");
     throw LuaScript::Exception(buffer);
  }catch(... ){
     char buffer[255];
     sprintf(buffer,"Could not obtain Agent with ID %i as it does not exist. \n\t Try \n\t getType()\n\t getTypeID(int agentID)",id);
     throw LuaScript::Exception(buffer);
  }

}


int LuaScript::send_osc(lua_State* L)
{

    try{
	//ag   lo_address t_address = lo_address_new(addr.c_str(),port.c_str());
	int argCount = lua_gettop(L);
	DebugStream <<"Send args :" << argCount <<std::endl;
	if(argCount < 3){
	  char buffer[255];
	  sprintf(buffer,"Not enough arguments provided for send (%i). You need to provide at least an Ip Address, a port number and a path.\n\t Try \n\t send(string ipAddress,string port,string path)\n\t send(string ipAddress,int port,string path)",argCount);
	  throw LuaScript::Exception(buffer);
	  return lua_error(L);
	}
	int ipLocation = argCount * -1;
	DebugStream <<"ipLocation :" << ipLocation <<std::endl;

	std::string ipString,portString,pathString;
	//parse Ip address...
	if(lua_isstring(L,ipLocation)){
	    ipString = SafeGetStringLua(L,ipLocation);
	}else{
	  throw LuaScript::Exception("The IP address provided in the send is not formatted a string");
	  return lua_error(L);
	}


	int portLocation = ipLocation +1;
	if(lua_isstring(L,portLocation)){
	    portString = SafeGetStringLua(L,portLocation);
	}else if(lua_isnumber(L,portLocation)){
	    int number = SafeGetNumberLua(L,portLocation);
	    std::stringstream ss;
	    ss << number;
	    portString = ss.str();
	}else{
	  throw LuaScript::Exception("The port address provided in the send is not formatted a string");
	  return lua_error(L);
	}

	int pathLocation = ipLocation +2;
	if(lua_isstring(L,pathLocation)){
	    pathString = SafeGetStringLua(L,pathLocation);
	}else{
	  throw LuaScript::Exception("The path provided in the send is not formatted a string");
	  return lua_error(L);
	}





	lo_address address = lo_address_new(ipString.c_str(),portString.c_str());

	lo_message message = lo_message_new();
// 	if(lo_send(t_address,path.c_str(),"ss",prefix.c_str(),value.c_str()) == -1 ){
// 	    throw t_address;
//       }

	DebugStream << ipString << " "<< portString <<std::endl;
	bool hasTable = false;
	int dataLocation =pathLocation + 1;
	float t;
	std::string ts;
	while(dataLocation < 0){
	  DebugStream<< dataLocation<<std::endl;
	  if(hasTable)break;
	  if(lua_isnumber(L,dataLocation)){
	     t = SafeGetNumberLua(L,dataLocation);
	     lo_message_add_float(message,t);
	  }else if(lua_isstring(L,dataLocation)){
	    ts = SafeGetStringLua(L,dataLocation);
	    lo_message_add_string(message,ts.c_str());
	  }else if(lua_istable(L,dataLocation)){
	    DebugStream << "Table found at "<< dataLocation <<std::endl;


		lua_pushnil(L);  /* first key */
		PropertyValue val;
		std::string id ;

		while (lua_next(L, dataLocation-1) !=0 ) {

		//    if(!lua_isstring(L,-2)  ){
		//      throw LuaScript::Exception("Error parsing the table provided in send function. One of the ")
		//    }
		    id = SafeGetStringLua(L,dataLocation-1);
		     DebugStream << "id = " << id<< std::endl;
		    lo_message_add_string(message,id.c_str());


		    if( lua_isnumber(L,dataLocation)){
		      val = float(SafeGetNumberLua(L,-1));

		      lo_message_add_float(message,boost::get<float>(val));
		    }else if (lua_isstring(L,dataLocation)){
		      val = std::string(SafeGetStringLua(L,dataLocation));

		      lo_message_add_string(message, boost::get<std::string>(val).c_str());
		    }else{
		      val = 0;
		    }
		    lua_pop(L,1);
		   }
		  hasTable = true;
	  }


	   ++dataLocation;
	}


	if ( lo_send_message(address,pathString.c_str(),message)  == -1){
	   char buffer[255];
	  sprintf(buffer,"Could not send osc message. Basic IP,Port or Path data provided is incorrect (%s %s %s)",ipString.c_str(),portString.c_str(),pathString.c_str());
	  lo_address_free(address);
	  lo_message_free(message);
	  throw LuaScript::Exception(buffer);
	  return lua_error(L);
	}


	lo_address_free(address);
        lo_message_free(message);


	return 1;
    }catch(LuaScript::StringException){
        char buffer[255];
	sprintf(buffer,"Unable to parse string in send. You need to provide at least an Ip Address, a port number and a path.\n\t Try \n\t send(string ipAddress,string port,string path)\n\t send(string ipAddress,int port,string path)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }catch(LuaScript::NumberException){
        char buffer[255];
	sprintf(buffer,"Unable to parse number in send.)");
	throw LuaScript::Exception(buffer);
	return lua_error(L);
    }
}

int LuaScript::get_current_screen(lua_State* L)
{
   try{
	lua_pushnumber(L,WorldEngine::get_instance().get_current_screen() );
	return 1;
      }catch(...){
	 char buffer[255];
	 sprintf(buffer,"Could not obtain the current screen(image) ID from the world");
	 throw LuaScript::Exception(buffer);
	 return 0;
      }
}




/////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////



Vec2* LuaScript::lvec2_check(lua_State* L,int index)
{
	void *ud = luaL_checkudata(L, index, vec2_metaTableName);
	if(ud == NULL)throw LuaScript::Vec2Exception("Bad Vec2");
	luaL_argcheck(L, ud != NULL, 1, "bad vec2");
	return (Vec2 *)ud;
}

int LuaScript::lua_pushVec2(lua_State* L, const Vec2& in){
    Vec2* v = new( lua_newuserdata(L, sizeof(Vec2) ) ) Vec2(in);
    luaL_getmetatable(L, vec2_metaTableName);
    lua_setmetatable(L, -2);
    return 1;
}

int LuaScript::lvec2_new(lua_State* L)
{
  int args = lua_gettop(L);
  try{
    if(args == 2){

      float x= SafeGetNumberLua(L,-2);
      float y = SafeGetNumberLua(L,-1);
      return lua_pushVec2(L,Vec2(x,y));

    }else if(args == 1){
      float x= SafeGetNumberLua(L,-1);
      float y = 0;
      return lua_pushVec2(L,Vec2(x,y));
    }else{
      return lua_pushVec2(L,Vec2(0,0));
    }
  }catch(LuaScript::NumberException& e){
    return lua_pushVec2(L,Vec2(0,0));
  }
}
int LuaScript::lvec2_getx(lua_State* L){
  try{
	   Vec2* v = lvec2_check(L,1);
	 lua_pushnumber(L, v->m_x );
	 return 1;
 }catch(...){
   lua_pushfstring(L, "Could not obtain x of Vec2");
   return 0;
 }
}

int LuaScript::lvec2_gety(lua_State* L){
	try{
    Vec2* v = lvec2_check(L,1);
  	lua_pushnumber(L, v->m_y );
  	return 1;

  }catch(...){
    lua_pushfstring(L, "Could not obtain y of Vec2");
    return 0;
  }
}

int LuaScript::lvec2_set(lua_State* L){
  try{
  	Vec2* v = lvec2_check(L,1);
  	v->m_x = SafeGetNumberLua(L,-2);
  	v->m_y = SafeGetNumberLua(L,-1);
  	return 1;
  }catch(...){
    lua_pushfstring(L, "Could not set Vec2");
    return 0;
  }
}

int LuaScript::lvec2_setx(lua_State* L)
{
  try{
      Vec2* v = lvec2_check(L,1);
      v->m_x = SafeGetNumberLua(L,-1);
      return 1;
    }catch(...){
      lua_pushfstring(L, "Could not set x of Vec2");
      return 0;
    }
}
int LuaScript::lvec2_sety(lua_State* L)
{

  try{
      Vec2* v = lvec2_check(L,1);
      v->m_y = lua_tonumber(L,-1);
      return 1;
    }catch(...){
      lua_pushfstring(L, "Could not set y of Vec2");
      return 0;
    }
}


int LuaScript::lvec2_2string (lua_State *L) {
  try{
	   Vec2* v = lvec2_check(L,1);
	 lua_pushfstring(L, "[%f,%f]", v->m_x, v->m_y);
	 return 1;
 }catch(...){
   lua_pushfstring(L, "Could not obtain Vec2 to print");
   return 0;
 }
}

int LuaScript::lvec2_add(lua_State* L)
{
  try{
    const Vec2* v = lvec2_check(L,1);
    if(lua_isnumber(L,2)){
 	lua_pushVec2( L, (float)SafeGetNumberLua(L,2) + *v );
	return 1;
    }else if( lua_isuserdata(L,2)     ){
	 const Vec2* v2 = lvec2_check(L,2);
 	  lua_pushVec2(L, *v + *v2);
	  return 1;
    }
     return 0;
   }catch(...){
     lua_pushfstring(L, "Could perform addition on Vec2");
     return 0;
   }
}
int LuaScript::lvec2_sub(lua_State* L)
{

  try{
    Vec2* v = lvec2_check(L,1);
    if(lua_isnumber(L,2)){
 	lua_pushVec2(L , *v - (float)SafeGetNumberLua(L,2) );
	return 1;
    }else if( lua_isuserdata(L,2)     ){
	 Vec2* v2 = lvec2_check(L,2);
 	  lua_pushVec2(L,  *v - *v2);
	  return 1;
    }
  //   lua_newuserdata(L, sizeof(Vec2) ) ) ;
     return 0;
   }catch(...){
     lua_pushfstring(L, "Could perform subtraction on Vec2");
     return 0;
   }
}

int LuaScript::lvec2_mul(lua_State* L)
{
  try{
  Vec2* v = lvec2_check(L,1);
    if(lua_isnumber(L,2)){
 	lua_pushVec2(L , *v * (float)SafeGetNumberLua(L,2) );
	return 1;
    }
  //   lua_newuserdata(L, sizeof(Vec2) ) ) ;
     return 0;
   }catch(...){
     lua_pushfstring(L, "Could perform multiplication on Vec2");
     return 0;
   }
}
int LuaScript::lvec2_div(lua_State* L)
{
  try{
    Vec2* v = lvec2_check(L,1);
    if(lua_isnumber(L,2)){
 	lua_pushVec2(L, *v / (float)SafeGetNumberLua(L,2)) ;
	return 1;
    }
  //   lua_newuserdata(L, sizeof(Vec2) ) ) ;
     return 0;
   }catch(...){
     lua_pushfstring(L, "Could perform division on Vec2");
     return 0;
   }
}

int LuaScript::lvec2_unm(lua_State* L)
{
  try{
    Vec2* v = lvec2_check(L,1);
    *v =  -*v;
  //   lua_newuserdata(L, sizeof(Vec2) ) ) ;
     return 1;
   }catch(...){
     lua_pushfstring(L, "Could perform uniary inversion on Vec2");
     return 0;
   }
}

int LuaScript::lvec2_dot(lua_State* L)
{
  try{
  Vec2* v = lvec2_check(L,1);
  Vec2* v2 = lvec2_check(L,2);
  lua_pushnumber(L,v->dot(*v2));
  return 1;
  }catch(...){
      lua_pushfstring(L, "Could perform dot product on Vec2");
      return 0;
  }
}
int LuaScript::lvec2_mag(lua_State* L)
{
  try{
    Vec2* v = lvec2_check(L,1);
    lua_pushnumber(L,v->magnitude());
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could perform calculate the magnitude on Vec2");
    return 0;
  }
}
int LuaScript::lvec2_norm(lua_State* L)
{
  try{
   Vec2* v = lvec2_check(L,1);
   lua_pushVec2(L,v->normalise());
   return 1;
 }catch(...){
   lua_pushfstring(L, "Could perform normalize Vec2");
   return 0;
 }
}
int LuaScript::lvec2_perp(lua_State* L)
{
  try{
   Vec2* v = lvec2_check(L,1);
   lua_pushVec2(L,v->perp());
   return 1;
 }catch(...){
   lua_pushfstring(L, "Could not obtain perpendicular of Vec2");
   return 0;
 }
}




static const struct luaL_reg vec2lib_f [] = {
	{"new",LuaScript::lvec2_new},
	{NULL, NULL}
};

static const struct luaL_reg vec2lib_m [] = {
	{"x",LuaScript::lvec2_getx},
	{"y",LuaScript::lvec2_gety},
	{"set",LuaScript::lvec2_set},
	{"__tostring",LuaScript::lvec2_2string},
	{"__add",LuaScript::lvec2_add},
	{"__sub",LuaScript::lvec2_sub},
	{"__mul",LuaScript::lvec2_mul},
	{"__div",LuaScript::lvec2_div},
	{"__unm",LuaScript::lvec2_unm},
	{"dot",LuaScript::lvec2_dot},
	{"mag",LuaScript::lvec2_mag},
	{"norm",LuaScript::lvec2_norm},
	{"perp",LuaScript::lvec2_perp},
	{NULL, NULL}
};
int LuaScript::setup_vec2lib(lua_State* L)
{
	luaL_newmetatable(L, vec2_metaTableName);

	lua_pushstring(L, "__index");
	lua_pushvalue(L, -2);
	lua_settable(L, -3);
	luaL_openlib(L, NULL, vec2lib_m, 0);

	luaL_openlib(L, "Vec2", vec2lib_f, 0);
	return 1;
}




////////////////////////////////////////////////////////////

int LuaScript::lrgba_new(lua_State* L)
{

  int args = lua_gettop(L);
  float r(0),g(0),b(0),a(0);
  try{

  if(args == 4){
    r= SafeGetNumberLua(L,-4);
    g = SafeGetNumberLua(L,-3);
    b= SafeGetNumberLua(L,-2);
    a = SafeGetNumberLua(L,-1);
  }else if(args == 3){
    r= SafeGetNumberLua(L,-3);
    g = SafeGetNumberLua(L,-2);
    b= SafeGetNumberLua(L,-1);
  }

  }catch(LuaScript::NumberException& e){
      r = g = b = a = 0;
  }
  return lua_pushRGBA(L,RGBA(r,g,b,a));
}
int LuaScript::lrgba_getr(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    lua_pushnumber(L,col->R());
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could not get red of RGBA");
    return 0;
  }
}
int LuaScript::lrgba_getg(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    lua_pushnumber(L,col->G());
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could not get green of RGBA");
    return 0;
  }
}
int LuaScript::lrgba_getb(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    lua_pushnumber(L,col->B());
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could not get blue of RGBA");
    return 0;
  }
}
int LuaScript::lrgba_geta(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    lua_pushnumber(L,col->A());
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could not get alpha of RGBA");
    return 0;
  }
}

int LuaScript::lrgba_mean(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    lua_pushnumber(L,(col->R() + col->G() + col->B())/3.0f );
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could not get the average/mean of RGBA");
    return 0;
  }


}

static const struct luaL_reg rgbalib_f [] = {
	{"new",LuaScript::lrgba_new},
	{NULL, NULL}
};
static const struct luaL_reg rgbalib_m [] = {
	{"r",LuaScript::lrgba_getr},
	{"g",LuaScript::lrgba_getg},
	{"b",LuaScript::lrgba_getb},
	{"a",LuaScript::lrgba_geta},
  	{"mean",LuaScript::lrgba_mean},
	{"__tostring",LuaScript::lrgba_2string},
	{"__add",LuaScript::lrgba_add},
	{"__sub",LuaScript::lrgba_sub},
	{"__mul",LuaScript::lrgba_mul},
	{"__div",LuaScript::lrgba_div},
	{NULL, NULL}
};

int LuaScript::setup_rgbalib(lua_State* L)
{
	luaL_newmetatable(L, rgba_metaTableName);

	lua_pushstring(L, "__index");
	lua_pushvalue(L, -2);
	lua_settable(L, -3);
	luaL_openlib(L, NULL, rgbalib_m, 0);

	luaL_openlib(L, "RGBA", rgbalib_f, 0);
	return 1;
}

RGBA* LuaScript::lrgba_check(lua_State* L, int index)
{
	void *ud = luaL_checkudata(L, index, rgba_metaTableName);
  if(ud == NULL)throw LuaScript::RGBAException("Bad RGBA");
	luaL_argcheck(L, ud != NULL, 1, "bad RGBA");
	return (RGBA *)ud;
}
int LuaScript::lrgba_2string(lua_State* L)
{
  try{
	   RGBA* col = lrgba_check(L,1);
	   lua_pushfstring(L, "[%f,%f,%f,%f]", col->R(),col->G(),col->B(),col->A());
	   return 1;
  }catch(...){
    lua_pushfstring(L, "Invalid RGBA");
    return 0;
  }
}

int LuaScript::lua_pushRGBA(lua_State* L, const RGBA& in)
{
    RGBA* col = new( lua_newuserdata(L, sizeof(RGBA) ) ) RGBA(in);
    luaL_getmetatable(L, rgba_metaTableName);
    lua_setmetatable(L, -2);
    return 1;
}


int LuaScript::lrgba_set(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    col->R() = SafeGetNumberLua(L,-4);
    col->G() = SafeGetNumberLua(L,-3);
    col->B() = SafeGetNumberLua(L,-2);
    col->A() = SafeGetNumberLua(L,-1);
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could not set RGBA");
    return 0;
  }
}

int LuaScript::lrgba_setr(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    col->R() = SafeGetNumberLua(L,-1);
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could not set red of RGBA");
    return 0;
  }
}

int LuaScript::lrgba_setg(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    col->G() = SafeGetNumberLua(L,-1);
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could not set green of RGBA");
    return 0;
  }

}
int LuaScript::lrgba_setb(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    col->B() = SafeGetNumberLua(L,-1);
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could not set blue of RGBA");
    return 0;
  }
}
int LuaScript::lrgba_seta(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    col->A() = SafeGetNumberLua(L,-1);
    return 1;
  }catch(...){
    lua_pushfstring(L, "Could not set alpha of RGBA");
    return 0;
  }
}

int LuaScript::lrgba_add(lua_State* L)
{
  try{

   RGBA* col = lrgba_check(L,1);
    if(lua_isnumber(L,2)){
 	    lua_pushRGBA( L, *col + (float)SafeGetNumberLua(L,2) );
	      return 1;
    }else if( lua_isuserdata(L,2)     ){
	  RGBA* col2 = lrgba_check(L,2);
 	  lua_pushRGBA(L, *col + *col2);
	  return 1;
    }
     return 0;
   }catch(...){
     lua_pushfstring(L, "Could not add RGBA");
     return 0;
   }
}

int LuaScript::lrgba_sub(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    if(lua_isnumber(L,2)){
 	lua_pushRGBA( L, *col - (float)SafeGetNumberLua(L,2) );
	return 1;
    }else if( lua_isuserdata(L,2)     ){
	  RGBA* col2 = lrgba_check(L,2);
 	  lua_pushRGBA(L, *col - *col2);
	  return 1;
    }
     return 0;
   }catch(...){
     lua_pushfstring(L, "Could not subtract RGBA");
     return 0;
   }
}

int LuaScript::lrgba_mul(lua_State* L)
{
  try{
    RGBA* col = lrgba_check(L,1);
    if(lua_isnumber(L,2)){
 	lua_pushRGBA( L, *col * (float)SafeGetNumberLua(L,2) );
	return 1;
    }else if( lua_isuserdata(L,2)     ){
	  RGBA* col2 = lrgba_check(L,2);
	  RGBA newCol(col->R()*col2->R(), col->G()*col2->G(),col->B()*col2->B(),col->A()*col2->A() );;
 	  lua_pushRGBA(L, newCol);
	  return 1;
    }
     return 0;
   }catch(...){
     lua_pushfstring(L, "Could not multiply RGBA");
     return 0;
   }
}

int LuaScript::lrgba_div(lua_State* L)
{
  try{
   RGBA* col = lrgba_check(L,1);
    if(lua_isnumber(L,2)){
 	lua_pushRGBA( L, *col / (float)SafeGetNumberLua(L,2) );
	return 1;
    }else if( lua_isuserdata(L,2)     ){
	  RGBA* col2 = lrgba_check(L,2);
	  RGBA newCol(col->R()/col2->R(), col->G()/col2->G(),col->B()/col2->B(),col->A()/col2->A() );;
 	  lua_pushRGBA(L, newCol);
	  return 1;
    }
     return 0;

   }catch(...){
       lua_pushfstring(L, "Could not divide RGBA");
       return 0;
     }
}
///////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////


//this needs to deal with rgba and vec2
int LuaScript::custom_print(lua_State* L)
{
//
  int numArgs = lua_gettop(L);
  static std::string t;
  static std::string prefix;
  static TestOutput out;
  static int id = WorldEngine::get_instance().get_current_agent_id();
  static char buffer[33];
  sprintf(buffer,"%i (",id);

  prefix.clear();

  try{
    TreeNode::Ptr activeAgent = TreeNode::get_treenode_by_id(id).lock();
    prefix = std::string(buffer) + activeAgent->get_script_name() + ") : ";
  }catch(TreeNode::IdException& e){
    prefix = "(?) ";
  }
  t = prefix;

  for(int n = 1; n <= numArgs; ++n){
    if(lua_isstring(L,n)){

      t += lua_tostring(L,n);
    }else if(  lua_isboolean( L, n ) ){
      static bool printedBool = lua_toboolean(L,n);
      if(printedBool){
        t += " true ";
      }else{
        t += " false ";
      }
    }else if(lua_isuserdata(L,n)){
	   void *ud = luaL_checkudata(L, n, vec2_metaTableName);
	if(ud != NULL){
	  t += "Vec2 found";
	}else{
	  t += "Not Vec2 found";
	}
	if((Vec2 *)ud ){
//	  t += "Vec2 found";
	}else{
	  ud = luaL_checkudata(L,n,rgba_metaTableName);
	}
      }

  }
  out.print(t);

  return 0;

}


static const struct luaL_Reg printlib[] = {
  {"print",LuaScript::custom_print},
  {NULL,NULL}
};

int LuaScript::luaopen_print(lua_State* L)
{
  lua_getglobal(L,"_G");
  luaL_register(L,NULL,printlib);
  lua_pop(L,1);
}





//////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////



struct api_data{
  std::string m_functionName,m_arguementData,m_returns,m_exampleUsage,m_description;
  api_data(std::string name,std::string returns,std::string argueData,std::string desc):
    m_functionName(name),
    m_arguementData(argueData),
    m_returns(returns),
    m_description(desc)
  {

  }
};

std::ostream& operator<< (std::ostream& out, api_data& data){
  out << "\\subsubsection*{"<<data.m_functionName<< "}"<<std::endl;
  out << data.m_returns << " "<< data.m_functionName << "( "<< data.m_arguementData << " ) \\newline"<<std::endl;
  out << data.m_description <<std::endl;
  out << data.m_exampleUsage<<std::endl;
}

typedef std::vector<api_data> DocuVector;

class LuaRegistor{
  DocuVector m_documentVector;
public:
  LuaRegistor(){

  }
  void register_function(lua_State* L, lua_CFunction function,std::string name,std::string returns,std::string argueData,std::string desc){
      lua_register(L,name.c_str(),function);
      m_documentVector.push_back(api_data(name,returns,argueData,desc));

  }


  void print(){
    for(DocuVector::iterator it = m_documentVector.begin();it !=m_documentVector.end();++it){
	DebugStream << (*it);
    }

  }
  void print_to_latex(std::string path){
    std::fstream file(path.c_str(),std::fstream::out);
    file << "\\chapter{Lua API}"<<std::endl;
    file << "\\section{Agent Functions}"<<std::endl;
    for(DocuVector::iterator it = m_documentVector.begin();it !=m_documentVector.end();++it){
	file << (*it);
    }


    file.close();

  }

};

/////////////////////////////////////////////////


int LuaScript::test_function(lua_State* L)
{
  int args = lua_gettop(L);
  try{
  lvec2_check(L,-1);
  std::cout << "Vec"<<std::endl;
  }catch(LuaScript::Vec2Exception& e){
    std::cout << "no vec2"<<std::endl;
  }


}







/////////////////////////////////////////////////


void LuaScript::register_lua_functions(lua_State* L)
{
  DebugStream << "Registering lua functions"<<std::endl;
  lua_register(L,"initai",LuaScript::initai);

  lua_register(L,"getDT",LuaScript::getdt);
  lua_register(L,"getID",LuaScript::getid);
  lua_register(L,"hasChildren",self_call< LuaScript::has_children>);
  lua_register(L,"hasChildrenID",LuaScript::has_children );
  lua_register(L,"numberOfChildren",self_call< LuaScript::number_of_children>);
  lua_register(L,"numberOfChildrenID",LuaScript::number_of_children );
  lua_register(L,"getChild",self_call< LuaScript::get_childId>);
  lua_register(L,"getChildID",LuaScript::get_childId );
  lua_register(L,"getParent",self_call< LuaScript::get_parentid>);
  lua_register(L,"getParentID",LuaScript::get_parentid );
  lua_register(L,"die",self_call< LuaScript::kill>);
  lua_register(L,"killID",LuaScript::kill );
  lua_register(L,"spawn",LuaScript::spawn);
  lua_register(L,"spawnRelative",LuaScript::spawnrelative);
  lua_register(L,"spawnP",LuaScript::spawnArgs);
  lua_register(L,"spawnRelativeP",LuaScript::spawnrelativeArgs);
  lua_register(L,"getLifetimeID",LuaScript::getlifetime);
  lua_register(L,"getLifetime",self_call< LuaScript::getlifetime>);
  lua_register(L,"getPositionID", LuaScript::getposition);
  lua_register(L,"getPosition",self_call< LuaScript::getposition>);
  lua_register(L,"setPositionID",LuaScript::setposition);
  lua_register(L,"setPosition",self_call< LuaScript::setposition >);
  lua_register(L,"moveID",LuaScript::move);
  lua_register(L,"move",self_call<  LuaScript::move >);
  lua_register(L,"initPID",LuaScript::initproperties);
  lua_register(L,"initP",self_call<  LuaScript::initproperties >);
  lua_register(L,"addPID",LuaScript::addproperty);
  lua_register(L,"setPID",LuaScript::setproperty);
  lua_register(L,"getPID",LuaScript::getproperty);
  lua_register(L,"addP",self_call<LuaScript::addproperty>);
  lua_register(L,"setP",self_call<LuaScript::setproperty>);
  lua_register(L,"getP",self_call<LuaScript::getproperty>);

  //got here with the docs!

  lua_register(L,"changeMotionModelID",LuaScript::changemotionmodel);
  lua_register(L,"changeMotionModel",self_call<  LuaScript::changemotionmodel >);

  lua_register(L,"scNew",self_call<  LuaScript::sc_new>);
  lua_register(L,"scSet",self_call<  LuaScript::sc_set> );
  lua_register(L,"scFree",self_call<  LuaScript::sc_free>);
  lua_register(L,"getNearest",self_call< LuaScript::getnearest>);
  lua_register(L,"getNearestID",LuaScript::getnearest);
  lua_register(L,"drawRelativeID",LuaScript::drawrgbarelative);
  lua_register(L,"drawRelative",self_call< LuaScript::drawrgbarelative >);
  lua_register(L,"getRedFromPoint",LuaScript::getredfromsurface);
  lua_register(L,"getBlueFromPoint",LuaScript::getbluefromsurface);
  lua_register(L,"getGreenFromPoint",LuaScript::getgreenfromsurface);
  lua_register(L,"getTotalFromPoint",LuaScript::gettotalfromsurface);
  lua_register(L,"getMeanFromPoint",LuaScript::getmeanfromsurface);



  lua_register(L,"getRID",LuaScript::getredID);
  lua_register(L,"getR",self_call< LuaScript::getredID>);
  lua_register(L,"getGID",LuaScript::getgreenID);
  lua_register(L,"getG",self_call< LuaScript::getgreenID>);
  lua_register(L,"getBID",LuaScript::getblueID);
  lua_register(L,"getB",self_call< LuaScript::getblueID>);
  lua_register(L,"getTotalColID",LuaScript::gettotalColID);
  lua_register(L,"getTotalCol", self_call<  LuaScript::gettotalColID>);
  lua_register(L,"getMeanColID",LuaScript::getmeanColID);
  lua_register(L,"getMeanCol",self_call< LuaScript::getmeanColID>);
  lua_register(L,"getRGBA",self_call<LuaScript::get_rgba>);
  lua_register(L,"getRGBAID",LuaScript::get_rgba);
  lua_register(L,"getNearbyID",LuaScript::getNearbyID);
  lua_register(L,"getNearby",self_call< LuaScript::getNearbyID>);
  lua_register(L,"getWorldSize",LuaScript::getworldsize);
  lua_register(L,"oscSF",LuaScript::rawoscSF);
  lua_register(L,"oscF",LuaScript::rawoscF);
  lua_register(L,"oscSI",LuaScript::rawoscSI);
  lua_register(L,"oscI",LuaScript::rawoscI);
  lua_register(L,"oscSS",LuaScript::rawoscSS);
  lua_register(L,"oscS",LuaScript::rawoscS);
  lua_register(L,"killChildrenID",LuaScript::kill_children);
  lua_register(L,"killChildren",self_call< LuaScript::kill_children>);
  lua_register(L,"initP",LuaScript::initproperties);
  // lua_register(L,"parsePropertyTable",LuaScript::parseTable);
  lua_register(L,"addPathPoint",LuaScript::addpathpoint);
  lua_register(L,"removePathPoint",LuaScript::removepathpoint);
  lua_register(L,"movePathPoint",LuaScript::movepathpoint);
  lua_register(L,"addPath",LuaScript::add_path);
  lua_register(L,"removePath",LuaScript::remove_path);
  lua_register(L,"movePath",LuaScript::movepath);
  lua_register(L,"hasPID",LuaScript::hasProperty);
  lua_register(L,"hasP",self_call< LuaScript::hasProperty >);
  lua_register(L,"getTypeID",LuaScript::get_type);
  lua_register(L,"getType",self_call<LuaScript::get_type>);
  lua_register(L,"send",LuaScript::send_osc);
  lua_register(L,"currentScreen",LuaScript::get_current_screen);


  lua_register(L,"drawLine",LuaScript::draw_line);
  lua_register(L,"drawRect",LuaScript::draw_rect);
  lua_register(L,"drawCircle",LuaScript::draw_circle);


  lua_register(L,"testFunction",LuaScript::test_function);

  setup_vec2lib(L);
  setup_rgbalib(L);


  luaopen_print(L);


}
