#ifndef __LUAFUNCTIONS_HPP_
#define __LUAFUNCTIONS_HPP_

#include "lua/lua.hpp"
#include "lua/llimits.h"
#include <exception>
#include <string>

#include "Surface.hpp"

class Vec2;

namespace LuaScript{

int initai(lua_State* L);
int getdt(lua_State* L);
int getid(lua_State* L);
int kill(lua_State* L);

int spawn(lua_State* L);
int spawnrelative(lua_State* L);

int spawnArgs(lua_State* L);
int spawnrelativeArgs(lua_State* L);


int getposition(lua_State* L);
int setposition(lua_State* L);
int move(lua_State* L);
int getlifetime(lua_State* L);

int changemotionmodel(lua_State* L);

int sc_new(lua_State* L);
int sc_set(lua_State* L);
int sc_free(lua_State* L);


int addproperty(lua_State* L);


int setproperty(lua_State* L);
int getproperty(lua_State* L);


int getnearest(lua_State* L);


int drawrgbarelative(lua_State* L);


int kill_children(lua_State* L);
int has_children(lua_State* L);
int number_of_children(lua_State* L);
int get_childId(lua_State* L);
int get_parentid(lua_State* L);

//temporary
int getredfromsurface(lua_State* L);
int getbluefromsurface(lua_State* L);
int getgreenfromsurface(lua_State* L);
int gettotalfromsurface(lua_State* L);
int getmeanfromsurface(lua_State* L);

//path functions
int addpathpoint(lua_State* L);
int removepathpoint(lua_State* L);
int movepathpoint(lua_State* L);

int movepath(lua_State* L);


//return colour data from the position of agent
int get_rgba(lua_State* L);



int getredID(lua_State* L);
int getblueID(lua_State* L);
int getgreenID(lua_State* L);
int gettotalColID(lua_State* L);
int getmeanColID(lua_State* L);

int getworldsize(lua_State* L);


int rawoscSF(lua_State* L);
int rawoscF(lua_State* L);

int rawoscSI(lua_State* L);
int rawoscI(lua_State* L);


int rawoscSS(lua_State* L);
int rawoscS(lua_State* L);


int parseTable(lua_State* L);

int initproperties(lua_State* L);

int get_type(lua_State* L);


int hasProperty(lua_State* L);

int send_osc(lua_State* L);

int get_current_screen(lua_State* L);

//VEC2
const char* const vec2_metaTableName = "SP.vec2";
int lua_pushVec2(lua_State* L, const Vec2& in);
Vec2* lvec2_check(lua_State* L, int index);
int lvec2_new(lua_State* L);
int lvec2_getx(lua_State* L);
int lvec2_gety(lua_State* L);
int lvec2_setx(lua_State* L);
int lvec2_sety(lua_State* L);
int lvec2_set(lua_State* L);
int lvec2_2string (lua_State *L);
int lvec2_add(lua_State* L);
int lvec2_sub(lua_State* L);
int lvec2_div(lua_State* L);
int lvec2_mul(lua_State* L);
int lvec2_unm(lua_State* L);
int lvec2_dot(lua_State* L);
int lvec2_perp(lua_State* L);
int lvec2_mag(lua_State* L);
int lvec2_norm(lua_State* L);

int setup_vec2lib(lua_State* L);

//RGBA
const char* const rgba_metaTableName = "SP.rgba";
int lua_pushRGBA(lua_State* L, const RGBA& in);
RGBA* lrgba_check(lua_State* L, int index);
int lrgba_new(lua_State* L);
int lrgba_getr(lua_State* L);
int lrgba_getg(lua_State* L);
int lrgba_getb(lua_State* L);
int lrgba_geta(lua_State* L);
int lrgba_setr(lua_State* L);
int lrgba_setg(lua_State* L);
int lrgba_setb(lua_State* L);
int lrgba_seta(lua_State* L);
int lrgba_set(lua_State* L);
int lrgba_2string (lua_State *L);
int setup_rgbalib(lua_State* L);
int lrgba_add(lua_State* L);
int lrgba_sub(lua_State* L);
int lrgba_mul(lua_State* L);
int lrgba_div(lua_State* L);
int lrgba_mean(lua_State* L);



//getting other agents
int getNearbyID(lua_State* L);


int draw_line(lua_State* L);
int draw_rect(lua_State* L);
int draw_circle(lua_State* L);

int add_path(lua_State* L);
int remove_path(lua_State* L);

static int custom_print(lua_State* L);

extern int luaopen_print(lua_State* L);

int test_function(lua_State* L);


void register_lua_functions(lua_State* L);

class Exception : public std::exception{
      std::string m_msg;
      public:
	Exception(const char* msg) : m_msg(msg) {}
        virtual const char* what() const throw() { return m_msg.c_str(); }
	virtual ~Exception() throw() { }
    };
class NumberException : public std::exception{
      std::string m_msg;
      public:
	NumberException(const char* msg) : m_msg(msg) {}
        virtual const char* what() const throw() { return m_msg.c_str(); }
	virtual ~NumberException() throw() { }
};
class StringException : public std::exception{
      std::string m_msg;
      public:
	StringException(const char* msg) : m_msg(msg) {}
        virtual const char* what() const throw() { return m_msg.c_str(); }
	virtual ~StringException() throw() { }
};

class Vec2Exception : public std::exception{
      std::string m_msg;
      public:
	Vec2Exception(const char* msg) : m_msg(msg) {}
        virtual const char* what() const throw() { return m_msg.c_str(); }
	virtual ~Vec2Exception() throw() { }
};

class RGBAException : public std::exception{
      std::string m_msg;
      public:
	RGBAException(const char* msg) : m_msg(msg) {}
        virtual const char* what() const throw() { return m_msg.c_str(); }
	virtual ~RGBAException() throw() { }
};



}
#endif
