#include "MotionModel.hpp"
#include <cassert>

#include "LinearMotionModel.hpp"
#include "PhysicsMotionModel.hpp"
#include "StaticMotionModel.hpp"
#include "PathMotionModel.hpp"


MotionModel::Ptr MotionModel::factory(MotionModel::MotionType type, TreeNode* parent)
{
  MotionModel::Ptr t;
  switch(type){
    case(MT_STATIC):
      t = (Ptr(new StaticMotionModel(parent)));
      break;
    case(MT_LINEAR):
      t = ( Ptr(  new LinearMotionModel(parent)));
      break;
    case(MT_PHYSICS): 
      t = ( Ptr(  new PhysicsMotionModel(parent)));
      break;
    case(MT_PATH):
      t = (Ptr( new PathMotionModel(parent)));
      break;
    default:
	assert(false && "Wrong enum provided for factory");
	break;
  };
  
    assert(t && "Could not create motion model ptr in factory");
    return t;
}



MotionModel::MotionModel(TreeNode* parent):
  m_parent(parent)
{

}

TreeNode* MotionModel::get_parent()const
{
  return m_parent;
}



