#include "PathManager.hpp"
#include "tinyxml/tinyxml.h"

PathManager::PathManager()
{
  PathPtr p = PathPtr(new Path);
//   for(int n = 0; n < 4; ++n){
//     for(int r = 0; r < 4;++r){
//      p->add_point(100 + (n*200),100+(r*200));
//     }
//   }
  add_path(p);

}


PathManager& PathManager::get_instance()
{
  static PathManager p;
  return p;
}

int PathManager::add_path(PathPtr path)
{
  if(!path) return 0 ;
  m_paths.push_back(path);
  return m_paths.size() -1;
}


PathPtr PathManager::get_path(int index)
{
  if(index > m_paths.size()-1) index = index % m_paths.size();
  return m_paths.at(index);
}


int PathManager::get_paths_size()
{
  return m_paths.size();
}


void PathManager::remove_path(int index)
{
  if(index >= m_paths.size())return;
  PathVector::iterator it = m_paths.begin() + index;
  m_paths.erase(it);
}

void PathManager::move_path(int index, Vec2 delta)
{
  if(index >= m_paths.size())return;
  m_paths.at(index)->move(delta);

}


void PathManager::save_path_database(std::string filepath)
{
  DebugStream << "Attempting to save to "<< filepath<<std::endl;
  if(m_paths.empty()){
    DebugStream << "No paths in the system, not saving"<<std::endl;
    return;
  };
  TiXmlDocument doc(filepath.c_str());
  TiXmlDeclaration * decl = new TiXmlDeclaration( "1.0", "", "" );
  doc.LinkEndChild( decl );
  TiXmlElement * root = new TiXmlElement( "Paths" );
  doc.LinkEndChild( root );



  for(int path = 0; path < m_paths.size();++path ){
    TiXmlElement* PathElement = new TiXmlElement("Path");
    root->LinkEndChild(PathElement);
    PathElement->SetAttribute("ID",path);
    PathPtr currentPath = m_paths.at(path);
    for(int point = 0; point < currentPath->size();++point){
        TiXmlElement* PointElement = new TiXmlElement("Point");
	PathElement->LinkEndChild(PointElement);
	PointElement->SetAttribute("X", currentPath->at(point).pos.m_x );
	PointElement->SetAttribute("Y", currentPath->at(point).pos.m_y );
    }
  }

  doc.SaveFile(filepath.c_str());
  DebugStream << "File saved"<<std::endl;

}

void PathManager::load_path_database(std::string filepath)
{
    TiXmlDocument doc(filepath.c_str());
    if(!doc.LoadFile()) throw PathLoadException("Unable to open file");
    DebugStream << "Path file found"<<std::endl;
    TiXmlHandle docHandle( &doc );
    TiXmlElement* PathsElement = docHandle.FirstChild( "Paths" ).Element();
    if(!PathsElement)throw PathLoadException("Unable to find paths, please check that is the correct xml file");
    TiXmlElement* path = PathsElement->FirstChildElement("Path");
    m_paths.clear();
    while(path){
      PathPtr p = PathPtr(new Path);
      TiXmlElement* point = path->FirstChildElement("Point");
      while(point){
	       int x,y;
	        point->Attribute("X",&x);
	         point->Attribute("Y",&y);
	          p->add_point(x,y);
	           point = point->NextSiblingElement("Point");
      }
      m_paths.push_back(p);
      path = path->NextSiblingElement("Path");
    }

}








/////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

SelectedPointObject::SelectedPointObject():
  m_primarySelection(NULL)
{

}

void SelectedPointObject::add_to_selection(Point* point)
{
  if(point!=NULL){
      m_pointSet.insert(point);
      point->selected = true;
  }
}

SelectedPointObject::~SelectedPointObject()
{

}


void SelectedPointObject::set_primary_selection(Point* point)
{
   if(point!=NULL){
      clear_selection();
      m_primarySelection = point;
      m_pointSet.insert(point);
      point->selected = true;
  }
}

void SelectedPointObject::clear_primary(){
    m_primarySelection = NULL;
}

void SelectedPointObject::clear_selection(){
     for(PointSet::iterator it = begin();it!=end();++it){
      (*it)->selected = false;
  }
  m_pointSet.clear();
}

void SelectedPointObject::remove_selection(Point* point){
   if(point!=NULL){
      point->selected = false;
      m_pointSet.erase(point);
      if(point == m_primarySelection){
	  m_primarySelection = NULL;
      }
  }
}

SelectedPointObject::PointSet& SelectedPointObject::get_node_set(){
  return m_pointSet;
}

Point* SelectedPointObject::get_primary(){
  return m_primarySelection;
}

SelectedPointObject::PointSet::iterator SelectedPointObject::begin(){
  return m_pointSet.begin();
}
SelectedPointObject::PointSet::iterator SelectedPointObject::end(){
  return m_pointSet.end();
}

bool SelectedPointObject::empty(){
  return m_pointSet.empty();
}
int SelectedPointObject::selection_size(){
  return m_pointSet.size();
}

SelectedPointObject& PathManager::get_selected_point_objects()
{
  return m_selectedPointObject;
}

