#include "PropertyHeader.hpp"
#include "mainwindow.hpp"
#include <QDebug>
#include <QMouseEvent>
#include "TreeNode.hpp"
#include "PropertyModel.hpp"

PropertyHeader::PropertyHeader(Qt::Orientation orientation, QWidget* parent, MainWindow* window): QHeaderView(orientation, parent),
  m_currentSelectionIndex(0),
  m_mainWindow(window),
  m_currentProperty(NULL),
  m_mouseMode(PMM_NORMAL),
  m_dragRange(1000),
  m_mouseClickPos(0,0)
{
  setClickable(true);
  
}




void PropertyHeader::mousePressEvent(QMouseEvent* e)
{
  try{
  m_currentSelectionIndex = logicalIndexAt( e->pos() );
  QString propertyName = model()->headerData(m_currentSelectionIndex, Qt::Vertical).toString();
  int currentNodeIndex = m_mainWindow->get_current_index();
  m_currentProperty = &TreeNode::get_treenode_by_id(currentNodeIndex).lock()->get_property(propertyName.toStdString());
  m_mouseClickPos = Vec2(e->pos().x(),e->pos().y());

  
    qDebug()<< "PropertyHeader pressed logical "  << m_currentSelectionIndex;
    qDebug()<< "Current node = "<<currentNodeIndex;
    qDebug()<< "Property name " << propertyName;
    
    m_mouseMode = PMM_CLICKED;
  }catch(...){
    m_currentProperty = NULL;
  }
  
 // QHeaderView::mousePressEvent(e);
  
}
void PropertyHeader::mouseMoveEvent(QMouseEvent* e)
{
//   qDebug()<< "PropertyHeader move " << e->pos();
   Vec2 newVal(0,0);
   Vec2 mpos(e->pos().x(),e->pos().y());
   float multiplier = 1;
   if(e->modifiers() == Qt::ShiftModifier)multiplier = 0.1;
   bool sendData = false;
  switch(m_mouseMode){
    case(PMM_NORMAL):{
      break;
    }
    case(PMM_CLICKED):{
	   m_mouseMode = PMM_DRAGGING;
	   break;
    }
    case(PMM_DRAGGING):{
      newVal = mpos - m_mouseClickPos;
      qDebug()<< "delta" << (newVal).m_x; 
      sendData = true;
      break;
    }
  };
  //this needs to take into account ranges!!!!
  if(sendData){
    //  qDebug()<<"sending";
      const std::type_info& ctype = m_currentProperty->get_value().type();
      if(ctype == typeid(Vec2) ){
	         m_currentProperty->set_value(newVal * multiplier  );
      }else if(ctype == typeid(float) ){
	         m_currentProperty->set_value(newVal.m_x * 0.1f *multiplier);
      }else if(ctype == typeid(int) ){
           m_currentProperty->set_value(newVal.m_x *multiplier);
      }
    //  emit dataChanged(QModelIndex( ),QModelIndex());
      dynamic_cast<PropertyModel*>(model())->update();
     // TreeNode::get_treenode_by_id(currentNodeIndex).lock()->set_property(propertyName.toStdString(),PropertyValue(newVal) );
    //  emit dataChanged();
      //&TreeNode::get_treenode_by_id(currentNodeIndex).lock()->get_property(propertyName.toStdString());
  }
 // QHeaderView::mouseMoveEvent(e);
}
void PropertyHeader::mouseReleaseEvent(QMouseEvent* e)
{
//   qDebug()<< "PropertyHeader released " << e->pos();
  m_mouseClickPos = Vec2(0,0);
  m_mouseMode = PMM_NORMAL;
  QHeaderView::mouseReleaseEvent(e);
}
