#include "PropertyModel.hpp"
#include "TreeNode.hpp"
#include <sstream>
#include <iostream>
#include "Utilities.hpp"
#include "Surface.hpp"

PropertyModel::PropertyModel(QObject* parent,int nodeId): 
  QAbstractTableModel(parent),
  m_nodeId(nodeId),
  m_node(TreeNode::get_treenode_by_id(nodeId))
{
  
}	
int PropertyModel::columnCount(const QModelIndex& parent) const
{
  return 2;
}
int PropertyModel::rowCount(const QModelIndex& parent) const
{
  
  try{
    TreeNode::Ptr node = TreeNode::get_treenode_by_id(m_nodeId).lock();
    return node->get_property_count();
   
  }catch(TreeNode::IdException& e){
      return 0;
  }
}
QVariant PropertyModel::data(const QModelIndex& index, int role) const
{
  if (!index.isValid()) return QVariant();
  if (role != Qt::DisplayRole) return QVariant();
  if(!m_node.lock())return QVariant();
  switch(index.column()){
     case(0): 
      {
	std::stringstream ss;
	ss << m_node.lock()->get_property_at(index.row())->second.get_value();
	return QString::fromStdString( ss.str() );
      }
     case(1):
     {
      std::stringstream ss;
      ss << m_node.lock()->get_property_at(index.row())->second.get_osc_path();
      return QString::fromStdString( ss.str() );
     }
  }
  return QVariant();
}

QVariant PropertyModel::headerData(int section, Qt::Orientation orientation, int role) const
{
   if (orientation == Qt::Horizontal && role == Qt::DisplayRole){
      switch(section){
	case 0: return QString("Value");
	case 1: return QString("OSC Name");
      }
    }else if (orientation == Qt::Vertical && role == Qt::DisplayRole){
	if(!m_node.lock())return QVariant();
	return  QString::fromStdString(  m_node.lock()->get_property_at(section)->first) ;	
    }
     return QVariant();
}



Qt::ItemFlags PropertyModel::flags(const QModelIndex& index) const
{
    
  if (!index.isValid())
         return 0;

return  Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable;
}


bool PropertyModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
  try{
      if (index.isValid() && role == Qt::EditRole &&  index.column() ==0) {

        std::string str =  value.toString().toStdString();
	DebugStream << "New value "<<str  <<std::endl;
	std::istringstream iss;
	iss.str(str);
	if(str.empty())return false;
	if(!m_node.lock())return false;
	PropertyMap::const_iterator it =  m_node.lock()->get_property_at(index.row());
	const std::type_info& t=it->second.get_value().type();
	if( t == typeid(float) ){
	  float f;
	  iss >> f;
	  m_node.lock()->set_property(it->first, PropertyValue(f));
	} else if( t == typeid(int) ){
	  int i;
	  iss >> i;
	   m_node.lock()->set_property(it->first, PropertyValue(i));
	} else if(t == typeid(std::string) ){
	    m_node.lock()->set_property(it->first,PropertyValue(str));
	}else if(t == typeid(Vec2) ){
	    Vec2 t;
	    iss >> t.m_x;
	    iss >> t.m_y;
	    m_node.lock()->set_property(it->first,PropertyValue(t));
	}else if(t == typeid(RGBA)){
	    RGBA col;
	    int val = 0;
	    iss >> val;
	    col.set_R(val);
	    val =0;
	    iss >> val;
	    col.set_G(val);
	    val =0;
	    iss >> val;
	    col.set_B(val);
	    val =0;
	    iss >> val;
	    col.set_A(val);
// 	    m_node.lock()->set_property(it->first,PropertyValue(col));
	}else{
	    return false;
	}
	
	
         emit dataChanged(index, index);
         return true;
     }else if(index.isValid() && role == Qt::EditRole &&  index.column() ==1){
	PropertyMap::const_iterator it =  m_node.lock()->get_property_at(index.row());
	std::string str =  value.toString().toStdString();
	DebugStream << "New value "<<str  <<std::endl;
	if(str.empty())return false;
	m_node.lock()->set_property_osc_path(it->first,str);  
	emit dataChanged(index, index);
	return true;
     }
     return false;

  }catch(...){
    return false;
  }
  
//   return QAbstractItemModel::setData(index, value, role);
}

void PropertyModel::update(){

		emit dataChanged(QModelIndex(),QModelIndex());
}
