#include "Surface.hpp"

#include <iostream>
//#include "cv.h"
#include <opencv2/opencv.hpp>
//#include "highgui.h"
#include "opencv2/core/types_c.h"
#include "Utilities.hpp"

Surface::Surface():m_image(NULL)
{
  DebugStream << "Spawning new surface"<<std::endl;
  
}


const unsigned char*const Surface::get_image_buffer()
{
  return (unsigned char*)m_image->imageData;
}
Vec2 Surface::get_size() const
{
  return Vec2(m_image->width,m_image->height);
}
RGBA Surface::get_rgba_at(const Vec2& pos)
{
  if(pos.m_x < 0 )return RGBA();
  if(pos.m_x > m_image->width )return RGBA();
  if(pos.m_y < 0 )return RGBA();
  if(pos.m_y > m_image->height )return RGBA();
  CvScalar s;
  s=cvGet2D(m_image,pos.m_y,pos.m_x);
  return RGBA( s.val[2],s.val[1],s.val[0],s.val[3]  );
}
void Surface::set_rgba_at(const Vec2& pos,  RGBA& col)
{
  if(pos.m_x < 0 )return ;
  if(pos.m_x > m_image->width )return;
  if(pos.m_y < 0 )return ;
  if(pos.m_y > m_image->height )return;
  CvScalar s;
  if(m_image->nChannels == 4){
      s = cvScalar(col.R(),col.G(),col.B(),col.A()  );
      cvSet2D(m_image,pos.m_y,pos.m_x,s);
  }else{
      s = cvScalar(col.B(),col.G(), col.R() );
      cvSet2D(m_image,pos.m_y,pos.m_x,s);
  }
  
  
}

void Surface::draw_line(const Vec2& posA, const Vec2& posB,  RGBA& col)
{
  if(posA.m_x < 0 )return ;
  if(posA.m_x > m_image->width )return;
  if(posA.m_y < 0 )return ;
  if(posA.m_y > m_image->height )return;
  if(posB.m_x < 0 )return ;
  if(posB.m_x > m_image->width )return;
  if(posB.m_y < 0 )return ;
  if(posB.m_y > m_image->height )return;
  CvScalar s;
  if(m_image->nChannels == 4){
      s = cvScalar(col.R(),col.G(),col.B(),col.A()  );
  }else{
      s = cvScalar(col.B(),col.G(), col.R() );
  }
  cvLine(m_image,cvPoint(posA.m_x,posA.m_y ),cvPoint(posB.m_x,posB.m_y),s);
  
  
}


void Surface::draw_rect(const Vec2& posA, const Vec2& posB,  RGBA& col, int thickness)
{
  if(posA.m_x < 0 )return ;
  if(posA.m_x > m_image->width )return;
  if(posA.m_y < 0 )return ;
  if(posA.m_y > m_image->height )return;
  if(posB.m_x < 0 )return ;
  if(posB.m_x > m_image->width )return;
  if(posB.m_y < 0 )return ;
  if(posB.m_y > m_image->height )return;
  CvScalar s;
  if(m_image->nChannels == 4){
      s = cvScalar(col.R(),col.G(),col.B(),col.A()  );
  }else{
      s = cvScalar(col.B(),col.G(), col.R() );
  }
  cvRectangle(m_image,cvPoint(posA.m_x,posA.m_y ),cvPoint(posB.m_x,posB.m_y),s,thickness);
  
  
}



void Surface::draw_circle(const Vec2& posA, int radius,  RGBA& col, int thickness)
{
   if(posA.m_x < 0 )return ;
  if(posA.m_x > m_image->width )return;
  if(posA.m_y < 0 )return ;
  if(posA.m_y > m_image->height )return;
  CvScalar s;
  if(m_image->nChannels == 4){
      s = cvScalar(col.R(),col.G(),col.B(),col.A()  );
  }else{
      s = cvScalar(col.B(),col.G(), col.R() );
  }
  cvCircle(m_image,cvPoint(posA.m_x,posA.m_y ),radius,s,thickness);
  
}





void Surface::set_image(IplImage* image)
{
    if(!image) throw Surface::Exception();
    m_image = image;
//     DebugStream << "Setting surface image"<<std::endl;

}

Surface::SurfaceType Surface::is_type() const
{
  return ST_SURFACE;
}
 

Surface::~Surface()
{
  if(m_image){
    cvReleaseImage(&m_image);
  }
}

void Surface::save_surface(std::string path)
{
  if(!path.empty()){
    std::vector<int> compression_params;
    compression_params.push_back(CV_IMWRITE_PNG_COMPRESSION);
    compression_params.push_back(0);
    cv::imwrite(path,cv::Mat( m_image),compression_params); 
    
  }
    
}
void Surface::set_name(std::string name)
{
  m_name = name;
}


std::string Surface::get_name()
{
  return m_name;
}




