#ifndef SURFACE_HPP
#define SURFACE_HPP

#include "Vec2.hpp"


#include <string>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <opencv2/core/types_c.h>


class RGBA
{
float v[4];
public:
   RGBA() {
v[0]=v[1]=v[2]=v[3]=0;
}
RGBA(float r,float g,float b,float a=1){
v[0]=r;
v[1]=g;
v[2]=b;
v[3]=a;
}
float& R(){return v[0];}
float& G(){return v[1];}
float& B(){return v[2];}
float& A(){return v[3];}
float* get_array(){return v;}
void set_R(float R){
v[0] = R;
}
void set_G(float G){
v[1] = G;
}
void set_B(float B){
v[2] = B;
}
void set_A(float A){
v[3] = A;
}
RGBA operator + (const float& r)const{
      return RGBA(v[0]+r,v[1]+r,v[2]+r,v[3]+r);
}
RGBA operator - (const float& r)const{
      return RGBA(v[0]-r,v[1]-r,v[2]-r,v[3]-r);
}
RGBA operator * (const float& r)const{
      return RGBA(v[0]*r,v[1]*r,v[2]*r,v[3]-r);
}

RGBA operator / (const float& r)const{
      return RGBA(v[0]/r,v[1]/r,v[2]/r,v[3]/r);
}





RGBA operator + (RGBA rhs) {return RGBA(v[0]+rhs.v[0],v[1]+rhs.v[1],v[2]+rhs.v[2],v[3]+rhs.v[3]);}
RGBA operator - (RGBA rhs) {return RGBA(v[0]-rhs.v[0],v[1]-rhs.v[1],v[2]-rhs.v[2],v[3]-rhs.v[3]);}
RGBA operator * (float rhs) {return RGBA(v[0]*rhs,v[1]*rhs,v[2]*rhs,v[3]*rhs);}
};

inline std::ostream& operator << (std::ostream& s, RGBA& op){
	s << "<" << op.R() << "," << op.G() <<"," <<op.B()<< ","<<op.A() <<  ">";
	return s;
}


class Surface
{
  IplImage* m_image;
  std::string m_name;
public:
  enum SurfaceType{
      ST_SURFACE =0,
      ST_STATIC,
      ST_VIDEO,
      ST_STREAM,
      ST_FILLED
  };
  typedef boost::shared_ptr<Surface> Ptr;
  typedef boost::weak_ptr<Surface> WPtr;
  typedef std::vector<Ptr> SurfaceVector;
  class Exception {};
  
  Surface();
  virtual ~Surface();
  const unsigned char* const get_image_buffer(); 
  Vec2 get_size()const;
  RGBA get_rgba_at(const Vec2& pos);
  void set_rgba_at(const Vec2& pos,  RGBA& col);
  void draw_line(const Vec2& posA,const Vec2& posB,RGBA& col);
  void draw_circle(const Vec2& posA, int radius, RGBA& col, int thickness = 1);
  void draw_rect(const Vec2& posA,const Vec2& posB, RGBA& col,int thickness = 1);  
  void save_surface(std::string path);
  virtual SurfaceType is_type()const;
  virtual void tick(float dt)=0;
  void set_name(std::string name);
  std::string get_name();
protected:
  void set_image(IplImage* image);
};

#endif // SURFACE_HPP
