#include "TreeNode.hpp"
#include <iostream>
#include <cassert>
#include <algorithm>
#include "WorldEngine.hpp"
#include "WorldAgent.hpp"
#include <stdexcept>
#include "LinearMotionModel.hpp"
#include "GridManager.hpp"

#include <sstream>
//watch out
#include "Agent.hpp"


int TreeNode::g_id = 0;
TreeNode::TreeNodeMap TreeNode::g_nodeMap;
const int NULLSCNODE = -1;
Trigger TreeNode::s_constructionTrigger = NULL;
Trigger TreeNode::s_destructionTrigger = NULL;


TreeNode::Ptr TreeNode::factory(TreeNode::TreeNodeType type, Vec2 pos, const char* brain)
{
  TreeNode::Ptr t;
  switch(type){
    case(TN_TREE):
	t = ( Ptr(  new TreeNode(pos)));
	break;
    case(TN_AGENT):  
	t = ( Ptr(  new Agent(pos,brain)));
	break;
    case(TN_WORLD):
	t = (Ptr(new WorldAgent(pos)));
	break;
    default:
	assert(false && "Wrong enum provided for factory");
	break;
  }
  assert(t && "Could not create TreeNode ptr in factory");
//   DebugStream << "TreeNode::factory created with id "<<t->get_id()<<std::endl;
  g_nodeMap.insert( std::make_pair( t->get_id(), t ) ); 
  return t;
}

TreeNode::TreeNode(Vec2 pos):
  m_isAlive(true),
  m_id(g_id++),
  m_lifeTime(0),
  m_activeTime(0),
  m_motionModel(MotionModel::factory(MotionModel::MT_STATIC,this)),
  m_scId(NULLSCNODE),
  m_isSelected(false),
  m_hasChildren(false),
  m_active(true)
{
  m_parent.reset();
//   DebugStream << "Tree Node spawned "<< m_id<< std::endl;
  //TODO get motion model to check whether this exists, if not create....
  add_property("Title",  Property(  std::string("TreeNode") )             );
  add_property("MotionModel",  Property(  std::string("Static") )             );

  add_property("SynthDef",Property(std::string("")));
  add_property("Velocity",Property(  Vec2(0,0)));
  add_property("Visibility",Property(10.0f));
  add_property("Mass",Property(0.0f));
  add_property("Target",Property(0));
  add_property("pos", Property( pos,Property::PF_READ | Property::PF_WRITE | Property::PF_SEND ));
  add_property("Thrust",Property(Vec2(0 ,0 )));
  add_property("Damping",Property(1.0f));
  add_property("HasDuration",Property(0));
  add_property("dur",Property(0.0f));
  add_property("alive",Property(0.0f));
  add_property("image",Property(0.0f));
  add_property("attraction",Property(0.0f));
  add_property("grid",Property(Vec2(0,0)));
  add_property("size",Property(Vec2(20,20)));
  //index for the path to use
  add_property("path",Property(0));
  //index for the currently active segment
  add_property("segment",Property(0));
  //relative segpos
  add_property("segpos",Property(0.0f));
//   set_property_osc_path("Title","osctest");
  add_property("pathLock",Property(0));
  add_property("pspeed",Property(1.0f));
  

   if(s_constructionTrigger)s_constructionTrigger(m_id);
 

}

void TreeNode::copy_properties_from(const TreeNode& node)
{
  m_propertyMap.clear();
  m_propertyMap.insert(node.m_propertyMap.begin(),node.m_propertyMap.end());
  Vec2 t = boost::get<Vec2>( get_property("Velocity").get_value());
  DebugStream << t <<std::endl;
}





void TreeNode::initialize_sc_connection()
{
    kill_sc_node();
    m_scId = m_id + SC_OFFSET; 
//     DebugStream << "Initializing sc "<< m_scId << std::endl;
//     DebugStream << "SC spawn id = "<< id <<std::endl;
    std::string name =boost::get<std::string>(get_property("SynthDef").get_value());
//     DebugStream << "SC synth name = " << name<<std::endl;
    lo_address t = WorldEngine::get_instance().get_lo_address();
    
    //lo_send(t,"/s_new","siiisi",name.c_str(),m_scId,1,1,"freq",440);
    lo_message msg = lo_message_new();
    lo_message_add_string(msg,name.c_str());
    lo_message_add_int32(msg,m_scId);
    lo_message_add_int32(msg,1);
    lo_message_add_int32(msg,1);
    for(PropertyMap::const_iterator it = m_propertyMap.begin(); it != m_propertyMap.end(); ++it){
	const Property& prop = it->second;
	const std::string& pname = it->first;
	if(prop.get_flags() & Property::PF_SEND){
	  if(prop.get_value().type() == typeid(float)  ){
	      lo_message_add_string(msg,pname.c_str());
	      lo_message_add_float(msg, boost::get<float>( prop.get_value() )  );
	    }else if(prop.get_value().type() == typeid(Vec2)  ){
	      lo_message_add_string(msg,(pname+"_x").c_str());
	      lo_message_add_float(msg, boost::get<Vec2>( prop.get_value() ).m_x  );
	      lo_message_add_string(msg,(pname+"_y").c_str());
	      lo_message_add_float(msg, boost::get<Vec2>( prop.get_value() ).m_y  );
	    }
	}
    }
    lo_send_message(t,"/s_new",msg);
    lo_message_free(msg);
    
    
    
  
  
}

void TreeNode::kill_sc_node()
{
  if(m_scId !=  NULLSCNODE){
      lo_address t = WorldEngine::get_instance().get_lo_address();
      lo_send(t,"/n_free","i",m_scId);
      m_scId = NULLSCNODE;
  }
}




TreeNode::~TreeNode()
{
  if(s_destructionTrigger)s_destructionTrigger(m_id);

  kill_sc_node();
  g_nodeMap.erase(m_id);
//    DebugStream << "TreeNode destroyed : "<< m_id<<std::endl;
}

const Property& TreeNode::get_property(const std::string& name)const
{
      try{
	return m_propertyMap.at(name);
      }catch(std::out_of_range& e){
	       std::string msg("Could not find const property '");
      	msg+=name+"'";
      	throw Property::Exception(msg.c_str());
      }
}
Property& TreeNode::get_property(const std::string& name)
{
      try{
	return m_propertyMap.at(name);
    }catch(std::out_of_range& e){
	std::string msg("Could not find property '");
	msg+=name+"'";
	throw Property::Exception(msg.c_str());
    }
       
}

void TreeNode::add_property(const std::string& name, const Property& value)
{
    assert(!name.empty() && "Property name empty" ); 
    bool ok = m_propertyMap.insert( std::make_pair(name,value) ).second;
    if(!ok){
//	DebugStream << "WARNING: property "<< name << " already exists - setting instead"<<std::endl; 
	set_property(name,value.get_value());
    }

    
}

bool TreeNode::property_exists(const std::string& name)
{
  if(m_propertyMap.find(name)==m_propertyMap.end())return false;
  return true;
}



void TreeNode::set_property(const std::string& name, const PropertyValue& value)
{
  
   if(m_propertyMap.find(name)== m_propertyMap.end()){
  //    DebugStream << "WARNING: Property "<< name << " not found in node " << m_id << std::endl;
      return;
   }
  
   Property& prop = m_propertyMap.at(name);
   
   prop.set_value(value); 
   if(name == "SynthDef"){
      initialize_sc_connection();
   }else if(name == "dur"){
      set_property("HasDuration",PropertyValue( 1) );
   }else if(name == "MotionModel"){
      MotionModel::MotionType type = m_motionModel->is_type();
      try{
	std::string val =boost::get<std::string>(prop.get_value()); 
	if(val == "static" || val == "Static" ){
	    set_motion_model(MotionModel::MT_STATIC);
	}else if(val == "linear" || val =="Linear"){
	  set_motion_model(MotionModel::MT_LINEAR);
	}else if(val == "physics" || val == "Physics"){
	  set_motion_model(MotionModel::MT_PHYSICS);
	  set_property("Velocity",Vec2(1,1));
	}else if(val == "path" || val =="Path"){
	  set_motion_model(MotionModel::MT_PATH);
	}else{
	  throw "Error ";
	}
     }catch(...){
	switch(m_motionModel->is_type()){
	  case  MotionModel::MT_STATIC:
	    prop.set_value("static");
	    break;
	  case  MotionModel::MT_LINEAR:
	    prop.set_value("linear");
	    break;
	  case  MotionModel::MT_PHYSICS:
	    prop.set_value("physics");
	    break;
	  case  MotionModel::MT_PATH:
	    prop.set_value("path");
	    break;
	};
    }
   }
   
   
   if(m_scId != NULLSCNODE    &&  (prop.get_flags() & Property::PF_SEND)  && prop.get_value().type() == typeid(float) ){
      lo_address t = WorldEngine::get_instance().get_lo_address();
      if(!prop.get_osc_path().empty()){
	  lo_send(t,"/n_set","isf",m_scId,prop.get_osc_path().c_str(), boost::get<float>(prop.get_value())      );
      }else{
	  lo_send(t,"/n_set","isf",m_scId,name.c_str(),    boost::get<float>(prop.get_value())      );
      }
   }
   if(m_scId != NULLSCNODE    &&  (prop.get_flags() & Property::PF_SEND)  && prop.get_value().type() == typeid(Vec2) ){
      lo_address t = WorldEngine::get_instance().get_lo_address();
     if(!prop.get_osc_path().empty()){
	  std::istringstream iss;
	  iss.str(prop.get_osc_path());
	  std::string partA,partB;
	  iss >> partA;
	  iss >> partB;
	  if(partB.empty()){
	      lo_send(t,"/n_set","isf",m_scId,(prop.get_osc_path()+"_x").c_str(),    boost::get<Vec2>(prop.get_value()).m_x      );
	      lo_send(t,"/n_set","isf",m_scId,(prop.get_osc_path()+"_y").c_str(),    boost::get<Vec2>(prop.get_value()).m_y      );
	  }else{
	      lo_send(t,"/n_set","isf",m_scId,partA.c_str(),    boost::get<Vec2>(prop.get_value()).m_x      );
	      lo_send(t,"/n_set","isf",m_scId,partB.c_str(),    boost::get<Vec2>(prop.get_value()).m_y      );
	  }
      }else{
	  lo_send(t,"/n_set","isf",m_scId,(name+"_x").c_str(),    boost::get<Vec2>(prop.get_value()).m_x      );
	  lo_send(t,"/n_set","isf",m_scId,(name+"_y").c_str(),    boost::get<Vec2>(prop.get_value()).m_y      );
      }
   }
}

void TreeNode::set_property_osc_path(const std::string& name, const std::string& path)
{
  Property& prop = m_propertyMap.at(name);
  prop.set_osc_path(path);
  int flgs = prop.get_flags() | Property::PF_SEND;
  prop.set_flags(flgs);
  set_property(name,prop.get_value());
}



const int TreeNode::get_property_count() const
{
  return m_propertyMap.size();
}
std::map< std::string, Property >::const_iterator TreeNode::get_property_at(int index) const
{
  //warning this is slow as hell
  assert(index < m_propertyMap.size() && index >= 0     && "requested property index out of range" );
  PropertyMap::const_iterator it = m_propertyMap.begin();
  int n = 0;
  while(n < index ) {
    ++it;
    ++n;
  }
  return it;
}
std::map< std::string, Property >::iterator TreeNode::get_property_at(int index)
{
  //warning this is slow as hell
  assert(index < m_propertyMap.size() && index >= 0     && "requested property index out of range" );
  PropertyMap::iterator it = m_propertyMap.begin();
  int n = 0;
  while(n < index ) {
    ++it;
    ++n;
  }
  return it;
}






void TreeNode::print_properties(std::ostream& stream) const
{
    for(PropertyMap::const_iterator it = m_propertyMap.begin(); it != m_propertyMap.end();++it){
	stream << it->first << "\t" <<it->second.get_value()<<"\n"; 
    }
  
  
}

int TreeNode::get_id() const
{
  return m_id;
}


TreeNode::Ptr TreeNode::add_child(TreeNode::Ptr child)
{
    assert(child && "Child not valid, could not add to node");
    child->m_parent = shared_from_this();
    m_pendingChildren.push_back(child);
    m_hasChildren = true;
    return child;
}
void TreeNode::update_pending_children()
{
    m_children.insert(m_children.end(),m_pendingChildren.begin(),m_pendingChildren.end());
    m_pendingChildren.clear();
}

TreeNode& TreeNode::get_child_at(int index)
{
  
  try{
    return *m_children.at(index);
  }catch(std::out_of_range& e){
      throw ChildIdException(index);
  }
  
}
const TreeNode& TreeNode::get_child_at(int index) const
{
  try{
    return *m_children.at(index);
  }catch(std::out_of_range& e){
      throw ChildIdException(index);
  }
}
TreeNode::PtrVector::iterator TreeNode::get_child_begin()
{
  return m_children.begin();
}
TreeNode::PtrVector::const_iterator TreeNode::get_child_begin() const
{
  return m_children.begin();
}
TreeNode::PtrVector::iterator TreeNode::get_child_end()
{
  return m_children.end();
}
TreeNode::PtrVector::const_iterator TreeNode::get_child_end() const
{
  return m_children.end();
}
const int TreeNode::number_of_children() const
{
  return m_children.size();
}
void TreeNode::remove_child_at(int index)
{
  assert(index < m_children.size() && "child index not valid for removal");
  assert(index >= 0 && "child index not valid for removal");

  m_children.erase(m_children.begin()+index) ; 
  if(m_children.empty())m_hasChildren = false;
}

TreeNode::WPtr TreeNode::get_parent()
{
  return m_parent;
}
const TreeNode::WPtr TreeNode::get_parent() const
{
  return m_parent;
}

int TreeNode::get_parent_id()
{
  return m_parent.lock()->get_id();
}


void TreeNode::kill_children()
{
  
  for(PtrVector::iterator it = get_child_begin(); it != get_child_end();++it){
    assert(*it  && "Kill Children, NULL child ptr" );
      (*it)->kill();
  }
  m_hasChildren = false;
}


bool TreeNode::has_children()
{
  return m_hasChildren;
}



void TreeNode::show_node_hierarchy(std::ostream& stream, const TreeNode::Ptr& node, int depth)
{
  for(int n = 0; n < depth; ++n) stream << "  ";
     stream << "|-Node id: "<<node->get_id()<<"\n";
     for(TreeNode::PtrVector::const_iterator it =node->get_child_begin(); it != node->get_child_end();++it ){
	show_node_hierarchy(stream,*it, depth + 1);
     }
}

TreeNode::WPtr TreeNode::get_treenode_by_id(int id)
{
//     DebugStream << "Requesting treenode "<< id <<std::endl;
    try{
	return g_nodeMap.at(id);  
    }catch(std::out_of_range& e){
	throw TreeNode::IdException(id);
    }
  
    
}
bool TreeNode::is_alive() const
{
//   DebugStream << "Treenode "<< m_id <<" alive: " <<m_isAlive<<std::endl;  
  return m_isAlive;
}
void TreeNode::kill()
{
  //DebugStream << "attempting to kill "<< m_id<<std::endl;
  for(PtrVector::iterator it = m_children.begin(); it != m_children.end();++it){
      (*it)->kill();
  }
  m_isAlive = false;
}

bool IsDeadPred(TreeNode::Ptr p){
    return !p->is_alive();
}

void TreeNode::tick(double dT)
{
  //if not active for some reason...
//   if(!m_active)return;
  
  try{
  
    m_lifeTime +=dT;
    if(boost::get<int>( get_property("HasDuration").get_value())  != 0){
	m_activeTime +=dT;
	float dur = boost::get<float>( get_property("dur").get_value());
  //       DebugStream << dur <<"\t"<<m_activeTime<< "\t"<< m_activeTime/dur << std::endl;
	set_property("alive",m_activeTime/dur);
	if(m_activeTime >= dur){
	  kill();
	}
    }
    //should this go before or after the children?
    m_motionModel->tick(dT);
    
  //   compute_grid_attraction(dT);
    if(!m_children.empty()){
      for(PtrVector::iterator it = m_children.begin(); it != m_children.end();++it){
	  (*it)->tick(dT);
      }
    }
    
    m_children.erase( std::remove_if( m_children.begin(), m_children.end(), IsDeadPred ), m_children.end() ); // found it!.. clue == vector not list.
  //   DebugStream << "Exit Tick() : " << m_id << std::endl;
    update_pending_children();
    
  }catch(...){
   DebugStream << "Error in treenode tick - Deactivating"<<std::endl;
//    m_active = false;
    
  }
}

void TreeNode::remove_treenode_from_existence()
{
  g_nodeMap.erase(get_id());
}




void TreeNode::show_node_usage(std::ostream& stream)
{
  stream << "Usage count\n";
  for(TreeNodeMap::iterator it = g_nodeMap.begin(); it != g_nodeMap.end();++it){
      stream << it->first << " "<< it->second.use_count()<< "\n"; 
  }
}
TreeNode::TreeNodeType TreeNode::is_type() const
{
  return TN_TREE;
}
const Vec2& TreeNode::get_pos() const
{
  try{
    return  boost::get<Vec2>( m_propertyMap.at("pos").get_value() ) ;
  }catch(std::out_of_range& e){
	throw Property::Exception("Could not find position property in get");
    
  }
}
void TreeNode::set_pos(Vec2 pos)
{
  try{
    m_propertyMap.at("pos").set_value(pos);
  }catch(std::out_of_range& e){
      throw Property::Exception("Could not find position property in set");
    
  }
    
  
  
}

TreeNode::TreeNodeMap::const_iterator TreeNode::get_global_node_list_begin() 
{
    return g_nodeMap.begin();
}
TreeNode::TreeNodeMap::const_iterator TreeNode::get_global_node_list_end() 
{
  return g_nodeMap.end();
}

float TreeNode::get_lifeTime() const
{
  return m_lifeTime;
}

void TreeNode::set_motion_model(MotionModel::MotionType type)
{
//   Vec2 pos = m_motionModel->get_pos();
  m_motionModel = MotionModel::factory(type,this);
//   DebugStream << "Motion model succesfully changed "<<type<<std::endl;
}
/* DEPRECATED
void TreeNode::compute_grid_attraction(double dT)
{
   
  int vGridNo = boost::get<Vec2>( get_property("grid").get_value()).m_y; 
  Vec2 pos = get_pos();
  float vVal = GridManager::get_instance().get_closest_value(vGridNo,pos.m_y)  ;
  float hVal = pos.m_x  ;
  Vec2 dir = pos - Vec2(hVal,vVal); 
  Vec2 vel =  boost::get<Vec2>(get_property("Velocity").get_value())    ;
  float attraction = boost::get<float>(get_property("attraction").get_value());
  set_property("Velocity", dir.normalise() * attraction *10 );
   set_pos(Vec2( pos.m_x,vVal )  );
}

// dir = (targetPos - mypos):norm()


*/

bool TreeNode::is_selected()
{
  return m_isSelected;
}





////////////////////////


class NearestFunctor{
  public:
    const Vec2 pos;
    int id;
    float max;
    NearestFunctor(const Vec2& inpos) : pos(inpos), id(-1), max(99999999) {}
    void operator()(const std::pair<const int, const TreeNode::WPtr>& w){
      TreeNode::Ptr p = w.second.lock();
      float d = ( p->get_pos()- pos).magnitude();
      //this is a yoink to avoid problem of self
      if( d < max && d !=0){
	max = d;
	
	id = p->get_id();
// 	DebugStream << "max= "<< d<< " id= "<< id<< std::endl;

      }
    }
};

int find_nearest_node(Vec2 pos)
{
  NearestFunctor functor(pos);
  TreeNode::TreeNodeMap::const_iterator it =TreeNode::get_global_node_list_begin() ;
  ++it ;
int id = std::for_each(it  ,TreeNode::get_global_node_list_end(),functor).id;  
  return id;
}

class NearbyFunctor{
public:
  const Vec2 pos;
  float radius;
  
  NearbyFunctor(const Vec2& inpos,float inradius):pos(inpos),radius(inradius){}
  void operator()(const std::pair<const int, const TreeNode::WPtr>& w){
      bool valid = false;
      TreeNode::Ptr p = w.second.lock();
      if(p->get_pos() == pos)return;
      Vec2 otherPos = p->get_pos();
      //http://stackoverflow.com/questions/481144/equation-for-testing-if-a-point-is-inside-a-circle
      valid = std::pow( otherPos.m_x - pos.m_x ,2   ) + std::pow( otherPos.m_y - pos.m_y ,2   ) < std::pow(radius,2);
      if(valid){
	s_nearbyIDVector.push_back(p->get_id());
      }
//       std::cout << pos << " " << " radius "<< radius << " id: "<< p->get_id() << " Agent Pos:" << otherPos << " valid: "<< valid<<std::endl; 
      
      /* stupid old algorithm
      float dx = abs( p->get_pos().m_x - pos.m_x);
      float dy = abs( p->get_pos().m_y - pos.m_y); 
      if(dx <= radius&& dy<=radius  )s_nearbyIDVector.push_back(p->get_id());
      */
  }
    
};

IdVector* get_nearby_ids(Vec2 pos, int radius){
    s_nearbyIDVector.clear();
    NearbyFunctor functor(pos,radius);
    TreeNode::TreeNodeMap::const_iterator it =TreeNode::get_global_node_list_begin() ;
    ++it ;
    std::for_each(it,TreeNode::get_global_node_list_end(),functor);
    return &s_nearbyIDVector;
}


void TreeNode::set_selection_state(bool state)
{
  m_isSelected = state;
}

bool TreeNode::is_within(Vec2 pos)
{
  Vec2 m_pos = get_pos();
  Vec2 size = boost::get<Vec2>(get_property("size").get_value());
  float hx(size.m_x*0.5),hy(size.m_y*0.5);
  if(pos.m_x < m_pos.m_x - hx)return false;
  if(pos.m_x > m_pos.m_x + hx)return false;
  if(pos.m_y < m_pos.m_y - hy)return false;
  if(pos.m_y > m_pos.m_y + hy)return false;
  return true;
  
}

void TreeNode::move(Vec2 v)
{
  Vec2 pos = get_pos();
  pos += v;
  set_pos(pos);
  for(TreeNode::PtrVector::iterator it = m_children.begin();it!=m_children.end();++it){
     (*it)->move(v);
  }
}


void TreeNode::set_active_state(bool state)
{
  m_active = state;
}

bool TreeNode::get_activity_state()
{
  return m_active;
}

void TreeNode::add_error(std::string msg)
{
  m_errors.push_back(msg);
}

void TreeNode::clear_errors()
{
  m_errors.clear();
}

StringVector& TreeNode::get_errors()
{
  return m_errors;
}

std::string TreeNode::get_script_name()
{
  return m_scriptName;
}

void TreeNode::set_script_name(std::string name)
{
  m_scriptName = name;
}


