#include "TreeNodeModel.hpp"
#include <QString>
#include <iostream>
#include "WorldEngine.hpp"

TreeNodeModel::TreeNodeModel(QObject* parent): QAbstractItemModel(parent)
{
 //setHeaderData(1, Qt::Horizontal, "Node");
 //set
}


QModelIndex TreeNodeModel::index(int row, int column, const QModelIndex& parent) const
{
//   DebugStream <<"index called row=" << row << " col=" << column << " int id =" << parent.internalId()   << std::endl;
  if( !parent.isValid() ){
    return createIndex(row, column, (quint32) 0);
  }
  else {
    try{
      TreeNode::Ptr parentPtr = TreeNode::get_treenode_by_id( parent.internalId() ).lock();
      //FIXME what if there are no children???!?!?!?!?
      if(parentPtr->number_of_children() > 0){
	return createIndex(row, column, (quint32) parentPtr->get_child_at( row ).get_id() );
      }else{
	return QModelIndex();
      }
    }catch(TreeNode::IdException& e){
	return QModelIndex();
    }
  }

  return QModelIndex();
}
QModelIndex TreeNodeModel::parent(const QModelIndex& child) const
{
  try{
	  TreeNode::Ptr childPtr = TreeNode::get_treenode_by_id( child.internalId() ).lock();
	if( TreeNode::Ptr parentPtr = childPtr->get_parent().lock()){
	    return createIndex(0, 0, (quint32) parentPtr->get_id() );
	  }return QModelIndex();
  }catch(TreeNode::IdException& e){
	return QModelIndex();
    }
  
}
QVariant TreeNodeModel::data(const QModelIndex& index, int role) const
{
  if (!index.isValid()) return QVariant();
  if (role != Qt::DisplayRole) return QVariant();
  
  
//   DebugStream <<"data called row=" << index.row() << " int id =" << index.internalId()   << std::endl;
  
  try{
  
    TreeNode::Ptr ptr = TreeNode::get_treenode_by_id( index.internalId() ).lock();

    switch(index.column()){
	case 0: return ptr->get_id();
	case 1: return  QString::fromStdString( boost::get<std::string>( ptr->get_property("Title").get_value() ) )  ;
    }
    return QVariant();
  }catch(TreeNode::IdException& e){
	return QVariant();
    }
};

int TreeNodeModel::rowCount(const QModelIndex& parent) const
{ 
  int numChildren = 0;
  if (!parent.isValid()){
     numChildren = WorldEngine::get_instance().get_world()->number_of_children();
  } else {
    try{
      TreeNode::WPtr ptr = TreeNode::get_treenode_by_id( parent.internalId() );
      numChildren = ptr.lock()->number_of_children();
    }catch(TreeNode::IdException& e){
	return 0;
    }
  }

//   DebugStream <<"row count called row=" << parent.row() << " int id =" << parent.internalId()  << " children=" << numChildren << std::endl;
  return numChildren;
}

int TreeNodeModel::columnCount(const QModelIndex& parent) const
{
  return 2;
}


Qt::ItemFlags TreeNodeModel::flags(const QModelIndex& index) const
{
 if (!index.isValid())
         return 0;

return  Qt::ItemIsEnabled | Qt::ItemIsSelectable;

}
QVariant TreeNodeModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole){
      switch(section){
	case 0: return QString("Id");
	case 1: return QString("Agent Type");
      }
    }
     return QVariant();

//   return QVariant(QString("shit"));
  //  return QAbstractItemModel::headerData(section, orientation, role);
}

void TreeNodeModel::update(){
  emit dataChanged(QModelIndex(),QModelIndex());

}

void TreeNodeModel::refresh(int agentID)
{
  QModelIndex indexChanged = createIndex(agentID,0,agentID);
  emit dataChanged(indexChanged,indexChanged);
}


