#ifndef __VEC2_HPP__
#define __VEC2_HPP__

#include <ostream>
#include <cmath>
#include <cassert>
//vec2 class needs the following

/*
scalar addition and subtraction
vector addition and subtraction
scalar multiplication and division
do not have vector multiplication and division
all functions need to operate in either order
unary negation ..... -v1 (use operator overload)
construct from components
equality - equal to 
magnitude - square root of sum of the components
normalization - multiply vector by 1/magnitude
dot product - (x1*x2)+(y1*y2)
cross product- if 3d, perp operator if 2d
*/

struct Vec2{
  float m_x,m_y;
  Vec2():m_x(0),m_y(0){}
  Vec2(float x, float y):
    m_x(x),m_y(y){}
  Vec2 operator + (const Vec2& r)const{
      return Vec2(m_x+r.m_x,m_y+r.m_y);
  }
  Vec2 operator + (const float& r )const{
      return Vec2(m_x +r,m_y +r);
  }
  Vec2 operator - (const Vec2& r)const{
      return Vec2(m_x-r.m_x,m_y-r.m_y);
  }
  Vec2 operator - (const float& r )const{	
      return Vec2(m_x -r,m_y -r);
  }
  Vec2& operator += (const Vec2& r){
		m_x += r.m_x;
		m_y += r.m_y;
		return *this;
  }
  Vec2& operator += (const float& r){
	  m_x += r;
	  m_y += r;
	  return *this;
  }
  Vec2& operator -= (const Vec2& r){
	  m_x -= r.m_x;
	  m_y -= r.m_y;
	  return *this;
  }
  Vec2& operator -= (const float& r){
	    m_x -= r;
	    m_y -= r;
	    return *this;
   }
  Vec2& operator *= (const float& r){
	  m_x *= r;
	  m_y *= r;
	  return *this;
  }
  Vec2& operator /= (const float& r){
	  m_x /= r;
	  m_y /= r;
	  return *this;
  }
  Vec2 operator * (const float& r )const{
      return Vec2(m_x *r,m_y *r);
  }
  Vec2 operator * (const Vec2& r )const{
      return Vec2(m_x *r.m_x,m_y *r.m_y);
  }
  
  
   Vec2 operator / (const float& r )const{
      return Vec2(m_x /r,m_y /r);
  }
  float magnitude()const{
      return (float) sqrt(m_x*m_x+m_y*m_y);
  }
  Vec2 normalise(void)const {
    const float tol = 0.0001f;
    float m = magnitude();
    if (fabs(m)<tol){
      return Vec2();
    }else{
	return *this * (1.0f/m);
    }
  }
  Vec2 operator - (void)const{
    return Vec2(-m_x,-m_y  );
  }
  Vec2 reverse()const{
    return Vec2(-m_x,-m_y  );
  }
  Vec2 perp()const{
   return Vec2( -m_y,m_x  ); 
  }
  float dot(const Vec2& v )const{
    return(m_x*v.m_x+m_y*v.m_y);
  }
  bool operator == (const Vec2& r)const{
	return m_x == r.m_x && m_y == r.m_y; 
  }
};



inline Vec2 operator * (const float& l, const Vec2& r){
	return r*l;
}

inline Vec2 operator + (const float& l, const Vec2& r){
	return r+l;
}
///Dot product
inline float dot (const Vec2& u, const Vec2& v){
	return u.dot(v);
}

inline std::ostream& operator << (std::ostream& s, const Vec2& op){
	s << "<" << op.m_x << "," << op.m_y <<  ">";
	return s;
}
inline bool test_tol(float a,float b){
    return  fabs( a-b )<0.0001;
}
inline bool test_vector_tol(Vec2 a,Vec2 b){
    return test_tol(a.m_x,b.m_x) && test_tol(a.m_y,b.m_y);
}

inline void test_vec2(){
  Vec2 t;
  assert(t + 10 == Vec2(10,10));
  Vec2 t2(10,5);
  Vec2 t3(1,2);
  assert(t3+t2 == Vec2(11,7));
  assert(t2*2 == Vec2(20,10));
  assert(t2/2 == Vec2(5,2.5));
  assert(test_tol(  t2.dot(t3),20 ));
  assert(test_tol(   t2.magnitude() , sqrt(125) ));
  assert( test_vector_tol(t2.normalise() , t2/t2.magnitude()));
}



#endif