#include "WorldEngine.hpp"
#include "LuaFunctions.hpp"
#include <iostream>
#include <cassert>
#include <string>
#include <sstream>
#include "Utilities.hpp"

// const int g_MAXSURFACENUM = 16;

WorldEngine& WorldEngine::get_instance()
{
  static WorldEngine t;
  return t;
}

WorldEngine::WorldEngine() :
  m_world( TreeNode::factory(TreeNode::TN_WORLD,Vec2(0,0))),
  m_L(0),
  m_dT(0),
  m_address(lo_address_new("127.0.0.1","8000")),
  m_surfaces(0),
  m_server(NULL),
  m_loadedSurfaces(0),
  m_currentScreen(0)
{
    
//     m_server.start_server();
    //this enables the system to boot without having any scripts loaded, allows osx build!
    if(m_L != 0)  lua_close(m_L);
    m_L = lua_open();
    luaL_openlibs(m_L);
    LuaScript::register_lua_functions(m_L);
    if ( luaL_loadstring(m_L,"function pause(delayTime)\nlocal t = 0\nwhile t < delayTime do\ncoroutine.yield()\nt = t + getDT()\nend\nend\n\nfunction ai_Emptygrain()\ninitai()\npause(0.1)\ndie()\nend")|| lua_pcall(m_L, 0, 0, 0) ){
      DebugStream << "could not load string"<<std::endl;
      
    }
    find_and_store_ai_functions();
    
    
    s_nearbyIDVector = IdVector();
}
WorldEngine::~WorldEngine()
{
  clear_server();
}

void WorldEngine::kill_server()
{
  if(m_address){
      lo_send(m_address,"/quit","i",0);
  }
}
void WorldEngine::clear_server()
{
  if(m_address){
      lo_send(m_address,"/g_freeAll","i",1);
  }
}

void WorldEngine::read_buffer_command(int bufferNumber, std::string path)
{
  if(m_address && !path.empty()){
      lo_send(m_address,"/b_allocRead","is",bufferNumber,path.c_str());
  }
}


void WorldEngine::load_supercollider_synths(std::string folderPath)
{
  if(folderPath.empty())return;
  if(m_address){
    lo_send(m_address,"/d_loadDir","s",folderPath.c_str());
  }
}




TreeNode::Ptr WorldEngine::get_world()
{
    return m_world;
}

lua_State* WorldEngine::get_lua_state()
{
  return m_L;
}
double WorldEngine::get_delta() const
{
  return m_dT;
}
int WorldEngine::get_current_agent_id() const
{
  return m_activeId;
}

void WorldEngine::set_current_agent_id(const int id)
{
  m_activeId = id;
}
void WorldEngine::load_lua_script(const char* filename, bool autoLoad)
{
  if(m_L != 0)  lua_close(m_L);
  m_L = lua_open();
  luaL_openlibs(m_L);
  LuaScript::register_lua_functions(m_L);
  if (luaL_loadfile(m_L, filename) || lua_pcall(m_L, 0, 0, 0)){
		std::stringstream ss;
		ss << "script error "<< lua_tostring(m_L,-1);
		throw LuaScript::Exception(ss.str().c_str());
	}else{
		DebugStream << "Lua script loaded successfully"<<std::endl;
		m_pathToScript = filename;
		ScriptData t(m_pathToScript,autoLoad);
		m_loadedScripts.insert(t);
	}
  find_and_store_ai_functions();
}

void WorldEngine::append_lua_script(const char* filename, bool autoLoad)
{
  if(m_L){
//     luaL_openlibs(m_L);
//     LuaScript::register_lua_functions(m_L);
    if (luaL_loadfile(m_L, filename) || lua_pcall(m_L, 0, 0, 0)){
		std::stringstream ss;
		ss << "script error "<< lua_tostring(m_L,-1);
		throw LuaScript::Exception(ss.str().c_str());
	}else{
		DebugStream << "Lua script appended successfully"<<std::endl;
		int exists = false;
		for(ScriptSet::iterator it = m_loadedScripts.begin();it != m_loadedScripts.end();++it){
		     if((*it).path == filename){
			exists = true;
		     }
		}
		    if(!exists){
		      DebugStream << "Could not find script in existing vector" <<std::endl;
		      m_loadedScripts.insert( ScriptData(  std::string(filename),autoLoad));
		    }
	}
  find_and_store_ai_functions();
  }
}



void WorldEngine::tick()
{
  assert(m_world && "Invalid world ptr");
  m_dT = 0.033;
  m_world->tick(m_dT);
}

lo_address WorldEngine::get_lo_address()
{
    return m_address;
}
void WorldEngine::set_lo_address(const char* host, const char* port)
{
   if( m_address)lo_address_free(m_address); 
   m_address =  lo_address_new(host, port);
   assert(m_address);
   DebugStream << "Succesfully created lo address: "<< host << " port: " << port<<std::endl;
}

void WorldEngine::list_lua_function_names()
{
  DebugStream << "Found the following lua function names:"<<std::endl;
  //lua_gettable (m_L, LUA_GLOBALSINDEX);
  lua_getglobal(m_L,"_G");
  assert( lua_istable(m_L, -1 ) && "Lua global index is not a table");
  for (lua_pushnil(m_L); lua_next(m_L, -2); lua_pop(m_L, 1))
  {
    std::string t;
    if(lua_isstring(m_L,-2) && lua_isfunction(m_L,-1)){
	t = lua_tostring(m_L,-2);
	if(t.find("ai_")!=std::string::npos){
	  DebugStream << "\t|"<< lua_tostring(m_L,-2)  <<std::endl;  
	}
   /* perform operation */
    }
  }

}

void WorldEngine::find_and_store_ai_functions()
{
   m_aiNames.clear();
    DebugStream << "Found the following lua function names:"<<std::endl;
  //lua_gettable (m_L, LUA_GLOBALSINDEX);
  lua_getglobal(m_L,"_G");
  assert( lua_istable(m_L, -1 ) && "Lua global index is not a table");
  for (lua_pushnil(m_L); lua_next(m_L, -2); lua_pop(m_L, 1))
  {
    std::string t;
    if(lua_isstring(m_L,-2) && lua_isfunction(m_L,-1)){
	t = lua_tostring(m_L,-2);
	if(t.find("ai_")!=std::string::npos){
	   m_aiNames.push_back(t);
	   DebugStream <<   lua_tostring(m_L,-2)  <<std::endl;  
	}
   /* perform operation */
    }
  }
}

std::vector< std::string >::const_iterator WorldEngine::get_aiName_begin() const
{
  return m_aiNames.begin();
}
std::vector< std::string >::const_iterator WorldEngine::get_aiName_end() const
{
  return m_aiNames.end();
}
std::vector< std::string >& WorldEngine::get_ai_name_vector() 
{
  return m_aiNames;
}

void WorldEngine::reset_world_state()
{
    if(m_address){
      lo_send(m_address,"/g_freeAll","i",1);
    }	
//     m_world.reset();
    m_world->kill_children();
    m_world->tick(0.01);


    //TODO this should load all previous
//     load_lua_script(m_pathToScript.c_str(),true);
//     m_world = TreeNode::factory(TreeNode::TN_WORLD,Vec2(0,0)) ;
}

Surface::Ptr WorldEngine::get_surface(int index)
{
  if(m_surfaces.empty())throw std::out_of_range("No surfaces");
  return m_surfaces.at(index);
}
int WorldEngine::add_surface( Surface::Ptr surface)
{
  if(surface){
    m_surfaces.push_back(surface);
  }
  return m_surfaces.size() -1;
}
int WorldEngine::get_number_of_surfaces()
{
  return m_surfaces.size();
}


ScriptSet& WorldEngine::get_loaded_scripts()
{
  return m_loadedScripts;
}


void WorldEngine::delete_surface(int index)
{
  if(m_surfaces.at(index)){
    Surface::SurfaceVector::iterator it = m_surfaces.begin() +index;
    m_surfaces.erase(it);
  }
  
}

void WorldEngine::set_primary_selection(TreeNode* node)
{
  if(node!=NULL){
      m_selectedNodeObject.set_primary_selection(node);
  }
}


void WorldEngine::snap_worldsize(int index)
{
  if(m_surfaces.at(index)){
      Vec2 size = m_surfaces.at(index)->get_size();
      m_world->set_property("WorldSize",size);
  }
  
}




SelectedNodeObject& WorldEngine::get_selections(Vec2 pos, bool clear)
{
    if(clear)m_selectedNodeObject.clear_selection();
   for(TreeNode::TreeNodeMap::const_iterator it = TreeNode::get_global_node_list_begin();it!=TreeNode::get_global_node_list_end();++it){
     TreeNode::Ptr node = (*it).second.lock();
     if( node->is_within(pos) ){
	m_selectedNodeObject.add_to_selection(node.get());
	for(TreeNode::PtrVector::iterator it = node->get_child_begin();it!=node->get_child_end();++it){
	    m_selectedNodeObject.add_to_selection( it->get() );
	}
     }
  }
  return m_selectedNodeObject;
  
}

SelectedNodeObject& WorldEngine::get_selected_object()
{
  return m_selectedNodeObject;
}



/////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////


SelectedNodeObject::SelectedNodeObject():
  m_primarySelection(NULL)
{

}

void SelectedNodeObject::add_to_selection(TreeNode* node)
{
  if(node!=NULL){
      m_nodeSet.insert(node);
      node->set_selection_state(true);
  }
}

void SelectedNodeObject::set_primary_selection(TreeNode* node)
{
  if(node!=NULL){
      clear_selection();
      m_primarySelection = node;
      m_nodeSet.insert(node);
      node->set_selection_state(true);
  }
}
void SelectedNodeObject::clear_primary()
{
  m_primarySelection = NULL;
}

void SelectedNodeObject::clear_selection()
{
  for(NodeSet::iterator it = begin();it!=end();++it){
      (*it)->set_selection_state(false);
  }
  m_nodeSet.clear();
}

void SelectedNodeObject::remove_selection(TreeNode* node)
{
  if(node!=NULL){
      node->set_selection_state(false);
      m_nodeSet.erase(node);
      if(node == m_primarySelection){
	  m_primarySelection = NULL;
      }
    
  }
}


SelectedNodeObject::NodeSet& SelectedNodeObject::get_node_set()
{
  return m_nodeSet;
}

TreeNode* SelectedNodeObject::get_primary()
{
  return m_primarySelection;
}

std::set< TreeNode* >::iterator SelectedNodeObject::begin()
{
  return m_nodeSet.begin();
}

std::set< TreeNode* >::iterator SelectedNodeObject::end()
{
  return m_nodeSet.end();
}
bool SelectedNodeObject::empty()
{
  return m_nodeSet.empty();
}

SelectedNodeObject::~SelectedNodeObject()
{

}
int SelectedNodeObject::selection_size()
{
  return m_nodeSet.size();
}



void WorldEngine::reset_autoload_states()
{
  for(ScriptSet::iterator it = m_loadedScripts.begin();it != m_loadedScripts.end();++it){
    std::string path =(*it).path; 
    DebugStream << path <<std::endl;
    
    unsigned found = path.find_last_of("/\\");
    std::string file = path.substr(found+1);
    if(file != "basic.lua"){
      m_loadedScripts.erase(it);
      ScriptData t(path,false);
      m_loadedScripts.insert(t);
    }
    
  }
  
  
  
}


int WorldEngine::spawn_server(int targetPort)
{
  int port = targetPort;
  try{
    m_server = new WorldServer(this,targetPort);
    m_server->start_server();
  }catch(...){
    port = spawn_server(targetPort +1 );
  }
  
  
  return port;
}

int WorldEngine::get_current_screen()
{
  return m_currentScreen;
}
void WorldEngine::set_current_screen(int id)
{
  m_currentScreen = id;
}

