#ifndef WORLDENGINE_HPP
#define WORLDENGINE_HPP

#include "TreeNode.hpp"
#include "lua/lua.hpp"
#include "lo/lo.h"
#include <vector>
#include "Surface.hpp"
#include "WorldServer.hpp"
#include <set>

class SelectedNodeObject{
public:
  typedef std::set<TreeNode*> NodeSet;
private:
  NodeSet m_nodeSet;
  TreeNode* m_primarySelection;
public:
  SelectedNodeObject();
  virtual ~SelectedNodeObject();
  void add_to_selection(TreeNode* node);
  void remove_selection(TreeNode* node);
  void set_primary_selection(TreeNode* node);
  void clear_selection();
  void clear_primary();
  TreeNode* get_primary();
  NodeSet& get_node_set();
  NodeSet::iterator begin();
  NodeSet::iterator end();
  bool empty();
  int selection_size();
  
};

struct ScriptData{
    std::string path;
    bool autoLoad;
    ScriptData(std::string path, bool autoLoad  ):
      path(path),
      autoLoad(autoLoad)
    {
    }
    void set_load_state(bool state){
      autoLoad = state;
    }
};

inline bool operator<( const ScriptData &s1, const ScriptData &s2 )
{
   return s1.path < s2.path;
}


typedef std::set<ScriptData> ScriptSet;

class WorldEngine 
{
    TreeNode::Ptr m_world;
    WorldEngine();
    int m_activeId;
    lua_State* m_L;
    double m_dT;
    lo_address m_address;
    std::vector<std::string> m_aiNames;
    ScriptSet m_loadedScripts;
    std::string m_pathToScript;
    Surface::SurfaceVector m_surfaces;  
    WorldServer* m_server;
    SelectedNodeObject m_selectedNodeObject;
    int m_loadedSurfaces;
    int m_currentScreen;
public:
    static WorldEngine& get_instance();
    virtual ~WorldEngine();
    //world management
    TreeNode::Ptr get_world();
    
    int spawn_server(int targetPort);
    
    //get lua state 
    lua_State* get_lua_state();
    
    //get delta 
    double get_delta()const;

    //get current agent id
    int get_current_agent_id()const;
    void set_current_agent_id(const int id);
    
    void load_lua_script(const char* filename,bool autoLoad);
    void append_lua_script(const char* filename,bool autoLoad);
    ///FIXME this needs to be handed a delta...
    void tick();
    //for general lo 
    void set_lo_address(const char* host,const char* port);
    lo_address get_lo_address();
    
    //functions to check the state of the lua stack
    void list_lua_function_names();
    void find_and_store_ai_functions();
    
    //functions to provide access to ai name types
    std::vector<std::string>::const_iterator get_aiName_begin()const;
    std::vector<std::string>::const_iterator get_aiName_end()const;
    
    std::vector<std::string>& get_ai_name_vector();
    ScriptSet& get_loaded_scripts();
    //resetting of the world
    void reset_world_state();
    
    //surface vector functions
    int add_surface(Surface::Ptr surface  );
    Surface::Ptr get_surface(int index);
    void delete_surface(int index);
    int get_number_of_surfaces();
    int get_current_screen();
    void set_current_screen(int id);
    
    void set_primary_selection(TreeNode* node);
    
    SelectedNodeObject& get_selections(Vec2 pos,bool clear);
    SelectedNodeObject& get_selected_object();
    
    void load_supercollider_synths(std::string folderPath);
    void kill_server();
    void clear_server();
    void reset_autoload_states();
    
    void read_buffer_command(int bufferNumber,std::string path);
    
    void snap_worldsize(int index);
    
};

#endif 
