#include "WorldServer.hpp"
#include <iostream>
#include "TreeNode.hpp"
#include <boost/algorithm/string.hpp>
#include <sstream>
#include "Utilities.hpp"


WorldServer::WorldServer(WorldEngine* engine,int portNo):
  m_engine(engine)
{
	std::stringstream ss;
	ss << portNo;
	m_serverThread = lo_server_thread_new(ss.str().c_str(),error);
	
//   	lo_server_thread_add_method(m_serverThread, "/test", "", test, NULL);
	lo_server_thread_add_method(m_serverThread, NULL, NULL, data_handler, NULL);

}

WorldServer::~WorldServer()
{
  lo_server_thread_stop(m_serverThread);
}

void WorldServer::start_server()
{
  
  lo_server_thread_start(m_serverThread);
  DebugStream << "World Server started"<<std::endl;
}


void WorldServer::error(int num, const char* m, const char* path)
{
  DebugStream << "World Server error " << path <<std::endl; 
  throw "Server Error";
}


int WorldServer::test(const char* path, const char* types, lo_arg** argv, int argc, void* data, void* user_data)
{
  DebugStream << "World server test message recieved"<<std::endl;
}




int WorldServer::data_handler(const char* path, const char* types, lo_arg** argv, int argc, void* data, void* user_data)
{
  DebugStream << "Recieved data on "<<  path <<std::endl; 
  std::string fullPath(path);
  //unsigned found = fullPath.find_first_of("/");
 // boost::algorithm::starts_with(fullPath.begin(),fullPath.end(),   "/");
  size_t firstBackslash = fullPath.find_first_of("/");
  if(firstBackslash == std::string::npos  ){
//       DebugStream << "No backslash in path provided " <<std::endl;
      return 0;
  }
  size_t secondBackslash  = fullPath.find("/",1);
  if(secondBackslash == std::string::npos){
//       DebugStream << "Could not find backslash after id number"<<std::endl;
      return 0;
  }
  DebugStream << "Second backslash found at "<< secondBackslash<<std::endl;
  int id = atoi( fullPath.substr(firstBackslash+1,secondBackslash-1).c_str());
  DebugStream << "id = "<< id <<std::endl;
  std::string propertyName = fullPath.substr(secondBackslash+1);
  DebugStream << "Property name " << propertyName <<std::endl;
  DebugStream << "argc "<<argc<<std::endl;

  TreeNode::Ptr  node;

  try{
    node = TreeNode::get_treenode_by_id(id).lock();

  }catch(TreeNode::IdException e){
    DebugStream << "Unable to find Node with id "<< id <<std::endl;
    return 0;

  }
  //TODO rework this to use the liblo types . . . maybe write little function to pull data

  if(argc >0){
  
      try{
	std::string t(types);
	DebugStream <<"total "<< argc << "t ypes " << t<<std::endl; 
	
	if(argc > 1){
	    DebugStream << "Vec2 "<<std::endl;
	    //VEC2 stuff here...
	    PropertyValue val =   node->get_property(propertyName).get_value();
	    int x( boost::get<Vec2>(val).m_x);
	    int y(boost::get<Vec2>(val).m_y);
	    if(t.substr(0,1) ==std::string("i")){
	      x = argv[0]->i;
	    }else if(t.substr(0,1) == std::string("f")){
	       x = argv[0]->f;
	       	    DebugStream << x <<std::endl;

	    }
	    if(&t[1] ==std::string("i")){
	      y = argv[1]->i;
	    }else if(&t[1] == std::string("f")){
	      y = argv[1]->f;
	    }
	    DebugStream << x <<std::endl;
	    if(val.type() == typeid(Vec2) ){ 
	      node->set_property(propertyName,Vec2(x,y));
	    }else{
	     node->set_property(propertyName,x); 
	    }
	    
	    
	}else{
	  
	  
	  
	  
	    if(&t[0] ==  std::string("i")){
	      node->set_property(propertyName,argv[0]->i);
	    }else if(&t[0] == std::string("f")){
	      node->set_property(propertyName,argv[0]->f);
	    }else if(&t[0] ==std::string( "s")){
// 	      node->set_property(propertyName,argv[0]->s);
	    }
	  
	  
	  
	}
	
      /*
      try{
	PropertyValue val =   node->get_property(propertyName).get_value();
	if( val.type() == typeid(float) ) {
	    // use first number .......
	    node->set_property(propertyName,  argv[0]->f);

	}else if(val.type() == typeid(int) ){ 
	    node->set_property(propertyName,  argv[0]->i);
	}else if(val.type() == typeid(std::string)   ) {
	    //TODO ....

	}else if (val.type() == typeid(Vec2)){
	    std::string t(types);
	    int x( boost::get<Vec2>(val).m_x);
	    int y(boost::get<Vec2>(val).m_y);
	    if( std::string(&t[0])== std::string("f")){
	      x = argv[0]->f;
	    }else if(std::string( &t[0])=="i"){
	      x = argv[0]->i;
	    }
	    
	    if(argc > 1){
	      if( std::string(&t[1])== std::string("f")){
		y = argv[1]->f;
	      }else if(std::string( &t[1])=="i"){
		y= argv[1]->i;
	      }    

	    }
	    node->set_property(propertyName,Vec2(x,y));


	}else if(val.type() == typeid(bool)){
	    if(argv[0]->i){
	      node->set_property(propertyName,true);
	    }else{
	      node->set_property(propertyName,false);
	    }

	}

      */
      }catch(std::exception& e){
	DebugStream << "Could not find "<< propertyName << "in selected node"<<std::endl;

      }

  }
  
  
}

