#include "WorldView.hpp"
#include <QMessageBox>
#include <QApplication>
#include <QGraphicsSceneMouseEvent>
#include <iostream>
#include "TreeNode.hpp"
#include <QMouseEvent>
#include "WorldEngine.hpp"
// #include "node.hpp"
#include <sstream>
#include "StaticSurface.hpp"
#include "VideoSurface.hpp"
#include "StreamSurface.hpp"
#include "ImageManager.hpp"
#include "GridManager.hpp"
#include "Agent.hpp"
#include "mainwindow.hpp"
#include "LuaFunctions.hpp"
#include <algorithm>
#include <QDebug>
#include "PathManager.hpp"
#include "FilledSurface.hpp"
#include "lua/lobject.h"

WorldView::WorldView(QWidget* parent,MainWindow* mainWind):
      QWidget(parent),
      m_mainWindow(mainWind),
      m_showVisibility(false),
      m_showSpeed(false),
      m_showMass(false),
      m_showTarget(false),
      m_duplicateId(0),
      m_duplicateState(false),
      m_mouseMode(MT_SPAWN),
      m_mouseMoveMode(MMM_NORMAL),
      m_clickLocation(0,0),
      m_showNames(false),
      m_ShowPaths(false),
      m_selectedPathIndex(0),
      m_closestPointPos(-1,-1),
      m_closePointFound(false),
      m_selectedSurfaceIndex(0)
{
//       setBackgroundBrush(QBrush(Qt::black));
	m_selectedBrainString = "ai_BasicPlayer";
// 	setMaximumHeight(1920);
	setMaximumSize(1920,1200);
	setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
}

void WorldView::set_duplicate_id(int id)
{
  m_duplicateId = id;
}

void WorldView::set_duplicate_state(bool state)
{
  m_duplicateState = state;
}



void WorldView::load_image(QString filename)
{

    if(!filename.isEmpty()){
	WorldEngine::get_instance().get_world()->set_property("Image",filename.toStdString());
	m_selectedSurfaceIndex =  WorldEngine::get_instance().add_surface( Surface::Ptr(new StaticSurface(filename.toStdString())  )  );
// 	DebugStream << "Image loaded "<< filename.toAscii()<<std::endl;
	// setBackgroundBrush(QBrush(m_activeImage));
// 	m_selectedSurfaceIndex +=1;
	update();
    }else{
	QMessageBox::critical(QApplication::topLevelWidgets().back(),"Error","Image could not be loaded");
    }
}

void WorldView::load_video(QString filename)
{
    if(!filename.isEmpty()){
	WorldEngine::get_instance().get_world()->set_property("Image",filename.toStdString());
	m_selectedSurfaceIndex =  WorldEngine::get_instance().add_surface(Surface::Ptr(new VideoSurface(filename.toStdString())  )  );
// 	DebugStream << "Image loaded "<< filename.toAscii()<<std::endl;
	// setBackgroundBrush(QBrush(m_activeImage));
// 	m_selectedSurfaceIndex +=1;
	update();
    }else{
	QMessageBox::critical(QApplication::topLevelWidgets().back(),"Error","Image could not be loaded");
    }
}
void WorldView::load_stream()
{
  	WorldEngine::get_instance().get_world()->set_property("Image","stream");
	m_selectedSurfaceIndex =  WorldEngine::get_instance().add_surface( Surface::Ptr(new StreamSurface()  )  );
// 	DebugStream << "Image loaded "<< filename.toAscii()<<std::endl;
	// setBackgroundBrush(QBrush(m_activeImage));
// 	m_selectedSurfaceIndex +=1;
	update();

}

void WorldView::load_filled(QColor col)
{

  	WorldEngine::get_instance().get_world()->set_property("Image","basic background");
	m_selectedSurfaceIndex =  WorldEngine::get_instance().add_surface(Surface::Ptr(new FilledSurface(RGBA(col.red(),col.green(),col.blue(),col.alpha()) )  )  );
// 	DebugStream << "Image loaded "<< filename.toAscii()<<std::endl;
	// setBackgroundBrush(QBrush(m_activeImage));
// 	m_selectedSurfaceIndex +=1;
	update();

}



void WorldView::set_draw_paths_state(bool state)
{
  m_ShowPaths = state;
  update();
}



void WorldView::mousePressEvent ( QMouseEvent * event )
{

    bool playState = m_mainWindow->get_play_state();
    if(playState) m_mainWindow->set_play_state(!playState);
    setCursor(Qt::OpenHandCursor);
    QPoint pos = event->pos();
    m_clickLocation = Vec2(event->x(),event->y());
    if( event->buttons() != Qt::RightButton){
//     DebugStream << "Mouse clicked "<< pos.rx()<<"," <<pos.ry() <<  std::endl;
//     pos -= QPoint(25,25);
    switch(m_mouseMode){
      case(MT_SPAWN):
	add_agent( pos);
	break;
      case(MT_SELECT):{
	select_agent(pos);
	break;
      }
      case(MT_MOVE):
	m_mouseMoveMode = MMM_CLICKED;
// 	move_agent(Vec2);
	break;
      case(MT_DELETE):
	delete_agent(pos);
	break;
      case(MT_PATH):
	m_mouseMoveMode = MMM_CLICKED;
	path_mode_clicked(pos,QApplication::keyboardModifiers() == Qt::ShiftModifier);
	break;
    }
      update();
    }else if(event->buttons() == Qt::RightButton && m_mouseMode == MT_PATH){
	//if the right button was clicked
//       DebugStream << "Right path triggered"<<std::endl;
	 WorldPointInfo info =  get_closest_point(Vec2(event->x(),event->y()));
	 if(info.pathIndex >=0 && info.pointIndex >=0){
	   PathManager::get_instance().get_path(info.pathIndex)->remove_point(info.pointIndex);
	} 
	update();
    }
      if(playState)m_mainWindow->set_play_state(playState);

//     QGraphicsScene::mousePressEvent(event);
     QWidget::mousePressEvent(event);
}

WorldPointInfo WorldView::get_closest_point(Vec2 pos)
{
	 float thresh = 10;
	 int pathId =0;
	 int pointId = 0;
	 Vec2 diff(10000,10000);
	 //for all paths
	 for(int n = 0 ; n < PathManager::get_instance().get_paths_size();++n){
	      PathPtr currentPath = PathManager::get_instance().get_path(n);

	      //for all points
	      for(int i = 0; i < currentPath->size();++i){
		if( (pos - currentPath->at(i).pos).magnitude() < diff.magnitude()   ){
// 		 DebugStream << "closer..."<<std::endl;
		 //set the new diff
		 diff = pos-currentPath->at(i).pos;
// 		 DebugStream << "diff "<< diff <<std::endl;
		 pathId = n;
		 pointId = i;
		}
	      }
// 	      DebugStream << "Closest = " << "path:"<<pathId<< " point:"<< pointId<<std::endl;
	      if(diff.m_x < 5 && diff.m_y < 5 && diff.m_y > -5 && diff.m_y > -5 ){
// 		currentPath->remove_point(pointId);
		return WorldPointInfo(pathId,pointId);
	      }
	      diff = Vec2(10000,10000);
	   
	      
	}
	return WorldPointInfo();

}






void WorldView::mouseMoveEvent(QMouseEvent* event)
{
  
  Vec2 pos(event->x(),event->y());

  switch(m_mouseMoveMode){
    case(MMM_NORMAL):
      break;
    case(MMM_CLICKED):{
      if(m_mouseMode == MT_PATH){
	  m_mouseMoveMode =MMM_PATHDRAGGING;
	  break;
      }
      m_mouseMoveMode = MMM_DRAGGING;
      break;
      }
    case(MMM_DRAGGING):{
//    std::cout << "Click " << m_clickLocation.m_x << " , " << m_clickLocation.m_y <<  " drag "<< pos<<std::endl;
      move_agent(    pos - m_clickLocation);
      m_clickLocation = pos;
      break;
    }
    case(MMM_PATHDRAGGING):{
      move_selected_points(   pos - m_clickLocation);
      m_clickLocation = pos;
      //DebugStream << "Path Motion "<<std::endl;
      break;
    }case(MMM_PATHMOTION):
      std::cout << "pm";
      path_motion(pos);
      break;
  };

  QWidget::mouseMoveEvent(event);
}


void WorldView::mouseReleaseEvent(QMouseEvent* event)
{
  
  m_mouseMoveMode = MMM_NORMAL;
  if(m_mouseMode == MT_PATH) m_mouseMoveMode =MMM_PATHMOTION;
  setCursor(Qt::ClosedHandCursor);
  QWidget::mouseReleaseEvent(event);
}


void WorldView::add_agent(QPoint pos)
{
//   addItem(new Node(this,0,pos));
  try{

    TreeNode::Ptr agent ;
    if(!m_duplicateState){
      agent = TreeNode::factory(TreeNode::TN_AGENT,Vec2( pos.rx(),pos.ry()),m_selectedBrainString.c_str());
      WorldEngine::get_instance().set_primary_selection(agent.get());
//       agent->set_selection_state(true);
    }else{
  //     agent=   TreeNode::get_treenode_by_id(m_duplicateId).lock().get();
  //     assert(agent);
  //     WorldEngine::get_instance().get_world()->add_child(agent  );
	TreeNode::Ptr existing = TreeNode::get_treenode_by_id(m_duplicateId).lock();
	TreeNode::TreeNodeType type =  existing->is_type();

	std::string brain("");
	if(type == TreeNode::TN_AGENT){
	  Agent::Ptr existingAgent =boost::dynamic_pointer_cast<Agent>(existing);
	  brain = existingAgent->get_brain_function_name();
	}
	agent = TreeNode::factory(type,Vec2(0,0),brain.c_str());
	agent->tick(0.0001);
	agent->copy_properties_from(*existing);
	agent->set_pos(Vec2(  pos.rx(),pos.ry()));


    }
  
    WorldEngine::get_instance().get_world()->add_child(agent  );
    bool state = true;
    if(QApplication::keyboardModifiers() == Qt::ShiftModifier)state = false;
    SelectedNodeObject& selections =  WorldEngine::get_instance().get_selections(Vec2(pos.x(),pos.y()),state);
    
    if(!selections.empty()){
    m_mainWindow->selected_node(selections.get_primary()->get_id());
    }

    
    
  }catch(LuaScript::Exception& e){
    QMessageBox::warning(m_mainWindow,"Error",e.what());

  }
}


void WorldView::select_agent(QPoint pos)
{
 qDebug()<< "Selecting agent";
 Vec2 p(pos.rx(),pos.ry());
 bool state = true;
 if(QApplication::keyboardModifiers() == Qt::ShiftModifier)state = false;
 SelectedNodeObject& selections =  WorldEngine::get_instance().get_selections(p,state);
 SelectedNodeObject::NodeSet& nodes = selections.get_node_set();
 for(SelectedNodeObject::NodeSet::iterator it = nodes.begin(); it != nodes.end(); ++it){
    qDebug() << (*it)->get_id();
}
 
 if(!selections.empty()){
  m_mainWindow->selected_node(selections.get_primary()->get_id());
 }

}
void WorldView::move_agent(Vec2 pos)
{
  
//   std::cout << "Moving by delta "<< pos << std::endl;
//   SelectedNodeObject& selections =  WorldEngine::get_instance().get_selections(pos,true);
//   for(std::set<TreeNode*>::iterator it = selections.begin();it!=selections.end();++it){
//       (*it)->move(pos);
//   }
   SelectedNodeObject::NodeSet& nodes = WorldEngine::get_instance().get_selected_object().get_node_set();
   for(SelectedNodeObject::NodeSet::iterator it = nodes.begin(); it != nodes.end() ; ++it){
     (*it)->move(pos);
  }
   
  
  update();
}
void WorldView::delete_agent(QPoint pos)
{
  bool old = m_mainWindow->get_play_state();
  m_mainWindow->set_play_state(false);
  qDebug()<< "Selecting agent";
  Vec2 p(pos.rx(),pos.ry());
  SelectedNodeObject& selections =  WorldEngine::get_instance().get_selections(p,true);
  for(std::set<TreeNode*>::iterator it = selections.begin();it!=selections.end();++it){
      (*it)->kill();
  }
  WorldEngine::get_instance().tick();
  m_mainWindow->set_play_state(old);
}

void WorldView::path_mode_clicked(QPoint pos,bool shiftDown)
{
  Vec2 vPos(pos.rx(),pos.ry());
  WorldPointInfo selected = get_closest_point(vPos);
//   std::cout << selected.pathIndex << " " << selected.pointIndex <<std::endl;
  if(!shiftDown)PathManager::get_instance().get_selected_point_objects().clear_selection(); 
  
  if(selected.pathIndex >= 0 && selected.pointIndex >=0){
    Point& sel = PathManager::get_instance().get_path(selected.pathIndex)->at(selected.pointIndex);
    PathManager::get_instance().get_selected_point_objects().add_to_selection(&sel);
    
    
  }else{
    add_path_point(vPos);
  }
 
}




void WorldView::add_path_point(Vec2 pos)
{
    PathManager::get_instance().get_path(m_selectedPathIndex)->add_point(pos);
    update();
}


void WorldView::move_selected_points(Vec2 pos)
{
  SelectedPointObject::PointSet& pointSet = PathManager::get_instance().get_selected_point_objects().get_node_set();
//   DebugStream << "Moving Points " << pos <<std::endl;
  for(SelectedPointObject::PointSet::iterator it = pointSet.begin(); it!=pointSet.end(); ++it){
    (*it)->pos += pos;
  }
  
  
  update();
}



void WorldView::path_motion(Vec2 pos)
{
  std::cout << "pm";
    WorldPointInfo closest = get_closest_point(pos);
    if(closest.pathIndex >=0 && closest.pointIndex >= 0){
      m_closestPointPos = PathManager::get_instance().get_path(closest.pathIndex)->at(closest.pointIndex).pos;
      m_closePointFound = true;
      std::cout << "Found close point"<< std::endl;
    }else{
      m_closePointFound = false;
    }
    update();
}




int WorldView::get_selected_path_index()
{
  return m_selectedPathIndex;
}
void WorldView::set_selected_path_index(int index)
{
  m_selectedPathIndex = index;
  update();

}






QColor WorldView::get_colour_at_point(QPoint pos)
{
    if(!m_activeImage.size().isEmpty()){
	QColor t = QPixmap::grabWidget(this,pos.rx(),pos.ry()).toImage().pixel(pos);
        return QColor(m_activeImage.pixel(pos));
    }else{
        return QColor(Qt::black);
    }

}

void WorldView::show_mass(bool state)
{
  m_showMass = state;
}
void WorldView::show_speed(bool state)
{
  m_showSpeed = state;
}
void WorldView::show_target(bool state)
{
  m_showTarget = state;
}
void WorldView::show_visibility(bool state)
{
  m_showVisibility = state;
}



void WorldView::paintEvent(QPaintEvent * event ){
    // Result in painter active
    QPainter painter(this);
//     painter.translate(QPointF(0,size().height()));

//     painter.rotate(-90);



    QRect bound = contentsRect();
    painter.fillRect(bound,Qt::white);
    painter.drawRect(bound);

    QFont f;
    f.setPixelSize(10);
    painter.setFont(f);
    bool osx = false;

#ifdef __APPLE__
    osx = true;
#endif
    //std::cout << "on Mac "<< osx <<std::endl;

    Surface::Ptr tsurf = WorldEngine::get_instance().get_surface(m_selectedSurfaceIndex);
    if(tsurf.get() && osx == false ){
      tsurf->tick(0.33);
      Vec2 imgsize = tsurf->get_size();
      QImage img(tsurf->get_image_buffer(), imgsize.m_x, imgsize.m_y, QImage::Format_RGB888);
      painter.drawImage(QRect(0,0,imgsize.m_x,imgsize.m_y),img.rgbSwapped() );
    }else if(tsurf.get() && osx ==true && tsurf.get()->is_type() == Surface::ST_FILLED){
    //  std::cout << "OSX DRAW"<<std::endl;
      tsurf->tick(0.33);
      Vec2 imgsize = tsurf->get_size();
      QImage img(tsurf->get_image_buffer(), imgsize.m_x, imgsize.m_y, QImage::Format_RGB888);
      painter.drawImage(QRect(0,0,imgsize.m_x,imgsize.m_y),img );
    
    }else if(tsurf.get() && osx ==true && tsurf.get()->is_type() == Surface::ST_STATIC){
    //  std::cout << "OSX DRAW"<<std::endl;
      tsurf->tick(0.33);
      Vec2 imgsize = tsurf->get_size();
      QImage img(tsurf->get_image_buffer(), imgsize.m_x, imgsize.m_y, QImage::Format_RGB888);
      painter.drawImage(QRect(0,0,imgsize.m_x,imgsize.m_y),img.rgbSwapped() );
    }else if(tsurf.get() && osx ==true && tsurf.get()->is_type() == Surface::ST_STREAM){
    //  std::cout << "OSX DRAW"<<std::endl;
      tsurf->tick(0.33);
      Vec2 imgsize = tsurf->get_size();
      static int temp = 0;
      static int t2 = 0;
      temp +=1;
      if(temp % 10 == 0){
        t2 +=1;
      }
      QImage img(tsurf->get_image_buffer(), imgsize.m_x, imgsize.m_y,QImage::Format( t2 % 16));

      painter.drawImage(QRect(0,0,imgsize.m_x,imgsize.m_y),img );
    }else if(tsurf.get() && tsurf.get()->is_type() == Surface::ST_FILLED ){
       tsurf->tick(0.33);
      Vec2 imgsize = tsurf->get_size();
      QImage img(tsurf->get_image_buffer(), imgsize.m_x, imgsize.m_y, QImage::Format_RGB888);
      painter.drawImage(QRect(0,0,imgsize.m_x,imgsize.m_y),img.rgbSwapped() );
    }


    draw_grids(painter,bound);

    float rad = 5;
    for(TreeNode::TreeNodeMap::const_iterator it = TreeNode::get_global_node_list_begin();  it != TreeNode::get_global_node_list_end();++it){
	    Vec2 pos = it->second.lock()->get_pos();
	    Vec2 vel =boost::get<Vec2>( it->second.lock()->get_property("Velocity").get_value());
	    float angle = sin(vel.magnitude());
 	   QPointF Qp(pos.m_x,pos.m_y);
// 	    painter.drawEllipse(Qp,rad,rad );
	    if(it->second.lock()->is_type() != TreeNode::TN_WORLD){
	      ImageManager::get_instance()->render_image( it->second.lock(),painter,Qp ,0);
	     
	    }else if(it->second.lock()->is_type() == TreeNode::TN_WORLD){
	        int rad=5;
		Vec2 size = boost::get<Vec2>(it->second.lock()->get_property("size").get_value());
		Vec2 pos = boost::get<Vec2>(it->second.lock()->get_property("pos").get_value());
		painter.save();
		painter.setBrush(QBrush(Qt::green));
		painter.drawEllipse(pos.m_x- (size.m_x*0.5),pos.m_y - (size.m_y * 0.5),size.m_x,size.m_y);
		painter.restore();
	    }
	    if(it->second.lock()->is_type() == TreeNode::TN_AGENT){
	      Agent* tempAgent = dynamic_cast<Agent*>(it->second.lock().get());

	      if(m_showNames == true){
		  QString name(QString::fromStdString( tempAgent->get_brain_function_name()));
  // 		DebugStream << name.toStdString()<<std::endl;
		  painter.drawText(QPointF(Qp.rx(),Qp.ry() + 5),name);
	      }
	      if(m_showVelocities == true){
		  painter.setPen(Qt::gray);
		  painter.drawLine(Qp,QPointF(Qp.rx()+vel.m_x,Qp.ry() + vel.m_y));

	      }

	    }


    }

    if(m_ShowPaths) draw_paths(painter);



}

void WorldView::draw_paths(QPainter& painter)
{
  //get paths
  QBrush brush(Qt::cyan);
  QPen pen(Qt::gray);
  pen.setStyle(Qt::DashLine);
  float w = 2;
  float hw= w*0.5f;
//   DebugStream << "No of paths "<< PathManager::get_instance().get_paths_size()<<std::endl;
  for(int n = 0; n < PathManager::get_instance().get_paths_size();++n){

    PathPtr current = PathManager::get_instance().get_path(n);
//     DebugStream << current->size() <<std::endl;
    if(n == m_selectedPathIndex){
	brush.setColor(Qt::black);
	pen.setColor(Qt::red);
    }else{
	brush.setColor(Qt::cyan);
	pen.setColor(Qt::gray);
    }
      for(int p = 0;p <  current->size();++p){
	    painter.setBrush(brush);
	    painter.setPen(pen);
	    Vec2 pos = current->at(p).pos;
	    painter.drawEllipse(QPointF(pos.m_x,pos.m_y) ,w,w);
	    painter.save();
	    
	    if(current->at(p).selected){
		float scale = 4;
		float halfScale = scale * 0.5;
		brush.setColor(Qt::transparent);
		pen.setColor(Qt::black);
		painter.drawRoundedRect( pos.m_x - halfScale,pos.m_y - halfScale,w*scale,w*scale,2,2);
	    }
	     if(n == m_selectedPathIndex){
		brush.setColor(Qt::black);
		pen.setColor(Qt::red);
	    }else{
		brush.setColor(Qt::cyan);
		pen.setColor(Qt::gray);
	    }
	    painter.restore();
	    
	    if(p < current->size()-1 ){
  		Vec2 pos2 = current->at(p+1).pos;
	   	painter.drawLine(QPointF(pos.m_x,pos.m_y),QPointF(pos2.m_x,pos2.m_y));
	    }
	    
	    
	    if(m_closePointFound){
	      painter.drawRoundedRect(pos.m_x - 10,pos.m_y - 10, 20,20,2,2);
	    }

      }

  }



}






void WorldView::draw_grids(QPainter& painter, QRect& rect)
{

  //TODO drawing of several
    for(ScaleVector::iterator it = GridManager::begin(); it != GridManager::end();++it){
      if( (*it)->is_active()){
	  for(Scale::ScaleValues::iterator scaleIt = (*it)->begin(); scaleIt != (*it)->end();++scaleIt  ){
		float val = (*scaleIt);
		if( (*it)->get_direction() ==Scale::SD_VERTICAL){
		    painter.drawLine(0,val,rect.width(),val    );
		}else{
		    painter.drawLine(val,0,val,rect.height());
		}
	  }
      }
    }


}







void WorldView::draw_visibility(TreeNode::Ptr node,   QPainter& painter, QPointF& pos)
{
      QPainterPath path;
      float rad2 = boost::get<float>( node->get_property("Visibility").get_value() )/2.0f;
      path.addEllipse( pos,rad2,rad2                 );
      path.setFillRule(Qt::WindingFill);
      painter.fillPath(path,QBrush(QColor(255,0,0,50)));
}
void WorldView::draw_mass(TreeNode::Ptr node, QPainter& painter, QPointF& pos)
{
    QPointF p =  pos - QPointF(20,0);
    std::stringstream ss;
    ss << "M:" <<boost::get<float>(node->get_property("Mass").get_value())    ;
    painter.font().bold();
    painter.drawText(p,   QString( ss.str().c_str()                    )  );
}
void WorldView::draw_speed(TreeNode::Ptr node, QPainter& painter, QPointF& pos)
{
    QPointF p =  pos - QPointF(20,10);
    std::stringstream ss;
//     ss << "S:" <<boost::get<float>(node->get_property_value("Speed"))    ;
    painter.font().bold();
    painter.drawText(p,   QString( ss.str().c_str()                    )  );

}
void WorldView::draw_target(TreeNode::Ptr node, QPainter& painter, QPointF& pos)
{
    TreeNode::Ptr t(TreeNode::get_treenode_by_id(0).lock());
    QPoint tpos(t->get_pos().m_x,t->get_pos().m_y);
    painter.drawLine(pos, tpos      );
}





void WorldView::set_selected_Ai(std::string& ai)
{
  m_selectedBrainString = ai;
  DebugStream << "Ai set to " << m_selectedBrainString<<std::endl;
}

void WorldView::resizeEvent(QResizeEvent* event)
{
    Vec2 wSize(width(),height());
    WorldEngine::get_instance().get_world()->set_property("WorldSize",wSize);
    QWidget::resizeEvent(event);
}


void WorldView::set_mouse_mode(WorldView::MouseModeTool mode)
{
  m_mouseMode = mode;
  m_mouseMoveMode = MMM_NORMAL;
  if(mode == MT_PATH){
    m_mouseMoveMode = MMM_PATHMOTION;
    std::cout << "Path motion activated"<<std::endl;
  }
}

void WorldView::set_show_name_state(bool state)
{
  m_showNames = state;
}


void WorldView::set_show_velocities_state(bool state)
{
  m_showVelocities = state;


}

WorldView::MouseMoveMode WorldView::get_current_mouse_mode()
{
  return m_mouseMoveMode;
}

int WorldView::get_selected_surface_index()
{
  return m_selectedSurfaceIndex;
}


void WorldView::set_selected_surface_index(int index)
{
  if(index < 0 || index >= WorldEngine::get_instance().get_number_of_surfaces())return;
  m_selectedSurfaceIndex = index;
  WorldEngine::get_instance().set_current_screen(m_selectedSurfaceIndex);
}

Surface::Ptr WorldView::get_current_surface()
{
  return WorldEngine::get_instance().get_surface(m_selectedSurfaceIndex);
}

int WorldView::remove_selected_surface()
{
    if(m_selectedSurfaceIndex ==0  || m_selectedSurfaceIndex >= WorldEngine::get_instance().get_number_of_surfaces())return m_selectedSurfaceIndex;
    
    WorldEngine::get_instance().delete_surface(m_selectedSurfaceIndex);
    
    if(m_selectedSurfaceIndex > 0)m_selectedSurfaceIndex -=1;
    update();
    WorldEngine::get_instance().set_current_screen(m_selectedSurfaceIndex);
    return m_selectedSurfaceIndex;
    
}




