#ifndef __WORLDVIEW_HPP_
#define __WORLDVIEW_HPP_

#include <QMessageBox>
#include <QGraphicsView>
#include <iostream>
#include <QImage>
#include <QWidget>
// #include "ndoe.hpp"
#include "TreeNode.hpp"
#include "Surface.hpp"

class MainWindow;

struct WorldPointInfo{
  int pointIndex,pathIndex;
  WorldPointInfo(int pathIndex = -1,int pointIndex = -1):
    pointIndex(pointIndex),
    pathIndex(pathIndex)
  {
    
  }
};



class WorldView : public QWidget{
  Q_OBJECT
public:
  enum MouseModeTool{
    MT_SPAWN = 0,
    MT_SELECT,
    MT_MOVE,
    MT_DELETE,
    MT_PATH
  };
  
private:
  MainWindow* m_mainWindow;  
  QImage m_activeImage;
  std::string m_selectedBrainString;
  int m_duplicateId;
  bool m_duplicateState;
  bool m_showVisibility, m_showSpeed,m_showMass,m_showTarget;
  Vec2 m_clickLocation;
  int m_selectedPathIndex;
  MouseModeTool m_mouseMode;
  void draw_visibility(TreeNode::Ptr node, QPainter& painter, QPointF& pos);
  void draw_speed(TreeNode::Ptr node,   QPainter& painter, QPointF& pos);
  void draw_mass(TreeNode::Ptr node,   QPainter& painter, QPointF& pos);
  void draw_target(TreeNode::Ptr node,   QPainter& painter, QPointF& pos);
  
  void draw_grids(QPainter& painter,QRect& rect);
  
  void draw_paths(QPainter& painter);
  
  WorldPointInfo get_closest_point(Vec2 pos);
  
  Vec2 m_closestPointPos;
  bool m_closePointFound;
  
  int m_selectedSurfaceIndex;
  
  enum MouseMoveMode{
      MMM_NORMAL = 0,
      MMM_CLICKED,
      MMM_DRAGGING,
      MMM_PATHDRAGGING,
      MMM_PATHMOTION
  }m_mouseMoveMode;
  bool m_showNames;
  bool m_ShowPaths;
  bool m_showVelocities;
  
  public:
    WorldView(QWidget* parent,MainWindow* mainwind);
    void load_image(QString filename);
    void load_video(QString filename);
    void load_stream();
    void load_filled(QColor col);
    void mousePressEvent(QMouseEvent* event);
    void mouseReleaseEvent ( QMouseEvent * event ); 
    void mouseMoveEvent(QMouseEvent* event);
    void add_agent(QPoint pos);
    void move_agent(Vec2 pos);
    void select_agent(QPoint pos);
    void delete_agent(QPoint pos);
    void add_path_point(Vec2 pos);
    void path_mode_clicked(QPoint pos, bool shiftDown=false);
    
    QColor get_colour_at_point(QPoint pos);
    void advance(int phase);
    void paintEvent(QPaintEvent* event);
    void set_selected_Ai(std::string& ai);
    void show_visibility(bool state);
    void show_speed(bool state);
    void show_mass(bool state);
    void show_target(bool state);
    virtual void resizeEvent(QResizeEvent* event);
    void set_duplicate_id(int id);
    void set_duplicate_state(bool state);
    void set_mouse_mode(MouseModeTool mode);
    void set_show_name_state(bool state);
    void set_draw_paths_state(bool state);
    void set_show_velocities_state(bool state);
    
    int get_selected_path_index();
    void set_selected_path_index(int index);
    
    MouseMoveMode get_current_mouse_mode();
  
    void move_selected_points(Vec2 pos);
    void path_motion(Vec2 pos);
    
    
    void set_selected_surface_index(int index);
    int get_selected_surface_index();
    Surface::Ptr get_current_surface();
    int remove_selected_surface();
};

#endif