#include "mainwindow.hpp"
#include "ui_MainWindow.h"
#include <iostream>
#include <QMessageBox>
#include <QFileDialog>
#include <QTimer>
#include "WorldEngine.hpp"
#include "OSCDialog.hpp"
#include <algorithm>
#include <QTextDocument>
#include <QUrl>
#include <QTextEdit>
#include <QFile>
#include <QTextStream>
#include "TreeNodeModel.hpp"
#include "PropertyModel.hpp"
#include "PropertyDelegate.hpp"
#include <QProcess>
#include <QThread>
#include <QSplitter>
#include <QDebug>
#include <sstream>
#include <QDateTime>
#include <QSettings>
#include "ui_BufferWindow.h"
#include "BufferWindow.hpp"
#include <QToolBar>
#include "GridManager.hpp"
#include <QFileSystemWatcher>
#include "PropertyHeader.hpp"
#include "GreetingDialog.hpp"
#include "LuaFunctions.hpp"
#include <QInputDialog>
#include "PathManager.hpp"
#include <sstream>
#include "Utilities.hpp"
#include "lua/llex.h"
#include <QColorDialog>
#include <QPlainTextEdit>
#include "FileDownloader.hpp"
#include "config.hpp"

MainWindow* MainWindow::s_instance = 0;

MainWindow* MainWindow::get_instance()
{
  if(!s_instance){
    s_instance = new MainWindow();
  }
  return s_instance;
}






QString get_text_from_file(QString fileName)
{
   QString output;
   if(!fileName.isNull()){

    QFile file(fileName);
     if( file.open( QIODevice::ReadOnly | QIODevice::Text )){
	DebugStream << file.isOpen()<<std::endl;
	QTextStream stream(&file);
	output.append(stream.readAll());
     }
  }
  return output;
}




MainWindow::MainWindow():
	 ui(new Ui::MainWindow),
	 m_screen(new WorldView(this,this)),
	 m_animationTimer(new QTimer(this)),
	 m_fileMonitorTimer(new QTimer(this)),
	 m_refreshSpeed(33),
	 m_propertyDelegate(new PropertyDelegate(this)),
	 m_currentlySelectedNode(0),
	 m_sclangProcess(new QProcess(this)),
	 m_bufferWindow(new BufferWindow(this,m_sclangProcess)),
	 m_duplicateState(false),
	 m_fileWatcher(new QFileSystemWatcher(this)),
	 m_mouseToolGroup(new QActionGroup(this)),
	 m_playing(false)
{


	QCoreApplication::setApplicationName("The Agent Tool");
	QCoreApplication::setOrganizationName("Stephen Pearse");
	QCoreApplication::setOrganizationDomain("stephenpearse.co.uk");



// 	m_sclangProcess->setStandardOutputFile("testSclang.txt");
// 	m_sclangProcess->setStandardErrorFile("testSclangErrorlog.txt");
// 	m_sclangProcess->setProcessChannelMode(QProcess::MergedChannels);
	//m_sclangProcess->execute("killall sclang");
	//m_sclangProcess->execute("killall scsynth");
	//m_sclangProcess->start("sclang",QIODevice::ReadWrite);
	//m_sclangProcess->waitForStarted();

 	ui->setupUi(this);
  setWindowTitle(QString::fromStdString(get_version_string()));



	ui->rightLayout->addWidget(m_screen);
	ui->treeNodeView->setAnimated(true);

	m_mouseToolGroup->addAction(ui->actionSpawn);
	m_mouseToolGroup->addAction(ui->actionSelect);
	m_mouseToolGroup->addAction(ui->actionMove);
	m_mouseToolGroup->addAction(ui->actionDelete_2);
	m_mouseToolGroup->addAction(ui->actionPath);

	m_propertyHeader = new PropertyHeader(Qt::Vertical,ui->propertyDock,this);

	ui->propertyTable->setVerticalHeader(m_propertyHeader);



// 	m_screen->setGeometry(10,30,640,480);
// 	m_screen->setFixedSize(800,600);
	populate_ai_list_widget();
	setup_scales();

	layout();



// 	m_screen->setGeometry(workRect);
// 	m_screen->setSceneRect(workRect);
// 	ui->graphicsView->setScene(m_screen);
// 	ui->graphicsView->setRenderHint(QPainter::Antialiasing);
	populate_ai_list_widget();

	restore_data();

	connect(ui->actionReloadLua,SIGNAL(triggered()),this,SLOT(on_reload_lua()));
	connect(ui->actionAbout,SIGNAL(triggered()),this,SLOT(on_about()   ));
	connect(ui->actionImport_Image,SIGNAL(triggered()),this,SLOT(on_import_image()));
	connect(ui->actionPlay_Pause,SIGNAL(triggered(bool)),this,SLOT(on_playstop(bool)));
	connect(m_animationTimer,SIGNAL(timeout()),this,SLOT(on_timer()));
	connect(ui->aiTypeList,SIGNAL(itemSelectionChanged ()),this,SLOT(currentAiChanged()) );
// 	connect(ui->dial,SIGNAL(dialMoved(int)), this,SLOT(on_speedChanged(int))  );
	connect(ui->treeNodeView,SIGNAL(clicked(QModelIndex)),this,SLOT(on_treeView_clicked(QModelIndex) ) );
	connect(ui->treeNodeView,SIGNAL(doubleClicked(QModelIndex)),this,SLOT(on_treeView_doubleclicked(QModelIndex)) );
	connect(m_sclangProcess,SIGNAL(readyRead()),this,SLOT(on_readyRead()) );
// 	connect(ui->m_recordButton,SIGNAL(clicked(bool)),this,SLOT(on_record(bool)));
// 	ui->dial->setValue(33);
	connect(m_fileMonitorTimer,SIGNAL(timeout()),this,SLOT(on_moniter_files()));
	connect(m_fileWatcher,SIGNAL(fileChanged(QString)),this,SLOT(on_file_edited_externally(QString)));
	connect(ui->PathList,SIGNAL(currentRowChanged(int)),this,SLOT(path_index_changed(int)));
	connect(ui->SurfaceList,SIGNAL(currentRowChanged(int)),this,SLOT(surface_index_changed(int)));


  QUrl fileUrl("http://www.stephenpearse.co.uk/latestagentversion.txt");
  m_fileDownloader = new FileDownloader(fileUrl,this);

  connect(m_fileDownloader,SIGNAL(downloaded()),this,SLOT(on_downloaded_file()));






	WorldEngine::get_instance().get_world()->add_child( TreeNode::factory(TreeNode::TN_AGENT,Vec2(0,0),"ai_Emptygrain")  );

 	WorldEngine::get_instance().tick();

	m_treeModel = new TreeNodeModel(ui->treeNodeView) ;

	ui->treeNodeView->setModel(m_treeModel);
	ui->treeNodeView->setAnimated(true);
	ui->propertyTable->setModel(new PropertyModel(ui->propertyTable,0));
// 	ui->propertyTable->setItemDelegate(m_propertyDelegate);
	ui->treeNodeView->viewport()->update();

	ui->addPathButton->setDefaultAction(ui->actionCreate_New_Path);
	ui->removePathButton->setDefaultAction(ui->actionRemove_Selected_Path);
	ui->BasicBackgroundButton->setDefaultAction(ui->actionBasic_Background);
	ui->ImageButton->setDefaultAction(ui->actionImport_Image);
	ui->VideoButton->setDefaultAction(ui->actionImport_Video);
	ui->StreamButton->setDefaultAction(ui->actionStart_Stream);
	ui->RemoveScreenButton->setDefaultAction(ui->actionRemove_Screen_2);





//   TreeNode::s_constructionTrigger  = boost::bind(&MainWindow::update_treemodel,this);
//   TreeNode::s_destructionTrigger  = boost::bind(&MainWindow::update_treemodel,this);


//       if(!QDir::home().exists("The Agent Project")){
// 	  QDir::home().mkdir("The Agent Project");
//       }


// 	m_baseFolder = QDir::home();
// 	m_baseFolder.cd("The Agent Project");

// 	if(!m_baseFolder.exists("AgentProjectRecordings")){
// 	    m_baseFolder.mkdir("AgentProjectRecordings");
// 	}

// 	m_recordingFolder = m_baseFolder;
// 	m_recordingFolder.cd("AgentProjectRecordings");
// 	DebugStream << m_recordingFolder.absolutePath().toStdString()<<std::endl;

// 	int r;
	// 	r = m_sclangProcess->write("s.boot;\n\x0c");
// 	std::stringstream ss;
// 	ss <<" s.waitForBoot({ s.recHeaderFormat = \"wav\";s.prepareForRecord(\"";
// 	ss << m_recordingFolder.absolutePath().toStdString();
// 	ss << "/test.wav\");});\n\x0c";
// 	r = m_sclangProcess->write( ss.str().c_str());

// 	QString bootFile = get_text_from_file("SuperBootSynths.scd");
// 	m_sclangProcess->write(bootFile.toStdString().c_str());





	setup_working_directory();
	load_boot_scripts();

// 	ui->scInput->hide();
// 	ui->dial->hide();
// 	ui->lcdNumber->hide();

	// 	m_sclangProcess->write();

// 	Sleeper::sleep(10);
	//pull any files that have been loaded already i.e basic script


// 	  DebugStream << path.path() <<std::endl;

	on_actionSpawn_triggered();
	ui->statusBar->showMessage(tr("Welcome to the Agent Project"),2000);


	QSettings settings;
	bool showGreeting = settings.value("Greeting",true).toBool();
	if(showGreeting){
	    GreetingDialog* greetdlg = new GreetingDialog(this);
	    greetdlg->show();
	    greetdlg->raise();
	    greetdlg->activateWindow();

	}
// 	m_fileMonitorTimer->start(1000);

	QPalette t_pallete = palette();

	t_pallete.setColor(ui->menuFile->backgroundRole(),QColor(255,255,255));
	ui->menuFile->setPalette(t_pallete);



	QString ipAdd = settings.value("IP_ADDRESS","127.0.0.1").toString();
	WorldEngine::get_instance().set_lo_address(ipAdd.toStdString().c_str(),"57110");

	bool bootServer = settings.value("BootServer",false).toBool();
	if(bootServer){
	  int portNo = WorldEngine::get_instance().spawn_server(8001);
	  std::stringstream ss;
	  ss << "Booted server on port "<< portNo;
	  QMessageBox::warning(this,"Notice", ss.str().c_str());
	}



	load_bundled_synths();

	Vec2 wSize(m_screen->geometry().width(),m_screen->geometry().height());
	WorldEngine::get_instance().get_world()->set_property("WorldSize",wSize);
	
	m_screen->load_filled(QColor(255,255,255));
	std::stringstream ss;
	ss << 255 << ","<< 255 << ","<<255;
	m_screen->get_current_surface()->set_name(ss.str());

	ui->actionPaths->setChecked(true);
	ui->actionSpawn->trigger();
	update_path_window();
	update_image_window();
	ui->SurfaceList->setCurrentRow(0);
}

void MainWindow::on_actionCopy_Default_Scripts_triggered()
{


  qDebug() << "Copying Scripts";

  status("Attempting to copy default scripts");
  if(!m_baseFolder.exists("scripts")){
    m_baseFolder.mkdir("scripts");
  }
  QDir scriptsFolder  = m_baseFolder;
  scriptsFolder.cd("scripts");
  QDir orig(":/scripts/basic.lua");

  QFile file(":/scripts/basic.lua");
  qDebug() << "file exists " << file.exists();
  file.copy(scriptsFolder.absolutePath() + "test.lua");



//   qDebug()<< "scripts "<< orig.absolutePath();

}


void MainWindow::populate_toolbar()
{

}


void MainWindow::on_actionSet_Record_Directory_triggered()
{

    status("Attempting to set up record directory");


    QFileDialog dialog(this);
    dialog.setDirectory(QDir::homePath());
    dialog.setFileMode(QFileDialog::Directory);
    dialog.setOption(QFileDialog::ShowDirsOnly);

// 	QDir directory = dialog.directory();
    QString file = QFileDialog::getExistingDirectory(this,tr("Choose a directory to save recordings"),QDir::homePath());
    if(!file.isEmpty()) m_recordingFolder = QDir(file);



// 	m_recordingFolder.setPath("test");
    DebugStream << m_recordingFolder.absolutePath().toStdString()<<std::endl;


    dialog.close();

}





void MainWindow::on_actionAppendSCFile_triggered()
{
  QString fileName = QFileDialog::getOpenFileName(this,
         tr("Open SCD file"), QDir::homePath(), tr("SuperColliderFiles (*.scd *.txt *.rtf)"));
  if(!fileName.isNull()){

    QString text = get_text_from_file(fileName);
//     DebugStream << fileName.toStdString()<<std::endl;

//     QFile file(fileName);
//      if( file.open( QIODevice::ReadOnly | QIODevice::Text )){
// 	DebugStream << file.isOpen()<<std::endl;
// 	QTextStream stream(&file);
// 	ui->scInput->append(text);

//      }
  }
}






void MainWindow::on_readyRead()
{
  if(!m_sclangProcess->bytesToWrite() ){
//     ui->scOutput->appendPlainText(  m_sclangProcess->readAll()    );
  }
}
void MainWindow::on_actionEvaluate_triggered()
{
  /*
  DebugStream << "Evaluate"<<std::endl;
  QTextCursor selected =  ui->scInput->textCursor();

  m_sclangProcess->write( selected.selectedText().append("\n\x0c").toAscii());
//   ui->
*/
}


MainWindow::~MainWindow()
{
  status("Closing");
  QSettings settings;
  settings.setValue("mainwindow",QMainWindow::saveState());
  settings.setValue("mainwindowGeometry", QMainWindow::saveGeometry());//     settings.beginGroup ("scripts");
//     settings.remove ("");
//   settings.endGroup ();

  qDebug()<< settings.fileName();

  update_autoload_scripts();
  /*
  settings.beginWriteArray("UserScripts");
  int scriptIndex = 0;
  ScriptSet& tempSet = WorldEngine::get_instance().get_loaded_scripts();
  for(ScriptSet::iterator it = tempSet.begin();it!=tempSet.end();++it){
	  std::string t = (*it).path;
	  DebugStream << t<<std::endl;
	  QDir path(QString::fromStdString(t));
	  if((*it).autoLoad){
	    DebugStream <<"Index "<< scriptIndex<< "  " << path.absolutePath().toStdString()<<std::endl;
	    settings.setArrayIndex(scriptIndex);
	    settings.setValue("script",path.absolutePath());
	    scriptIndex +=1;
	  }
    }
    settings.endArray();
    */
 // m_sclangProcess->execute("killall -9 sclang");
//  m_sclangProcess->execute("killall -9 scsynth");

  //m_sclangProcess->kill();
  //m_sclangProcess->close();
}


void MainWindow::update_autoload_scripts(){
	QSettings settings;
	settings.beginWriteArray("UserScripts");
  	int scriptIndex = 0;
  	ScriptSet& tempSet = WorldEngine::get_instance().get_loaded_scripts();
  	for(ScriptSet::iterator it = tempSet.begin();it!=tempSet.end();++it){
	  std::string t = (*it).path;
	  DebugStream << t<<std::endl;
	  QDir path(QString::fromStdString(t));
	  if((*it).autoLoad){
	    DebugStream <<"Index "<< scriptIndex<< "  " << path.absolutePath().toStdString()<<std::endl;
	    settings.setArrayIndex(scriptIndex);
	    settings.setValue("script",path.absolutePath());
	    scriptIndex +=1;
	  }
    }
    settings.endArray();

}

void MainWindow::restore_data()
{
  QSettings settings;
  //settings.get("playtoolbar",ui->playbackToolBar->saveGeometry());
  //settings.setValue("mainwindow",QMainWindow::saveState());
 restoreState(settings.value("mainwindow").toByteArray());
  restoreGeometry(settings.value("mainwindowGeometry").toByteArray());

//  ui->actionOpen_Server_on_Boot->setChecked(settings.value("BootServer",false).toBool() );

}
void MainWindow::selected_node(int id)
{
	m_currentlySelectedNode = id;
	ui->propertyTable->setModel(new PropertyModel(this,   m_currentlySelectedNode          ));
	m_screen->set_duplicate_id(m_currentlySelectedNode);
}


void MainWindow::on_treeView_clicked(const QModelIndex& index)
{

    m_currentlySelectedNode = index.internalId();

    status("Agent " + QString::number(m_currentlySelectedNode).toStdString() + " selected");
    try{
      if(QApplication::mouseButtons() == Qt::RightButton && index.internalId() != 0){
	   ui->menuSelected_Agent->popup(  cursor().pos() );

      }else{
	selected_node(m_currentlySelectedNode);
      }

    }catch(TreeNode::IdException& e){
	DebugStream << "Unable to spawn property model due to treenode id exception"<<std::endl;
    }
}

void MainWindow::on_treeView_doubleclicked(const QModelIndex& index)
{


}




void MainWindow::populate_ai_list_widget()
{
    ui->aiTypeList->clear();
    QStringList aiList;
    std::vector<std::string>& aiNames = WorldEngine::get_instance().get_ai_name_vector();
    for(std::vector<std::string>::iterator it = aiNames.begin(); it != aiNames.end();++it){
	 aiList.push_back(QString::fromStdString( *it ));
    }
    aiList.sort();
    ui->aiTypeList->addItems(aiList);

}

void MainWindow::on_reload_lua()
{
  DebugStream << "reloading lua"<<std::endl;
  on_playstop(false);

  ScriptSet& tempSet = WorldEngine::get_instance().get_loaded_scripts();
  for(ScriptSet::iterator it = tempSet.begin();it!=tempSet.end();++it){
        std::string t = (*it).path;
        DebugStream << t<<std::endl;
        bool autostate = (*it).autoLoad;

        load_script(QString::fromStdString(t),autostate);

    }

}

void MainWindow::on_speedChanged(int speed)
{
  m_refreshSpeed = speed;
  if(m_animationTimer->isActive()){
	m_animationTimer->start(m_refreshSpeed);
  }
}
void MainWindow::on_actionClear_Nodes_triggered()
{

    status("Clearing Agents");
    WorldEngine::get_instance().reset_world_state();
    m_screen->update();
    Vec2 wSize(m_screen->width(),m_screen->height());
    WorldEngine::get_instance().get_world()->set_property("WorldSize",wSize);
    ui->propertyTable->setModel(new PropertyModel(ui->propertyTable,WorldEngine::get_instance().get_world()->get_id()));
    m_treeModel->update();
    m_screen->layout();
    update();
}



/*
void MainWindow::on_actionReloadLua_triggered()
{
  //DebugStream << "reloading lua"<<std::endl;
}*/
void MainWindow::on_about()
{
    status("About");
    QMessageBox::about(this,"About","<h1>The Agent Tool</h1><p>(c) Stephen Pearse 2012 - 2015 </p>" + QString::fromStdString(get_version_string()));
}


void MainWindow::on_import_image()
{

    status("Attempting to import image");
  QString fileName = QFileDialog::getOpenFileName(this,
         tr("Open Image"),QDir::homePath(), tr("Image Files (*.png *.jpg *.bmp)"));
  if(!fileName.isNull()){
    DebugStream << fileName.toStdString()<<std::endl;
    try{


      m_screen->load_image(fileName);
      status("Imported "+ fileName.toStdString());


      QMessageBox::StandardButton ret = QMessageBox::question(this,"Snap","Would you like to resize the current world to the size of the image?",QMessageBox::Yes|QMessageBox::No);
      if(ret == QMessageBox::Yes){
	WorldEngine::get_instance().snap_worldsize(m_screen->get_selected_surface_index());
      }
      QFileInfo path(fileName);
      m_screen->get_current_surface()->set_name(path.fileName().toStdString());

      update_image_window();
    }catch(...){
      ui->actionImport_Image->setChecked(false);

    }
  }

}
void MainWindow::on_actionImport_Video_triggered()
{

     status("Attempting to import video file");
    QString fileName = QFileDialog::getOpenFileName(this,
         tr("Open Video"), QDir::homePath(), tr("Video Files (*.avi )"));
  if(!fileName.isNull()){
    DebugStream << fileName.toStdString()<<std::endl;
    try{
      m_screen->load_video(fileName);

      status("Imported "+ fileName.toStdString());

      QMessageBox::StandardButton ret = QMessageBox::question(this,"Snap","Would you like to resize the current world to the size of the video?",QMessageBox::Yes|QMessageBox::No);
      if(ret == QMessageBox::Yes){
	WorldEngine::get_instance().snap_worldsize(m_screen->get_selected_surface_index());
      }
       QFileInfo path(fileName);
      m_screen->get_current_surface()->set_name(path.fileName().toStdString());
      update_image_window();
    }catch(...){
	ui->actionImport_Video->setChecked(false);
    }
  }
}
void MainWindow::on_actionStart_Stream_triggered()
{
    status("Attempting to open Webcam");
  DebugStream << "Starting stream "<<std::endl;
  try{
    m_screen->load_stream();
    status("Opened Webcam");
      QMessageBox::StandardButton ret = QMessageBox::question(this,"Snap","Would you like to resize the current world to the size of the image?",QMessageBox::Yes|QMessageBox::No);
      if(ret == QMessageBox::Yes){
	WorldEngine::get_instance().snap_worldsize(m_screen->get_selected_surface_index());
      }
      m_screen->get_current_surface()->set_name("camera");
      update_image_window();
  }catch(...){
    ui->actionStart_Stream->setChecked(false);
    QMessageBox::warning(this, tr("Agentproject"),
                                tr("Could not detect a valid camera, please make sure it is plugged in!\n"),QMessageBox::Ok);
  }
}

void MainWindow::on_actionBasic_Background_triggered()
{
  DebugStream << "Basic colour triggered" <<std::endl;
  status("Attempting to create basic background");
  QColor col = QColorDialog::getColor(QColor(256,256,256),this,"Select a background colour");
  if(col == QColor(256,256,256))return;

  m_screen->load_filled(col);

  std::stringstream ss;
  ss << col.red() << ","<< col.green() << ","<<col.blue();
    m_screen->get_current_surface()->set_name(ss.str());
  update_image_window();

}


typedef std::map<int,StringVector*> ErrorStorage;

void MainWindow::on_playstop(bool state)
{

  DebugStream << "Play "<< state<<std::endl;
  if(state){
  		status("Playing");
      m_animationTimer->start(m_refreshSpeed);

  }else{
      status("Stopping");
      m_animationTimer->stop();

      ErrorStorage errors;
      //search for inactive or broken Agents
      QString message;
      message += "Errors found in Nodes:\n";

      bool errorsFound = false;
      bool firstError = true;
      for(TreeNode::TreeNodeMap::const_iterator it = TreeNode::get_global_node_list_begin(); it != TreeNode::get_global_node_list_end();++it ){
	TreeNode::Ptr node =  it->second.lock();
// 	DebugStream << node->get_id() <<     " node active "<< node->get_activity_state()<<std::endl;
	if(node->get_activity_state() == false){

		if(firstError){
			status("Error found");
			firstError = false;
		}

	  errorsFound = true;
	  int id = node->get_id();
	  std::stringstream ss;


	  ss << "\n\n" << node->get_script_name() << " (ID "<< id << ")";

	  StringVector& errorMessages =  node->get_errors();
	  for(int n = 0 ; n < errorMessages.size();++n){
	      ss <<"\n\t"<<errorMessages.at(n);
	  }
	  ss<< "\n";
	  message += QString::fromStdString(ss.str());
// 	  StringVector* er = &(*it).second.lock()->get_errors();
// 	  errors.insert( std::make_pair(  id,er )     );

	}
      }

      if(errorsFound){
      	message +="\nRemove all?";
		QMessageBox::StandardButton response = QMessageBox::question(this,"Errors found",message,QMessageBox::No | QMessageBox::Yes);
		if(response == QMessageBox::Yes){
			for(TreeNode::TreeNodeMap::const_iterator it = TreeNode::get_global_node_list_begin(); it != TreeNode::get_global_node_list_end();++it ){
				TreeNode::Ptr node =  it->second.lock();
				if(node->get_activity_state() == false){
					node->remove_treenode_from_existence();
					node.reset();
				}
			}
			m_screen->update();
	 	DebugStream << "yes, clear!!  "<<std::endl;
	}
      }
      ui->treeNodeView->collapseAll();
  }
  
  
  
  m_playing = state;
}

void MainWindow::set_play_state(bool state)
{
  on_playstop(state);
}
bool MainWindow::get_play_state()
{
  return m_animationTimer->isActive();
}


void MainWindow::on_timer()
{
  WorldEngine::get_instance().tick();
  m_screen->update();

}
void MainWindow::on_actionEdit_OSC_triggered()
{
	status("Edit outgoing OSC");
    OSCDialog* dlg = new OSCDialog(this);
    dlg->show();
}
void MainWindow::currentAiChanged()
{
	if(!ui->aiTypeList->selectedItems().empty()){

	  std::string t = ui->aiTypeList->selectedItems().first()->text().toStdString();

	  status(t + " selected");

	  m_screen->set_selected_Ai(t);
	}
}

void MainWindow::on_actionReloadLua_triggered()
{
	DebugStream << "Attempting to reload lua"<<std::endl;
	status("Reloading Lua scripts");

 int ret = QMessageBox::warning(this, tr("Agentproject"),
                                tr("Warning, this will destroy all agents.\n"),QMessageBox::Cancel|QMessageBox::Ok);
	if(ret == QMessageBox::Cancel)return;
	ui->propertyTable->clearSpans();
	ui->propertyTable->model()->revert();
	ui->treeNodeView->model()->revert();


	m_screen->update();
	populate_ai_list_widget();


}


void MainWindow::load_script(QString filePath, bool autoLoad)
{
	  status("Attempting to load " + filePath.toStdString());

	  WorldEngine::get_instance().append_lua_script(filePath.toStdString().c_str(),autoLoad);
	  ui->propertyTable->clearSpans();
	  ui->propertyTable->model()->revert();
	  ui->treeNodeView->model()->revert();
	  m_screen->update();
	  populate_ai_list_widget();
	  m_fileWatcher->addPath(filePath);

		update_autoload_scripts();

}



void MainWindow::append_script(QString filepath, bool query)
{
	try{
	  bool autoLoad = false;
	  if(query){
	    if (QMessageBox::Yes == QMessageBox::question(this, "Question", "Would you like to load this script by default?", QMessageBox::Yes|QMessageBox::No))
	    {
// 	      Qse
		  autoLoad = true;
		 /*
		  QSettings settings;
		  settings.beginGroup("bootscripts");
		  if(!settings.contains(filepath)){
		      QFileInfo info(filepath);

		      settings.setValue(info.fileName(),filepath);
		  }

		  settings.endGroup();
// 		  settings.beginWriteArray("bootscripts");
		  */
		      status("Will autoload " + filepath.toStdString() + " on boot");
	    }
	  }

	load_script(filepath,autoLoad);

	}catch(LuaScript::Exception e){

	  QMessageBox::warning(this,"Error",QString::fromStdString( e.what()));

	}
}

void MainWindow::load_boot_scripts()
{

    try{
        status("Auto Loading user scripts");

      QSettings settings;
      qDebug()<< settings.fileName();
       QStringList keys = settings.allKeys();
      qDebug()<< keys;

      int size = settings.beginReadArray("UserScripts");
      DebugStream << "Size of script array "<< size <<std::endl;
      for(int n = 0; n < size ;++n){
	   settings.setArrayIndex(n);
	   QString path = settings.value("script").toString();
	   DebugStream << "Attempting to load "<< path.toStdString()<<std::endl;
	if(!path.isEmpty()){
	  load_script(path,true);
	}
      }
      settings.endArray();

    }catch(...){

    }
}


void MainWindow::on_actionAppend_Lua_Script_triggered()
{
    DebugStream << "Attempting to append lua script"<<std::endl;
    QString filename = QFileDialog::getOpenFileName(this,tr("Load Lua Script"), QDir::homePath(), tr("Lua (*.lua )"));

    status("Attempting to load " + filename.toStdString() );

    append_script(filename,true);




}
void MainWindow::on_file_edited_externally(QString filename)
{


  qDebug() << filename << " edited externally";
  status(filename.toStdString() + " updated");
  append_script(filename);
}





void MainWindow::on_actionRecord_toggled(bool state)
{
  if(state){
      m_sclangProcess->write("s.recHeaderFormat = \"wav\";\n\x0c");
//       std::stringstream ss;
//       ss << "s.waitForBoot({\n\x0c";
//       ss << "s.prepareForRecord(\"/drive_d/profiles/linux/mup10sap/Desktop/test.wav\");\n\x0c";
//       ss << "s.sync;\n\x0c";
//       ss << "s.record;\n\x0c";
//       ss << "});\n\x0c";
      std::stringstream ss;
      ss <<"s.prepareForRecord(\"";
//
      ss << m_recordingFolder.absolutePath().toStdString();
      QDateTime dateTime;
      ss << "/ap_"<< dateTime.currentDateTime().toString("ddMMyy_hh_mm_ss").toStdString() <<".wav\"";                    //"/test.wav\"
      ss << ");\n\x0c";
//       m_sclangProcess->write("s.prepareForRecord(\"/drive_d/profiles/linux/mup10sap/Desktop/test.wav\");\n\x0c");
      m_sclangProcess->write(ss.str().c_str());

      m_sclangProcess->write("s.sync;\n\x0c");
      m_sclangProcess->write("s.record;\n\x0c");
//       m_sclangProcess->write(ss.str().c_str());

      status("Recording");
  }else{
      m_sclangProcess->write("s.stopRecording;\n\x0c");
        status("Stopping recording");
  }
}










void MainWindow::on_actionEditScript_triggered()
{
    DebugStream << "Edit hit"<<std::endl;
    QFile file("scrips/basic.lua");
    if( file.open( QIODevice::ReadOnly | QIODevice::Text )){
	DebugStream << "file opened"<<std::endl;
	QTextEdit* editor = new QTextEdit(NULL);
	 QTextStream in(&file);
	  while (!in.atEnd()) {
	    QString line = in.readLine();
	    DebugStream << line.toStdString()<<std::endl;
	    editor->append(line);
	    in.flush();
	  }
	  editor->showNormal();

     }



}


void MainWindow::on_actionVisibility_toggled(bool state)
{
  m_screen->show_visibility(state);
  m_screen->update();
}
void MainWindow::on_actionMass_toggled(bool state)
{
  m_screen->show_mass(state);
  m_screen->update();
}
void MainWindow::on_actionSpeed_toggled(bool state)
{
  m_screen->show_speed(state);
  DebugStream << "speed view state changed"<<std::endl;
  m_screen->update();
}
void MainWindow::on_actionTarget_toggled(bool state)
{
  m_screen->show_target(state);
  m_screen->update();
}

void MainWindow::on_actionStandard_triggered()
{
  qDebug() << "Standard mouse mode";
  setCursor(Qt::CrossCursor);
  g_mouseMode = MM_STANDARD;

}

void MainWindow::on_actionSurface_triggered()
{
  qDebug()<< "Surface mouse mode";
  setCursor(Qt::ClosedHandCursor);
  g_mouseMode = MM_SURFACE;

}


void MainWindow::on_actionShow_Buffers_toggled(bool state)
{
  if(state){
    m_bufferWindow->show();
  }else{
    m_bufferWindow->hide();
  }
}

void MainWindow::on_actionDelete_triggered()
{
    qDebug()<< "Delete";
//     ui->treeNodeView->collapseAll();
    bool prevPlaying = m_playing;
    if(prevPlaying)on_playstop(!m_playing);


    if(m_screen->get_current_mouse_mode() == WorldView::MT_SELECT){
       SelectedNodeObject::NodeSet& nodes = WorldEngine::get_instance().get_selected_object().get_node_set();
	for(SelectedNodeObject::NodeSet::iterator it = nodes.begin(); it != nodes.end() ; ++it){
	  (*it)->kill();
	}


    }else{
      status("Attempting to kill Agent " + QString::number(m_currentlySelectedNode).toStdString());
      //FIXME
      //m_treeModel->
      //     ui->treeNodeView->model().
      TreeNode::get_treenode_by_id(m_currentlySelectedNode).lock().get()->kill();

    }




   WorldEngine::get_instance().tick();
//    ui->treeNodeView->update();
   m_screen->update();
   if(prevPlaying)on_playstop(prevPlaying);
}

void MainWindow::update_screen()
{
  m_screen->update();
}

void MainWindow::on_actionDuplicate_toggled(bool state)
{
    qDebug()<< "Duplicate";
    m_duplicateState = state;
    m_screen->set_duplicate_state(state);
    m_screen->set_duplicate_id(m_currentlySelectedNode);
    status("Duplicating " + QString::number(m_currentlySelectedNode).toStdString());
}

int MainWindow::get_current_index()
{
  return m_currentlySelectedNode;
}



void MainWindow::setup_scales()
{
  /*
    ScalePtr scale1 = ScalePtr(new Scale());
     float scaleData [] = {1,  1.034482759,1.090909091,1.333333333,1.5,1.578947368,1.714285714};
//     float scaleData [] = {1,1.5};
    float baseNote = 40;
    int octaves = 4;
    for(int n = 0;n < 7;++n){
      for(int octave = 0; octave < octaves;++octave){
	      scale1->add_value( baseNote*scaleData[n]*(std::pow(2,octave)));
      }
    }
    scale1->print_scale();
    GridManager::get_instance().add_scale(scale1);
    */
}

void MainWindow::on_moniter_files()
{
  /*
  //TODO store old times and compare the new times against... if different.. load
//   qDebug() << "Monitering files";;
    QDateTime current = QDateTime::currentDateTime();
    QDateTime fileTime;
    int n = 0;
    for(std::set<std::string>::iterator it = m_loadedScriptSet->begin();it!=m_loadedScriptSet->end();++it){
      fileTime = QFileInfo( QString::fromStdString(*it ) ).lastModified();
      qDebug() << "n" << n << " "<< fileTime.toString();
      if( current.time() < fileTime.time()){
	  qDebug()<<"File updated";
      }
      ++n;
    }
      */
}


void MainWindow::setup_working_directory()
{
      QSettings settings;
	 QDir path ( settings.value("scriptpath",QDir::homePath()+QDir::separator()+QString("AgentProjectScripts")).toString());
	 qDebug()<< path;
      if(!path.exists("AgentProjectScripts")){
	  path.mkpath(path.absolutePath());
// 	  QDir::setCurrent(path.absolutePath());
      }

}



void MainWindow::on_actionRemove_Screen_triggered()
{
  qDebug()<< "Attempting to remove screen";
  status("Attempting to remove screen");
  bool state = m_animationTimer->isActive();
  if(state){
      on_playstop(false);
      WorldEngine::get_instance().delete_surface(0);
      on_playstop(true);
  }
}

void MainWindow::on_actionSpawn_triggered()
{
 qDebug()<< "Spawn tool";
 status("Spawn tool selected");
 m_screen->set_mouse_mode(WorldView::MT_SPAWN);
}

void MainWindow::on_actionSelect_triggered()
{
 qDebug() << "Select tool";
 status("Select tool active");
 m_screen->set_mouse_mode(WorldView::MT_SELECT);
}
void MainWindow::on_actionMove_triggered()
{
  qDebug() << "Move tool";
  status("Move tool selected");
  m_screen->set_mouse_mode(WorldView::MT_MOVE);
}
void MainWindow::on_actionDelete_2_triggered()
{
 qDebug() << "Delete tool";
 status("Delete tool selected");
 m_screen->set_mouse_mode(WorldView::MT_DELETE);
}
void MainWindow::on_actionPath_triggered()
{
  qDebug()<<"Path tool ";
  status("Path point tool selected");
  m_screen->set_mouse_mode(WorldView::MT_PATH);
}




void MainWindow::on_actionGet_Help_triggered()
{
  status("Opening help files");
  GreetingDialog* greetdlg = new GreetingDialog(this);
  greetdlg->show();
  greetdlg->raise();
  greetdlg->activateWindow();
}
void MainWindow::on_actionGuide_To_Scripting_triggered()
{
    status("Opening scripting help");
  GreetingDialog* greetdlg = new GreetingDialog(this);
  greetdlg->load_page(":help/scripting.html");
  greetdlg->show();
  greetdlg->raise();
  greetdlg->activateWindow();
}



void MainWindow::on_actionShow_Help_On_Boot_toggled(bool state)
{
  QSettings settings;
  settings.setValue("Greeting",state);
  if(state){
    status("Showing help on boot");
  }{
    status("Help will not show on boot");
  }
}


void MainWindow::on_actionSave_Image_triggered()
{
  QString filePath = QFileDialog::getSaveFileName(this,"Save Image",QDir::homePath(), tr("Images (*.png )"));
  if(! filePath.isEmpty()){
       WorldEngine::get_instance().get_surface(0)->save_surface(filePath.toStdString());
        status("Saving image");
  }


}



void MainWindow::on_actionSetup_Synth_Directory_triggered()
{
  status("Setting up scsynth directory");
  QString folderPath = QFileDialog::getExistingDirectory(this,"Open Synth folder",QDir::home().absolutePath());
  QSettings settings;
  settings.setValue("SynthFolder",folderPath);
  status("SuperCollider binary synth files set to " + folderPath.toStdString());

}

void MainWindow::on_actionLoad_SynthDefs_triggered()
{
  DebugStream << "Load synths"<<std::endl;
  status("Loading SuperCollider binary synth files");
  QSettings settings;
  QString folderPath = settings.value("SynthFolder").toString();
  if(folderPath.isEmpty()){
    QMessageBox::warning(this,"Warning","No Synth folder provided");
    return;

  }
  QDir folder(folderPath);
  QString full =folder.absolutePath() + folder.separator();
  DebugStream << full.toStdString()<<std::endl;
  WorldEngine::get_instance().load_supercollider_synths(full.toStdString());
}

void MainWindow::on_actionKill_Server_triggered()
{
    status("Killing SuperCollider server");
  WorldEngine::get_instance().kill_server();
}

void MainWindow::on_actionStop_Server_triggered()
{
    status("Stopping SuperCollider server");
  WorldEngine::get_instance().clear_server();
}
//FIXME
void MainWindow::load_bundled_synths()
{
//   QDir path(QCoreApplication::);
//   DebugStream << path.absolutePath().toStdString()<<std::endl;
}


void MainWindow::on_actionClear_AutoLoad_Scripts_triggered()
{

    try{
        status("Clearing autoload scripts");
      QSettings settings;
      qDebug()<< settings.fileName();
       QStringList keys = settings.allKeys();
      qDebug()<< keys;

      int size = settings.beginReadArray("UserScripts");
      DebugStream << "Size of script array "<< size <<std::endl;
      for(int n = 0; n < size ;++n){
	settings.setArrayIndex(n);
	settings.remove("script");
      }
      settings.endArray();

      WorldEngine::get_instance().reset_autoload_states();



    }catch(...){

    }

}


void MainWindow::on_actionLoad_Buffer_triggered()
{
    status("Loading audio file in SuperCollider");
  QString filePath = QFileDialog::getOpenFileName(this,"Load soundfile",QDir::home().absolutePath(),("Soundfiles (*.aif *.wav *.aiff)"));
  if(filePath.isEmpty())return;
  QDir file(filePath);
  int bufferNum = QInputDialog::getInt(this,"Buffer Select","Pick a buffer number to assign "+ file.path() + " to");
  WorldEngine::get_instance().read_buffer_command(bufferNum,filePath.toStdString());
  status("Attempted to load " + filePath.toStdString() + " in buffer " + QString::number(bufferNum).toStdString());
}



void MainWindow::on_actionAi_Names_toggled(bool state)
{
  m_screen->set_show_name_state(state);
  if(state){
    status("Showing Agent names");
  }else{
    status("Hiding Agent names");
  }
}


void MainWindow::on_actionPaths_toggled(bool state)
{
  m_screen->set_draw_paths_state(state);
if(state){
    status("Showing paths");
  }else{
    status("Hiding paths");
  }

}



void MainWindow::on_actionSave_Path_Data_triggered()
{
  QString filePath = QFileDialog::getSaveFileName(this,"Save path data",QDir::homePath(),"Xml (*.xml)");
  if(filePath.isEmpty())return;
  QFileInfo t(filePath);
  if(t.suffix() != ".xml"){
    DebugStream << "Filename does not end in xml"<<std::endl;
    filePath.append(".xml");
  }
  try{
    PathManager::get_instance().save_path_database(filePath.toStdString());
    status("Saved paths to " + filePath.toStdString());
  }catch(...){
    //this shoudl be safer
  }
}


void MainWindow::on_actionLoad_Path_Data_triggered()
{
    status("Attempting to load paths");
  QString filePath = QFileDialog::getOpenFileName(this,"Load path data",QDir::homePath(),"Xml (*.xml)");
  if(filePath.isEmpty())return;
  try{
    PathManager::get_instance().load_path_database(filePath.toStdString());
    m_screen->update();
    status("Loaded paths in " + filePath.toStdString());
  }catch(std::exception& e){
    QMessageBox::warning(this,"Error Loading Paths",QString::fromStdString( e.what()));
  }

}


void MainWindow::on_actionOpen_Server_on_Boot_toggled(bool state)
{
  QSettings settings;
  settings.setValue("BootServer",state);
  DebugStream << "server on boot toggled"<<std::endl;

  if(state){
    status("Internal server will start on boot");
  }else{
    status("Internal server will NOT start on boot");

  }

}


void MainWindow::on_actionBoot_Server_2_triggered()
{
  status("Attempting to boot internal server");

  DebugStream << "boot server triggered"<<std::endl;
  int portNum = QInputDialog::getInt(this,"Enter port number","port",8001,1000,10000);
  DebugStream << portNum <<std::endl;
  int portNo = WorldEngine::get_instance().spawn_server(portNum);
  std::stringstream ss;
  ss << "Booted server on port "<< portNo;
  QMessageBox::warning(this,"Notice", ss.str().c_str());

 status("Internal server booted");

}


void MainWindow::print_to_lua_console(std::string text)
{
  ui->luaOutput->appendPlainText(QString::fromStdString(text));


}


void MainWindow::on_actionAi_Velocities_toggled(bool state)
{
  std::cout << "Show AI Velocities set " << state<<std::endl;
  m_screen->set_show_velocities_state(state);

  if(state){
    status("Showing Velocities");
  }else{
    status("Hiding Velocities");
  }

}

void MainWindow::update_treemodel()
{
  std::cout << "Update Treemodel"<<std::endl;
//   ui->treeNodeView->collapseAll();
  //ui->treeNodeView->reset();
  m_treeModel->update();
}

void MainWindow::status(std::string text){
	statusBar()->showMessage(QString::fromStdString(text),1000);
}

void MainWindow::update_path_window()
{
  ui->PathList->clear();
  for(int n = 0 ; n < PathManager::get_instance().get_paths_size(); ++n){
      ui->PathList->addItem(QString::number(n));
  }
}


void MainWindow::on_actionCreate_New_Path_triggered()
{
  status("Creating new path");
  PathManager::get_instance().add_path(PathPtr(new Path()));
  update_path_window();


}
void MainWindow::on_actionRemove_Selected_Path_triggered()
{
  int index = ui->PathList->currentRow();

  status("Attempting to remove path " + QString::number(index).toStdString());
  int newIndex = index;

  if(m_screen->get_selected_path_index() == index && index > 0){
    newIndex = m_screen->get_selected_path_index() - 1;
    PathManager::get_instance().remove_path(index);
  }else if(index == 0){
    PathManager::get_instance().get_path(index)->remove_all();
  }

  update_path_window();
  ui->PathList->setCurrentRow(newIndex);

  m_screen->update();

}

void MainWindow::path_index_changed(int index)
{
  status("Path " + QString::number(index).toStdString() + " selected");
  m_screen->set_selected_path_index(index);
  m_screen->update();
}



void MainWindow::update_image_window()
{
  if(WorldEngine::get_instance().get_number_of_surfaces() ==0)return;
  ui->SurfaceList->clear();
  std::cout << WorldEngine::get_instance().get_number_of_surfaces() <<std::endl;
  for(int n = 0 ; n < WorldEngine::get_instance().get_number_of_surfaces(); ++n){
    if(WorldEngine::get_instance().get_surface(n)){
      ui->SurfaceList->addItem(QString::fromStdString( WorldEngine::get_instance().get_surface(n)->get_name()));
    }
  }
  int selected = m_screen->get_selected_surface_index();
  ui->SurfaceList->setCurrentRow(selected);

}
void MainWindow::surface_index_changed(int index)
{
  std::cout << index <<std::endl;
  m_screen->set_selected_surface_index(index);
  m_screen->update();
}

void MainWindow::on_actionRemove_Screen_2_triggered()
{
  int index = ui->SurfaceList->currentRow();
  if(index < 0)return;
  status("Attempting to remove Screen " + QString::number(index).toStdString() );

  int newIndex =  m_screen->remove_selected_surface();
  ui->SurfaceList->setCurrentRow(newIndex);

  update_image_window();
}
void MainWindow::on_downloaded_file(){

  QString data(m_fileDownloader->downloadedData());
  if(data.isEmpty())return;

  bool status = false;

  float versionNumber = data.toFloat(&status);
  float currentVersion = get_version_float();

  if(currentVersion < versionNumber){

    QMessageBox::information(this, tr("New Agent Tool Version Available"), tr("The Agent Tool Version ") + data + tr(" is available at www.theagentool.co.uk\nYou are currently using ")  + QString::fromStdString(get_version_string()));

  }

}
