#ifndef __MAINWINDOW_HPP__
#define __MAINWINDOW_HPP__

#include <QMainWindow>
#include "WorldView.hpp"
#include <QModelIndex>
#include <QListWidgetItem>
#include "PropertyDelegate.hpp"
#include <QThread>
#include <QDir>
#include <vector>
#include <set>
#include <QActionGroup>
#include <QPlainTextEdit>

class TreeNodeModel;namespace Ui {
  class MainWindow;
}
class QTimer;
class QProcess;
class QFileSystemWatcher;
class PropertyHeader;
class FileDownloader;


class Sleeper : public QThread
{
public:
    static void usleep(unsigned long usecs){QThread::usleep(usecs);}
    static void msleep(unsigned long msecs){QThread::msleep(msecs);}
    static void sleep(unsigned long secs){QThread::sleep(secs);}
};

enum MouseMode{
    MM_STANDARD =0,
    MM_SURFACE

};
static MouseMode g_mouseMode = MM_STANDARD;

class BufferWindow;



class MainWindow :public QMainWindow{
	Q_OBJECT
	WorldView* m_screen;
	QTimer* m_animationTimer,*m_fileMonitorTimer;
	int m_refreshSpeed;
	PropertyDelegate* m_propertyDelegate;
	int m_currentlySelectedNode;
	QProcess* m_sclangProcess;
	QDir m_recordingFolder,m_baseFolder;
	BufferWindow* m_bufferWindow;
	TreeNodeModel* m_treeModel;
	bool m_duplicateState;
	QFileSystemWatcher* m_fileWatcher;
	PropertyHeader* m_propertyHeader;
	QActionGroup* m_mouseToolGroup;
	bool m_playing;
	MainWindow();
	static MainWindow* s_instance;

  FileDownloader* m_fileDownloader;


public:
	static MainWindow* get_instance();


	virtual ~MainWindow();
	void populate_ai_list_widget();
	int get_current_index();
	void selected_node(int id);
	void set_play_state(bool state);
	bool get_play_state();
	void update_screen();
	void print_to_lua_console(std::string text);
  void status(std::string text);

	void update_treemodel();
	void update_path_window();
	void update_image_window();
private:
    Ui::MainWindow *ui;
    void populate_toolbar();
    void restore_data();
    void setup_scales();
    void append_script(QString filepath,bool query=false);
    void load_boot_scripts();
    void setup_working_directory();
    void load_script(QString filePath, bool autoLoad);
    void load_bundled_synths();
    void update_autoload_scripts();


protected slots:
    void on_reload_lua();
   // void on_actionReloadLua_triggered();
    void on_actionEdit_OSC_triggered();
    void on_about();
    void on_import_image();
    void on_playstop(bool state);
    void on_speedChanged(int speed);
    void on_timer();
    void currentAiChanged ( );
    void on_actionEditScript_triggered();
    void on_actionReloadLua_triggered();
    void on_actionVisibility_toggled(bool state);
    void on_actionSpeed_toggled(bool state);
    void on_actionMass_toggled(bool state);
    void on_actionTarget_toggled(bool state);
    void on_treeView_clicked ( const QModelIndex & index );
    void on_treeView_doubleclicked ( const QModelIndex & index );
    void on_actionImport_Video_triggered();
    void on_actionStart_Stream_triggered();
    void on_actionClear_Nodes_triggered();
    void on_actionEvaluate_triggered();
    void on_actionAppendSCFile_triggered();
    void on_readyRead();
    void on_actionStandard_triggered();
    void on_actionSurface_triggered();
    void on_actionRecord_toggled(bool state);
    void on_actionSet_Record_Directory_triggered();
    void on_actionShow_Buffers_toggled(bool state);
    void on_actionDelete_triggered();
    void on_actionDuplicate_toggled(bool state);
    void on_actionAppend_Lua_Script_triggered();
    void on_moniter_files();
    void on_file_edited_externally(QString filename);
    void on_actionRemove_Screen_triggered();
    void on_actionSpawn_triggered();
    void on_actionSelect_triggered();
    void on_actionMove_triggered();
    void on_actionDelete_2_triggered();
    void on_actionPath_triggered();
    void on_actionGet_Help_triggered();
    void on_actionShow_Help_On_Boot_toggled(bool state);
    void on_actionGuide_To_Scripting_triggered();
    void on_actionCopy_Default_Scripts_triggered();
    void on_actionSave_Image_triggered();
    void on_actionSetup_Synth_Directory_triggered();
    void on_actionLoad_SynthDefs_triggered();
    void on_actionKill_Server_triggered();
    void on_actionStop_Server_triggered();
    void on_actionClear_AutoLoad_Scripts_triggered();
    void on_actionLoad_Buffer_triggered();
    void on_actionAi_Names_toggled(bool state);
    void on_actionPaths_toggled(bool state);
    void on_actionSave_Path_Data_triggered();
    void on_actionLoad_Path_Data_triggered();
    void on_actionOpen_Server_on_Boot_toggled(bool state);
    void on_actionBoot_Server_2_triggered();
    void on_actionBasic_Background_triggered();
    void on_actionAi_Velocities_toggled(bool state);
    void on_actionCreate_New_Path_triggered();
    void on_actionRemove_Selected_Path_triggered();
    void on_actionRemove_Screen_2_triggered();


    void path_index_changed(int index);
    void surface_index_changed(int index);

    void on_downloaded_file();



};

QString get_text_from_file(QString file);

#endif
