function ai_Spiral()
  initai()
  changeMotionModel("Physics")
  setP("Mass",0.001)
  for n = 0, 10000,1 do
    local x = math.sin(n)  * (n*10)
    local y = math.cos(n ) * (n*10)
    setP("Velocity", Vec2.new(x,y))
    pause(0.1)
  end
  die()
end

function ai_SpiralSpawner()
  initai()
  while true do
    spawnRelative(getID(),1,"ai_Spiral",Vec2.new(0,0))
    pause(0.1)
  end
end
----------------------------
function ai_HelloWorld()
  initai()
  addP("freq",220)
  addP("amp",0.8)
  setP("SynthDef","default")
  pause(1)
  die()
end

----------------------------

function ai_PropertyExample()
  initai()
  addP("Hello","world")
  local extraProperties = {A = 1, B = Vec2.new(10,10), C= 3.14159 }
  addP(extraProperties)
  setP("A",13)
  local id = getID()
  setPID("Hello","information",id)
  pause(5)
  die()
end



-----------------------------

function ai_SendOscExample()
  initai()
  send("127.0.0.1",9000,"/hello", "pure data", 3.14159)
  setP("Velocity",Vec2.new(10,10))
  for i = 1, 5 do
    local pos = getP("pos")
    send("127.0.0.1",9000,"/position", "x", pos:x(), "y",pos:y())
    pause(1)
  end
  die()
end

------------------------------

function ai_SpawnExample()
  initai()
  local childProps = {A = 1,B = "Hello", C = Vec2.new(100,50)}
  local id = getID()
  spawnRelative(id,1,"ai_Emptygrain",0,0)
  spawnRelativeP(childProps,id,1,"ai_Emptygrain",10,10)
  pause(5)
  die()
end


--------------------------------

function ai_BasicGrain()
  initai()
  local pos = getP("pos")
  local worldsize = getWorldSize()
  local props = {freq =pos:y(),pan = pos:x() / worldsize:x(),amp = 0.2,dur = 0.1}
  addP(props)
  setP("SynthDef","default")
  while true do
    pos = getP("pos")
    setP("freq",pos:y())
    setP("pan", ((pos:x() / worldsize:x())*2)-1 )
  end
  die()
end

function ai_BasicGranular()
  initai()
  addP("rate",0.01)
  setP("Velocity",Vec2.new(50,0))
  local id = getID()
  while true do
    spawnRelative(id,1,"ai_BasicGrain",0,0)
    pause(getP("rate"))
  end
  die()

end


-------------------------------------


function ai_RecursiveSpawner()
  initai()
  local childProps = {depth = 4}
  local id = getID()
  spawnRelativeP(childProps,id,1,"ai_Recursor",0,0)
  pause(10)
  die()
end

function ai_Recursor()
  initai()
  local currentDepth = getP("depth")
  print(currentDepth)
  local id = getID()
  if( currentDepth > 0 ) then
    for i =1, 5,1 do
      if i % 3 == 0 then
        local childProps = {depth = currentDepth-1}
        local childId = spawnRelativeP(childProps,id,1,"ai_Recursor",-20 + (i*10),20)
        setPID(childProps,childId)
      end
    end
  end
  while true do
    coroutine.yield()
  end
  die()
end

----------------------------------------

function ai_LSystem()
  initai()
  local childProps = {depth = 12, theta=3.141592}
  local pos = getP("pos")
  spawnRelativeP(childProps,0,1,"ai_LSub",pos:x(),pos:y())
  die()
end

function ai_LSub()
  initai()
  pause(0.01)
  local pos = getP("pos")
  print(pos)
  local args = {freq = x,SynthDef = "default"}
  addP("freq", (pos:x()+pos:y())/2.0 )
  addP("amp",0.01)
  setP("SynthDef","default")
  if getP("depth") > 0 then
    pause(0.1 + math.random()*0.2)
    if math.random() > 0.3 then
      local t = ((math.random()*2 )- 1) * 0.5 + getP("theta")
      local cPos = pos + Vec2.new(math.sin(t)*25,math.cos(t)*25)
      local childProps = {depth = getP("depth")-1,theta = t}
      drawLine(currentScreen(),pos,cPos,math.random(10,255),math.random(10,255),math.random(10,255))
      spawnRelativeP(childProps,0,1,"ai_LSub",cPos:x(),cPos:y())
    end
    if math.random() > 0.3 then
      local t =((math.random()*2 )- 1) * 0.5 + getP("theta")
      local cPos = pos + Vec2.new(math.sin(t)*25,math.cos(t)*25)
      local childProps = {depth = getP("depth")-1,theta = t}
      drawLine(currentScreen(),pos,cPos,math.random(0,255),math.random(0,255),math.random(0,255))
      spawnRelativeP(childProps,0,1,"ai_LSub",cPos:x(),cPos:y())
    end
  end
  die()
end

----------------------------------------
function ai_GravityExample()
  initai()
  changeMotionModel("Physics")
  setP("Mass",0.001)
  local pos = getP("pos")
  local worldsize = getWorldSize()
  local props = {bufNum = 0 , pan =(  (pos:x() / worldsize:x()) * 2.0) -1, rate = pos:y()/261.6255,dur = dur,amp = 0.1 }
  addP(props)
  setP("SynthDef","player")
  while true do
    pos = getP("pos")
    worldsize = getWorldSize()
    local amp =  math.pow( getMeanCol(currentScreen()) /255,2 )
    setP("amp",amp)
    setP("pan",(  (pos:x() / worldsize:x()) * 2.0) -1)
    setP("rate",pos:y()/261.6255 )
    pause(0.01)
  end
  die()
end


----------------------------------------

function ai_MetaAgent()
  initai()
  local props = {freq = 440, amp = 0.3, dur = 60}
  addP(props)
  setP("Velocity",Vec2.new(100,0))
  setP("SynthDef","default")
  while true do
    local worldSize = getWorldSize()
    local position = getPosition()
    local frequency = (position:y() / worldSize:y()) * 10000
    local amp =  math.pow( getMeanCol(currentScreen()) /255,2 )
    setP("freq",frequency)
    setP("amp", amp)
    if position:x() >= worldSize:x() then
      setP("pos",Vec2.new(0,position:y()))
    end
    coroutine.yield()
  end
end
-----------------------------------------
function ai_MetaPlayerAgent()
  initai()
  local props = {rate = 1, amp = 0.3, dur = 60}
  addP(props)
  setP("Velocity",Vec2.new(100,0))
  setP("SynthDef","player")
  while true do
    local worldSize = getWorldSize()
    local position = getPosition()
    local amp =  math.pow( getMeanCol(currentScreen()) /255,2 )
    setP("rate", position:y()/261.6255)
    setP("amp", amp)
    if position:x() >= worldSize:x() then
      setP("pos",Vec2.new(0,position:y()))
    end
    coroutine.yield()
  end
end
-----------------------------------------
function ai_MetaHarmSpacer()
  initai()
  local id = getID()
  for i = 1,10,1 do
    local position = 49.0 * i;
    spawnRelative(id,1,"ai_MetaAgent",0,position)
  end
  pause(10)
  die()
end
----------------------------------------
-----------------------------------------
function ai_MetaPlayerHarmSpacer()
  initai()
  local id = getID()
  for i = 1,10,1 do
    local position = 49.0 * i;
    spawnRelative(id,1,"ai_MetaPlayerAgent",0,position)
  end
  pause(10)
  die()
end
---------------------------------------------
function ai_HarmonicSpacerPlayer()
  initai()
  local noOfChildren = 10
  local id = getID()
  addP("ChildVelocity",Vec2.new(100,0) )
  for i = 1, noOfChildren + 1 do
    local position = 49.0 * i;
    spawn(id,1,"ai_MetaAgent",0,position)
  end
  while true do
    for i = 0 , numberOfChildren() - 1  do
      local currentVel = getP("ChildVelocity")
      setPID("Velocity", currentVel, getChild(i) )
    end
    coroutine.yield()
  end
  die()
end
----------------------------------------------------


function ai_MetaComplexHarmSpacer()
  initai()
  local id = getID()
  --c1
  local base = 32
  for octave = 1, 5, 1 do
  local step = ((base*octave) * 2) / 12
    for i = 1,12,1 do
      local position = (base*octave) + (step*i) ;
      spawnRelative(id,1,"ai_MetaPlayerAgent",0,position)
    end
  end
  pause(10)
  die()
end


---------------------------------------------

function ai_PlayerMorpher()
  initai()
  local props = {bufAAmp = 0.3, bufBAmp = 0.3,bufCAmp = 0.3,dur = 10,mix = 0.5,room=0.5}
  addP(props)
  setP("Velocity",Vec2.new(100,0))
  setP("SynthDef","playerMorpher")
  while true do
    local worldSize = getWorldSize()
    local position = getPosition()
    local colour = getRGBA(0)
    setP("bufAAmp",colour:r() / 255)
    setP("bufBAmp",colour:g() / 255)
    setP("bufCAmp",colour:b() / 255)
    setP("mix",position:x()/worldSize:x())
    setP("room",position:y()/worldSize:y())
    if position:x() >= worldSize:x() then
      setP("pos",Vec2.new(0,position:y()))
    end
    coroutine.yield()
  end
end



--------------------------------------

function ai_RedGen()
  initai()
  changeMotionModel("Physics")
  addP("AgentToSpawn","ai_GravityExample")
  setP("Velocity",Vec2.new(200,0))
  setP("image",6)
  setP("dur",30)
  while true do
    local pos =  getP("pos")
    if getRedFromPoint(currentScreen(),pos) > 150 then
     spawnRelative(getID(),1,getP("AgentToSpawn"),0,0)
     pause(0.1)
   end
   coroutine.yield()
 end
end



------------------------------------

function colour_threshold( targetColour, threshold , id  )
  local col = getRGBAID(currentScreen(),id)
  if col:r() < (targetColour:r() - threshold) or col:r() > (targetColour:r() + threshold) then
    return false
  end
  if col:g() < (targetColour:g() - threshold) or col:g() > (targetColour:g() + threshold) then
    return false
  end
  if col:b() < (targetColour:b() - threshold) or col:b() > (targetColour:b() + threshold) then
    return false
  end
  return true
end


function ai_ColourDetector()
  initai()
  changeMotionModel("Physics")
  addP("AgentToSpawn","ai_GravityExample")
  setP("Velocity",Vec2.new(25,0))
  setP("image",6)
  setP("dur",10)
  while true do
    local found = colour_threshold(RGBA.new(255,0,0,125),10,getID())
    if found == true then
     spawnRelative(getID(),1,getP("AgentToSpawn"),0,0)
     pause(0.1)
   end
   coroutine.yield()
 end
 die()
end
-----------------------------------------------------------------------------------------
function colour_threshold( targetColour, threshold , id  )
  local col = getRGBAID(currentScreen(),id)
  if col:r() < (targetColour:r() - threshold) or col:r() > (targetColour:r() + threshold) then
    return false
  end
  if col:g() < (targetColour:g() - threshold) or col:g() > (targetColour:g() + threshold) then
    return false
  end
  if col:b() < (targetColour:b() - threshold) or col:b() > (targetColour:b() + threshold) then
    return false
  end
  return true
end

function colour_spawner(targetColour,threshold,id,agentName)
  local found = colour_threshold(targetColour,threshold,id)
  if found == true then
    spawnRelative(id,1,agentName,0,0)
  end
end

function ai_ColourSpawner()
  initai()
  changeMotionModel("Physics")
  setP("Velocity",Vec2.new(25,0))
  setP("image",6)
  setP("dur",10)
  local id = getID()
  while true do
    colour_spawner(RGBA.new(255,0,0,120),20,id,"ai_GrainA")
    colour_spawner(RGBA.new(0,255,0,120),20,id,"ai_ColourGranulator")
    colour_spawner(RGBA.new(0,0,255,120),20,id,"ai_GravityGrain")
    coroutine.yield()
  end
  die()
end



-----------------------------------------------------------------------------------------

function ai_GravityGrain()
  initai()
  changeMotionModel("Physics")
  setP("Mass",0.001)
  local pos = getP("pos")
  local worldsize = getWorldSize()
  local props = {bufNum = 0 , pan =(  (pos:x() / worldsize:x()) * 2.0) -1, rate = pos:y()/261.6255,dur = math.random(0.1,4),loop = 0,amp = 0.1 }
  addP(props)
  setP("SynthDef","player")
  while true do
    pos = getP("pos")
    worldsize = getWorldSize()
    local amp =  math.pow( getMeanCol(currentScreen()) /255,2 )
    setP("amp",amp)
    setP("pan",(  (pos:x() / worldsize:x()) * 2.0) -1)
    setP("rate",pos:y()/261.6255 )
    pause(0.01)
  end
  die()
end



-----------------------------------------------------------------------------------------

function ai_RedGenerator()
  initai()
  changeMotionModel("Physics")
  addP("AgentToSpawn","ai_GravityGrain")
  setP("Velocity",Vec2.new(math.random(150,200),0))
  setP("image",6)
  setP("dur",30)
  while true do
    local pos =  getP("pos")
    if getRedFromPoint(currentScreen(),pos) > 150 then
     spawnRelative(getID(),1,getP("AgentToSpawn"),0,0)
     pause(0.1)
   end
   coroutine.yield()
 end
end

----------------------------------------


function ai_GrainA()
  initai()
  local pos = getP("pos")
  local worldsize = getWorldSize()
  local props = {speed =pos:y()/261.625 ,startpos = pos:x() / worldsize:x(),amp = 0.5,bufNum = 0 }
  addP(props)
  setP("SynthDef","grain")
  while true do
    pos = getP("pos")
    local col = getRGBA(currentScreen())
    setP("speed",pos:y()/261.625)
    setP("startpos", pos:x() / worldsize:x() )
    setP("amp",math.pow( getMeanCol(currentScreen()) /255,2 ))
    coroutine.yield()
  end
  die()
end


function ai_ColourGranulator()
  initai()
  local id = getID()
  local props = {rate = 0.01, childAgent = "ai_GrainA"}
  addP(props)
  setP("Velocity",Vec2.new(20,0))
  changeMotionModel("Physics")
  while true do
    local mean = getMeanCol(currentScreen())
    local col = getRGBA(currentScreen())
    local x = math.random(col:r()* -1,col:r()) * 0.5
    local y = math.random(col:g()* -1,col:g()) * 0.5
    spawnRelativeP({dur = (mean /255)*2},id,1,getP("childAgent"), x,y)
    pause( getP("rate"))
  end
  die()
end
-----------------------------------
function ai_GrainB()
  initai()
  local pos = getP("pos")
  local worldsize = getWorldSize()
  local dur = 1
  addP("speed", pos:y()/261.625 )
  addP("startpos", pos:x() / worldsize:x() )
  addP("amp",0.2)
  addP("bufNum",0)
  setP("SynthDef","grain")
  local t = 0
  while t < dur do
    pos = getP("pos")
    setP("speed",pos:y()/261.625)
    setP("startpos", pos:x() / worldsize:x() )
    t = t + 0.01
    pause(0.01)
  end
  die()
end


function ai_Granulator()
  initai()
  local id = getID()
  local props = {rate = 0.01, childAgent = "ai_GrainA",xRandom = 3,yRandom = 3,childDur = 0.5}
  addP(props)
  setP("Velocity",Vec2.new(20,0))
  changeMotionModel("Physics")
  while true do
    local xRandom = getP("xRandom")
    local yRandom = getP("yRandom")
    local x = math.random(xRandom* -1,xRandom) * 0.5
    local y = math.random(yRandom* -1,yRandom) * 0.5
    spawnRelativeP({dur = getP("childDur")},id,1,getP("childAgent"), x,y)
    pause( getP("rate"))
  end
end
-------------------------------------------------------------

-----------------------------------------------------------------

function exponentialSpawn(overalltime,numspawns,ratio,parentid,type)
  local previous = 0
  for i =0,numspawns,1 do
    spawnRelative(parentid,1,type,0,0)
    local absT = (math.pow(i/numspawns,ratio)*overalltime )
    local deltaT = absT -   previous
    previous = absT
    pause( deltaT )
  end
end


function ai_RampGenerator()
  initai()
  local props  = {rampTime = 10, numberOfSpawns = 30, ratio = 0.25,childType = "ai_GrainA",run = 1}
  addP(props)
  while getP("run") > 0 do
    exponentialSpawn(getP("rampTime"),getP("numberOfSpawns"),getP("ratio"),getID(),getP("childType"))
    pause(0.1)
  end
  die()
end


-----------------------------------------------------------------------
function ai_TrackWorld()
  initai()
  changeMotionModel("Physics")
  local pos = getP("pos")
  local worldsize = getWorldSize()
  local props = {Mass = math.random(0.01,0.6),amp = 0.4,freq =pos:y()/261.625 ,pan =  ( (  pos:x() / worldsize:x())*2) -1 ,Attraction = 2,dur = 5}
  addP(props)
  setP("Velocity",Vec2.new(math.random(-30,30),math.random(-30,30)))
  setP("SynthDef","default")
  while true do
    pos = getP("pos")
    setP("freq",pos:y()/261.625)
    setP("pan", ( (  pos:x() / worldsize:x())*2) -1)
    setP("amp",math.pow( getMeanCol(currentScreen()) /255,2 ))
    drawCircle(currentScreen(),pos,3,RGBA.new(255,0,0,150),-1)
    local targetPos = getPID("pos",0)
    local dir = (targetPos - pos):norm()
    setP("Thrust",dir* getP("Attraction"))
    coroutine.yield()
  end
  die()
end

-------------------------------------------------------------------------

function ai_TrackWorldFlock()
  initai()
  setPID("pos",Vec2.new(400,400),0)
  local props = {FlockAttraction = 0.9,run = 1}
  addP(props)
  for i = 1, 10 , 1 do
    spawn(getID(),1,"ai_TrackWorld",math.random(0,1000),math.random(0,1000))
  end
  pause(1)
  while getP("run") > 0 do
    local attract = getP("FlockAttraction")
    print(numberOfChildren())
    for i = 0 , numberOfChildren()-1, 1 do
      setPID("Attraction",attract, getChild(i))
    end
    coroutine.yield()
  end
end



-----------------------------------------------------------------------------

function ai_BasicFollower()
  initai()
  changeMotionModel("Physics")
  local pos = getP("pos")
  local worldsize = getWorldSize()
  local props = {Mass = math.random(0.01,0.6),amp = 0.4,freq =pos:y()/261.625 ,pan =  ( (  pos:x() / worldsize:x())*2) -1 ,attraction = 1.2,dur = 20}
  addP(props)
  setP("Velocity",Vec2.new(math.random(-30,30),math.random(-30,30)))
  setP("SynthDef","default")
  while true do
    pos = getP("pos")
    setP("freq",pos:y()/261.625)
    setP("pan",  ( (  pos:x() / worldsize:x())*2) -1 )
    setP("amp",math.pow( getMeanCol(currentScreen()) /255,2 ))
    drawCircle(currentScreen(),pos,3,RGBA.new(255,0,0,150),-1)
    local targetId = getNearest(pos)
    local targetPos = getPID("pos",targetId)
    local dir = (targetPos - pos):norm()
    setP("Thrust",dir* getP("attraction"))
    coroutine.yield()
  end
  die()
end

--------------------------------------------------------------------------------------


function ai_BasicGroupFollower()
  initai()
  changeMotionModel("Physics")
  local pos = getP("pos")
  local worldsize = getWorldSize()
  local props = {Mass = math.random(0.1,0.6),amp = 0.4,freq =pos:y()/261.625 ,pan =  ( (  pos:x() / worldsize:x())*2) -1 ,attraction = 1.2,range = 200,dur = 20}
  addP(props)
  setP("Velocity",Vec2.new(math.random(-30,30),math.random(-30,30)))
  setP("SynthDef","default")
  while true do
    pos = getP("pos")
    setP("freq",pos:y())
    setP("pan",  ( (  pos:x() / worldsize:x())*2) -1 )
    local nearby = getNearby(getP("range"))
    local avgPos = Vec2.new(0,0)
    local pos = getP("pos")
    for i = 1, #nearby do
      avgPos = avgPos + getPID("pos",nearby[i])
    end
    avgPos = avgPos/#nearby
    if avgPos:x() > 0 and avgPos:y() > 0 then
      local dir = (avgPos - pos):norm()
      setP("Thrust",dir* getP("attraction") * 0.5)
    end
    coroutine.yield()
  end
  die()
end

--------------------------------------------------------------
function move_to_average(range,attraction)
  local nearby = getNearby(range)
  local avgPos = Vec2.new(0,0)
  local pos = getP("pos")
  for i = 1, #nearby do
    avgPos = avgPos + getPID("pos",nearby[i])
  end
  avgPos = avgPos/#nearby
  if avgPos:x() > 0 and avgPos:y() > 0 then
    local dir = (avgPos - pos):norm()
    setP("Thrust",dir* attraction * 0.5)
  end

end

function move_to_average_type(range,attraction,type)
  local nearby = getNearby(range)
  local avgPos = Vec2.new(0,0)
  local pos = getP("pos")
  for i = 1, #nearby do
   if getTypeID(nearby[i]) == type then
     avgPos = avgPos + getPID("pos",nearby[i])
     print("found")
   end
 end
 avgPos = avgPos/#nearby
 if avgPos:x() > 0 and avgPos:y() > 0 then
  local dir = (avgPos - pos):norm()
  setP("Thrust",dir* attraction * 0.5)
end

end


function ai_GroupGranulator()
  initai()
  local id = getID()
  local props = {Mass = 0.3,rate = 0.01, childAgent = "ai_GrainB",xRandom = 3,yRandom = 3,childDur = 0.5,range = 300,attraction = 4.3}
  addP(props)
  setP("Velocity",Vec2.new(20,0))
  changeMotionModel("Physics")
  while true do
    local xRandom = getP("xRandom")
    local yRandom = getP("yRandom")
    local x = math.random(xRandom* -1,xRandom) * 0.5
    local y = math.random(yRandom* -1,yRandom) * 0.5
    spawnRelativeP({dur = getP("childDur")},id,1,getP("childAgent"), x,y)
    move_to_average_type(getP("range"),getP("attraction"),"ai_GroupGranulator")
    pause( getP("rate"))
  end
  die()
end




-----------------------------------------------------------------------------

function boid_calculation(position,range,cohesion,alignment,separation,type)
  local nearby = getNearby(range)
  local avgPos = Vec2.new(0,0)
  local align = Vec2.new(0,0)
  if type == not nil then
    for i = 1, #nearby do
     if getTypeID(nearby[i]) == type then
       avgPos = avgPos + getPID("pos",nearby[i])
       align = align +  getPID("Thrust",nearby[i])
     end
   end
  else
    for i = 1, #nearby do
      avgPos = avgPos + getPID("pos",nearby[i])
      align = align +  getPID("Thrust",nearby[i])
    end
  end
  avgPos = avgPos/#nearby
  align = align/#nearby
  local closest = getNearest(position)
  local closetPos = getPID("pos",closest)
  local closestDir = (closetPos - position):norm() * -1
  if avgPos:x() > 0 and avgPos:y() > 0 then
    local dir = (avgPos - position):norm()
    setP("Thrust",(dir * cohesion) + (align:norm()* alignment ) + (closestDir*separation ) )
  end
end


function ai_BoidGranulator()
  initai()
  local id = getID()
  local props = {Mass = 0.1,rate = 0.01, childAgent = "ai_GrainB",xRandom = 3,yRandom = 3,childDur = 0.5,range = 300,cohesion = 0.7, alignment = 1,separation = 0.1}
  addP(props)
  setP("Velocity",Vec2.new(20,0))
  changeMotionModel("Physics")
  while true do
    local xRandom = getP("xRandom")
    local yRandom = getP("yRandom")
    local x = math.random(xRandom* -1,xRandom) * 0.5
    local y = math.random(yRandom* -1,yRandom) * 0.5
    spawnRelativeP({dur = getP("childDur")},id,1,getP("childAgent"), x,y)  
    boid_calculation(getP("pos"),getP("range"),getP("cohesion"),getP("alignment"),getP("separation"),ai_BoidGranulator)
    pause( getP("rate"))
  end
  die()
end

----------------------------------------------------------------------------
function ai_Speaker()
  initai()
  addP("speaker","left")
  while true do
    coroutine.yield()
  end
  die()
end

function ai_SpeakerFiveOneSpawner()
  initai()
  local left = spawn(0,1,"ai_Speaker",100,100)
  local right =spawn(0,1,"ai_Speaker",300,100)
  local centre = spawn(0,1,"ai_Speaker",200,75)
  local leftS = spawn(0,1,"ai_Speaker",100,300)
  local rightS = spawn(0,1,"ai_Speaker",300,300)
  pause(0.1)
  setPID("speaker","left",left)
  setPID("speaker","right",right)
  setPID("speaker","centre",centre)
  setPID("speaker","leftS",leftS)
  setPID("speaker","rightS",rightS)
  pause(1)
  die()
end

function calculate_five_one_coefficients()
 local numWorldChildren = numberOfChildrenID(0)
 local currentChildID = 0
 local speakerID = ""
 local delta = Vec2.new(0,0)
 local amp = 0
 for i = 0 , numWorldChildren-1, 1 do
   amp = 0
   currentChildID = getChildID(i,0)
   if getTypeID(currentChildID) == "ai_Speaker" then
    speakerID =  getPID("speaker",currentChildID)
    amp = (getPID("pos",currentChildID) - pos):mag() * 0.0001
    print(amp)
    if speakerID == "left" then
      setP("leftAmp",amp)
      elseif speakerID == "right" then
        setP("rightAmp",amp)
        elseif speakerID == "centre" then
          setP("centreAmp",amp)
          elseif speakerID == "leftS" then
            setP("leftSAmp",amp)
            elseif speakerID == "rightS" then
              setP("rightSAmp",amp)
            else
            end
          end
        end
      end
    end
  end
end

function ai_Boid51()
  initai()
  local id = getID()
  local props = {Mass = 0.1,rate = 0.01,range = 300,cohesion = 0.7, alignment = 1,separation = 0.1,freq = 440,leftAmp = 0,rightAmp = 0, centreAmp = 0, leftSAmp = 0, rightSAmp = 0}
  addP(props)
  setP("Velocity",Vec2.new(20,0))
  changeMotionModel("Physics")
  setP("SynthDef","fiveOneTester")
  while true do
    local pos = getP("pos")
    setP("freq", pos:y())
    calculate_five_one_coefficients()
    boid_calculation(getP("pos"),getP("range"),getP("cohesion"),getP("alignment"),getP("separation"),ai_BoidGranulator)
    coroutine.yield()
  end
end
----------------------------------------------------------------------------

function ai_BoidFiveOneGranulator()
  initai()
  local id = getID()
  local props = {Mass = 0.1,rate = 0.01,range = 300,childAgent = "ai_GrainC",childDur = 0.5,xRandom = 3,yRandom = 3,childDur = 0.5,cohesion = 0.7, alignment = 1,separation = 0.1,freq = 440,leftAmp = 0,rightAmp = 0, centreAmp = 0, leftSAmp = 0, rightSAmp = 0}
  addP(props)
  setP("Velocity",Vec2.new(20,0))
  changeMotionModel("Physics")
  setP("SynthDef","fiveOneTester")
  print("boid 51 gran spawned")
  while true do
    local pos = getP("pos")
    local xRandom = getP("xRandom")
    local yRandom = getP("yRandom")
    local x = math.random(xRandom* -1,xRandom) * 0.5
    local y = math.random(yRandom* -1,yRandom) * 0.5
    spawnRelativeP({dur = getP("childDur")},id,1,getP("childAgent"), x,y)
    boid_calculation(getP("pos"),getP("range"),getP("cohesion"),getP("alignment"),getP("separation"),"ai_BoidFiveOneGranulator")
    coroutine.yield()
  end
end


function ai_GrainC()
  initai()
  local id = getID()
  local pos = getP("pos")
  local worldSize = getWorldSize()
  local props = {Mass = 0.1,speed = pos:y()/261.625,startpos = pos:x()/worldSize:x(),bufNum = 0,leftAmp = 0,rightAmp = 0, centreAmp = 0, leftSAmp = 0, rightSAmp = 0}
  addP(props)
  changeMotionModel("Physics")
  setP("SynthDef","fiveOneGrain")
  while true do
    pos = getP("pos")
    worldSize = getWorldSize()
    setP("speed",pos:y()/261.625)
    setP("startpos", pos:x() / worldSize:x() )
    calculate_five_one_coefficients()
    coroutine.yield()
  end
  die()
end


-----------------------------------------------------------------------------

function ai_ReverbController()
  initai()
  local worldsize = getWorldSize()
  local pos = getP("pos")
  local props = {outChannel = 0,mix = pos:y()/worldsize:y(),room = pos:x()/worldsize:x(), damp = 0.3,amp = 1}
  addP(props)
  changeMotionModel("Physics")
  setP("SynthDef","reverbController")
  print("Reverb initialized")
  while true do
    worldsize = getWorldSize()
    pos = getP("pos")
    setP("mix",pos:y()/worldsize:y())
    setP("room,",pos:x()/worldsize:x())
    coroutine.yield()
  end
end

function ai_DelayController()
  initai()
  local worldsize = getWorldSize()
  local pos = getP("pos")
  local props = {outChannel = 0,mix = 0.5,delayTime = pos:x()/worldsize:x() , decayTime =pos:y()/worldsize:y() * 10 ,amp = 1}
  addP(props)
  changeMotionModel("Physics")
  setP("SynthDef","delayController")
  print("Delay initialized")
  while true do
    worldsize = getWorldSize()
    pos = getP("pos")
    setP("decayTime",pos:y()/worldsize:y()* 10)
    setP("delayTime,",pos:x()/worldsize:x())
    coroutine.yield()
  end
end

------------------------------------------------------------------

function ai_PlayerReverb()
  initai()
  changeMotionModel("Physics")
  local pos = getP("pos")
  local worldsize = getWorldSize()
  local props = {Mass = math.random(0.01,0.6),amp = 0.4,mix =pos:y()/worldsize:y() ,room = pos:x() / worldsize:x() ,attraction = 1.2,dur = 20}
  addP(props)
  setP("Velocity",Vec2.new(math.random(-30,30),math.random(-30,30)))
  setP("SynthDef","reverbPlayer")
  while true do
    pos = getP("pos")
    setP("mix",pos:y()/worldsize:y())
    setP("room", pos:x() / worldsize:x() )
    local nearby = getNearby(getP("range"))
    local avgPos = Vec2.new(0,0)
    local pos = getP("pos")
    for i = 1, #nearby do
      avgPos = avgPos + getPID("pos",nearby[i])
    end
    avgPos = avgPos/#nearby
    if avgPos:x() > 0 and avgPos:y() > 0 then
      local dir = (avgPos - pos):norm()
      setP("Thrust",dir* getP("attraction") * 0.5)
    end
    coroutine.yield()
  end
  die()
end

-----------------------------------------------------------------
function ai_PathDrawer()
  initai()
  local rt = 0
  local gt = 0
  local bt = 0
  local props = {Red = 0 , Green = 0,Blue = 0,RedInc = 0.01,RedRange = 125,GreenRange = 125,BlueRange = 125,GreenInc = 0.1,BlueInc = 0.25,Radius = 20,Thickness = -1}
  addP(props)
  setP("Velocity",Vec2.new(10,10))
  setP("MotionModel","path")
  while true do
    local pos = getP("pos")
    local r = getP("Red") + (math.sin(rt) * getP("RedRange"))
    local g = getP("Green") + (math.sin(gt) * getP("GreenRange"))
    local b = getP("Blue") + (math.sin(bt) * getP("BlueRange"))
    drawCircle(currentScreen(),pos,getP("Radius"),r,g,b,getP("Thickness"))
    rt = rt + getP("RedInc")
    gt = gt + getP("GreenInc")
    bt = bt + getP("BlueInc")
    coroutine.yield()
  end
  die()
end

function find_red_max(screen ,pos, thres )
  local ret = pos
  local max = thres
  for n = 0, 50, 1 do
    local a = (2 * math.pi / 10) * n
    local v = pos - Vec2.new(math.cos(a)*10,math.sin(a)*10)
    local mag = getRedFromPoint(screen,v)
    if mag > max then
      max = mag
      ret = v
    end
  end
  return ret
end

function move_to_peak_red(pos,attract)
 local target = find_red_max(currentScreen(),pos, 30)
 setP("Thrust",  (target - pos):norm() *attract * 0.1    )
end


function ai_RedAttract()
  initai()
  local props = {freq = 440,amp = 0.1,pan = 0}
  addP(props)
  setP("SynthDef","default")
  setP("MotionModel","physics")
  setP("Velocity",Vec2.new(math.random(-10,10),math.random(-10,10)))
  while true do
    local pos =  getP("pos")
    local worldSize = getWorldSize()
    local col = getRGBA(currentScreen())
    setP("freq", pos:y())
    setP("pan", ( (  pos:x() / worldsize:x())*2) -1)
    setP("amp", math.pow( col:r()/255,2) )
    setP("Thrust",getP("Thrust") * 0.33   )
    move_to_peak_red(pos,1)
    coroutine.yield()
  end
end

----------------------------------------------------------------------------------
function find_blue_max(screen ,pos, thres )
  local ret = pos
  local max = thres
  for n = 0, 50, 1 do
    local a = (2 * math.pi / 10) * n
    local v = pos - Vec2.new(math.cos(a)*50,math.sin(a)*50)
    local mag = getBlueFromPoint(screen,v)
    if mag > max then
      max = mag
      ret = v
    end
  end
  return ret
end

function avoid_peak_blue(pos,avoid)
 local target = find_blue_max(currentScreen(),pos, 30)
 setP("Thrust", getP("Thrust") +( (target - pos):norm()* 0.1 * -1) * avoid    )
end



function ai_RedCulture()
  initai()
  local props = {rate = 1,amp = 0.1,pan = 0}
  addP(props)
  setP("SynthDef","player")
  changeMotionModel("Physics")
  setP("Thrust",Vec2.new(math.random(-10,10),math.random(-10,10)))
  setP("Velocity",Vec2.new(math.random(-10,10),math.random(-10,10)))
  while true do
    local pos =  getP("pos")
    local worldSize = getWorldSize()
    local col = getRGBA(currentScreen())
    setP("rate", pos:y()/261.625)
    setP("pan",((pos:x()/worldSize:x()) * 2)-1 )
    setP("amp", math.pow( col:r()/255,2) )
    setP("Thrust",getP("Thrust") * 0.33   )
    move_to_peak_red(pos,1.2)
    avoid_peak_blue(pos,0.5)
    if col:r() > 150 then
      spawnRelative(getParent(),1,"ai_RedCulture", math.random(-10,10),math.random(-10,10))
    end
    if col:b() > 180 then
      die()
    end
    drawCircle(currentScreen(),pos,5,col:r()* 0.5,col:g()* 0.75,col:b()*0.9,-1 )
    coroutine.yield()
  end
end


function ai_ExampleCulture()
  initai()
  removePath(0)
  local total = 30
  for i = 0, total do
    local x = (math.sin(i) * 200) + 250
    local y = (math.cos(i)* 200) + 250
    addPathPoint(0,Vec2.new(x,y))
  end
  local child  = spawn(getID(),1,"ai_PathDrawer",0,0)
  pause(0.2)
  setPID("pspeed",3,child)
  child  = spawn(getID(),1,"ai_PathDrawer",0,0)
  pause(0.2)
  setPID("pspeed",0.5,child)
  local worldSize = getWorldSize()
  for i = 0, 3 do
    spawn(getID(),1, "ai_RedCulture",math.random(0,worldSize:x()),math.random(0,worldSize:y()))
  end
  while true do
    coroutine.yield()
  end
  die()
end


-------------------------------------------------------------------
--This is a template for live coding.....
function ai_LiveSpawn()
  initai()
  addP("UpdateRate",0.25)
  local id = getID()
  local run = true;
  local count = 0
  while run do
    run = live_code(id,count) ---- live code callback function
    count = count +1
    pause(getP("UpdateRate"))
  end
  die()
end

--live code in this function..
function live_code(parentid,count)
  if count % 2 == 0 then
    spawnP({bufNum = 2 },parentid,1,"ai_GravityExample",100,100)
  end
  if count % 4 == 0 then
    spawnP({bufNum = 1 },parentid,1,"ai_GravityExample",100,400)
  end
  return true
end



----------------------------------------------------------------



function ai_HorizontalFollower()
  initai()
  changeMotionModel("Physics")
  local pos = getP("pos")
  local worldsize = getWorldSize()
  local props = {Mass = math.random(0.01,0.6),amp = 0.4,speed =pos:y()/261.625 ,startpos = pos:x() / worldsize:x() ,attraction = 1.2,dur = 30}
  addP(props)
  setP("SynthDef","grain")
  while t < dur do
    pos = getP("pos")
    setP("speed",pos:y()/261.625)
    setP("startpos", pos:x() / worldsize:x() )
    setP("amp",math.pow( getMeanCol(currentScreen()) /255,2 ))
    drawCircle(currentScreen(),pos,3,RGBA.new(255,0,0,150),-1)
    local targetId = getNearest(pos)
    local targetPos = getPID("pos",targetId)
    local dir = (targetPos - pos):norm()
    setP("Thrust", Vec2.new(dir:x() * getP("attraction"),getP("Thrust"):y()))
    coroutine.yield()
  end
  die()
end



function ai_HorizontalSim()
  initai()
  setP("pos",Vec2.new(60,60))
  local props = {FlockAttraction = 2,run = 1}
  addP(props)
  for i = 1, 6 , 1 do
     spawn(getID(),1,"ai_HorizontalFollower",(i * 100) + math.random(-10,10),60 + (i * 100) + math.random(-5,5))
  end
  pause(1)
  while getP("run") > 0 do
    local attract = getP("FlockAttraction")
    for i = 0 , numberOfChildren()-1, 1 do
      setPID("attraction",attract, getChild(i))
    end
    coroutine.yield()
  end
end


-------------------------------------------------------

function ai_ConstantCanopy()
  initai()
  local props = {run = 1,spawnrate=0.2,childBuffer=0,childVel = Vec2.new(20,0),grainsize=1,childAmp = 0.2}
  addP(props)
  changeMotionModel("Physics")
  local id = getID()
  while getP("run") > 0 do
    local childid = spawnRelative(id,1,"ai_GrainA",(math.random()* 5) - 5*0.5,     (math.random()* 5) - 5 *0.5 )
    local args = {bufNum= getPID("childBuffer",id),Velocity = getPID("childVel",id),amp =getPID("childAmp",id),dur = getPID("grainsize",id)  }
    setPID(args,childid)
    pause(getPID("spawnrate",id))
  end
  die()
end

----------------------------------------------------------


function ai_QuadGranularController()
  initai()
  local space = 30
  local props = {run = 1, rate = 0.01,ChildAPos = Vec2.new(0,space *-2) , ChildBPos = Vec2.new(0,space *-1),ChildCPos = Vec2.new(0,space ) , ChildDPos = Vec2.new(0,space *2)}
  local id = getID()
  addP(props)
  local childA = spawnRelative(id,1,"ai_ColourGranulator",getP("ChildAPos"))
  local childB = spawnRelative(id,1,"ai_ColourGranulator",getP("ChildBPos"))
  local childC = spawnRelative(id,1,"ai_ColourGranulator",getP("ChildCPos"))
  local childD = spawnRelative(id,1,"ai_ColourGranulator",getP("ChildDPos"))
  setP("Velocity",Vec2.new(3,0))
  while getP("run") > 0 do
    local pos = getP("pos")
    setPID("pos",getP("ChildAPos") + pos,childA)
    setPID("pos",getP("ChildBPos") + pos,childB)
    setPID("pos",getP("ChildCPos") + pos,childC)
    setPID("pos",getP("ChildDPos") + pos,childD)
    pause(getP("rate"))
  end
  die()
end
----------------------------------------------------------


function ai_QuadClusterGranularController()
  initai()
  local space = 50
  local props = {run = 1, rate = 1,implosion = 0.4,ChildAPos = Vec2.new(space * -1,space * -1) , ChildBPos = Vec2.new(space ,space * -1),ChildCPos = Vec2.new(space ,space ) , ChildDPos = Vec2.new(space * -1,space)}
  local id = getID()
  addP(props)
  local childA = spawnRelative(id,1,"ai_ColourGranulator",getP("ChildAPos"))
  local childB = spawnRelative(id,1,"ai_ColourGranulator",getP("ChildBPos"))
  local childC = spawnRelative(id,1,"ai_ColourGranulator",getP("ChildCPos"))
  local childD = spawnRelative(id,1,"ai_ColourGranulator",getP("ChildDPos"))
  while getP("run") > 0 do
    print("running")
    local pos = getP("pos")
    setPID("pos",getP("ChildAPos") + pos,childA)
    setPID("pos",getP("ChildBPos") + pos,childB)
    setPID("pos",getP("ChildCPos") + pos,childC)
    setPID("pos",getP("ChildDPos") + pos,childD)
    print("positions set")
    setPID("Velocity",Vec2.new(0,0),childA)
    setPID("Velocity",Vec2.new(0,0),childB)
    setPID("Velocity",Vec2.new(0,0),childC)
    setPID("Velocity",Vec2.new(0,0),childD)
    local diff = pos - getPID("pos",childA)
    local dir = diff:norm() * getP("implosion")
    setPID("Thrust",dir * 0.5 ,childA)
    diff = pos - getPID("pos",childB)
    dir = diff:norm() * getP("implosion")
    setPID("Thrust",dir * 0.5,childB)
    diff = pos - getPID("pos",childC)
    dir = diff:norm() * getP("implosion")
    setPID("Thrust",dir * 0.5,childC)
    diff = pos - getPID("pos",childD)
    dir = diff:norm() * getP("implosion")
    setPID("Thrust",dir * 0.5,childD)
    pause(getP("rate"))
  end
  die()
end


----------------------------------------------------------------------------------------------

--ai_PosPlayer
function ai_StackSpawn()
  initai()
  local props = {dur =5, childTotal = 10,stackDistribution=4,stackHeight = 400,childType = "ai_GrainA",run = 1}
  addP(props)
  local id = getID()
  while getP("run")>0 do
    local args = {dur = getPID("dur",id)}
    local total =getPID("childTotal",id)
    local height = getPID("stackHeight",id)
    for i = 0, total ,1 do
      local ratio = getPID("stackDistribution",id)
      local child  = spawnRelativeP(args,id,1,getPID("childType",id), 0,math.pow(i/total,ratio) *height )
    end
    pause(getPID("dur",id))
  end
  die()
end


function ai_EndlessStacks()
  initai()
  local props = {xdistribution = 50,ydistribution = 50,childType = "ai_StackSpawn",randomTime = 5, run = 1}
  addP(props)
  local id = getID()
  while getP("run")>0 do
    local pos = getPID("pos",id)
    local xdist = getPID("xdistribution",id)
    local ydist = getPID("ydistribution",id)
    spawnRelative(id,1,getPID("childType",id), math.random(xdist * -1.0 , xdist ), math.random(ydist * -1.0 , ydist ) )
    pause(math.random(0,getPID("randomTime",id)))
  end
  die()
end

---------------------------------------------------------------------------------------------------


function ai_Mangler()
  initai()
  local args = {range = 9,offset = 0,childdur = 0.2,rate = 0.01,x = 50,y = 50,amp = 0.1,rrate =  0.4,run = 1}
  addP(args)
  local id = getID()
  changeMotionModel("Physics")
  while getP("run")>0 do
    local args = {dur = getP("childdur"),amp = getP("amp"),bufNum = math.random()* getP("range") + getP("offset")}
    local x = getP("x")
    local y = getP("y")
    local child = spawnRelativeP(args,id,1,"ai_GrainA",(math.random()* x)- x*0.5,(math.random()* y)- y*0.5 )
    pause(getP("rate"))
    setPID(args,child)
  end
  die()
end

function ai_ColourMangler()
  initai()
  local args = {range = 9,offset = 0,childdur = 0.2,rate = 0.01,x = 50,y = 50,amp = 0.1,rrate =  0.4,run = 1 }
  addP(args)
  local id = getID()
  changeMotionModel("Physics")
  while getP("run")>0 do
    local colAvg = getMeanCol(currentScreen()) /255;
    local args = {dur = getP("childdur"),amp = getP("amp"),bufNum = (colAvg * getP("range") )+getP("offset") }
    local x = getP("x")
    local y = getP("y")
    local child = spawnRelativeP(args,id,1,"ai_GrainA",(math.random()* x)- x*0.5,(math.random()* y)- y*0.5 )
    pause(getP("rate"))
    setPID(args,child)
  end
  die()
end
----------------------------------------------------------------------------------------------------------


--------------------------------------------------------------------------------------------------
function ai_Predator()
	initai()
	changeMotionModel("Physics")
	setP("Velocity",Vec2.new(math.random(-150,150),math.random(-150,150)  ))
	setP("dur",10)
	local kills = 0
	local run = true
	while run do
   if kills > 3 then
    run = false
    break
  end
  local nearby = getNearby(500)
  local closestPrey = 0
  local diff = 1000
  local found =false
  local currentDiff =1001
  for i = 1 , #nearby-1 do
    if hasPID("creature",nearby[i]) then
      found = true
      currentDiff =(getPID("pos",nearby[i]) - getP("pos"):mag()
      if currentDiff < diff then
        diff = currentDiff
        closestPrey = nearby[i]
      end
    end
  end
  if found == true then
   local targetPos = getPID("pos",closestPrey)
   local dir = (targetPos -pos ):norm()
   setP("Thrust",dir)
   if dir:mag() < 1 then
     killID(closestPrey)
     kills = kills + 1
   end
  end
  die()
end



function ai_MikroPreyGrainSynth( )
	initai()
	local props = {creature = 0,rate = 0.01,cdur = 0.5,xDeviation = 2, yDeviation=2,childType = "ai_GrainB",run = 1,cbufNum = 0, cAmp = 0.7,lowerR = 0.01,upperR =0.05,bufRange = 0,bufNum = 0}
	local id = getID()
	changeMotionModel("Physics")
	addP(props)
	setP("Velocity",Vec2.new( math.random(-100,100),math.random(-100,100)  ))
	while getPID("run",id) > 0 do
		local col = getMeanCol(0)
		local cArgs = {dur = getPID("cdur",id), bufNum = getPID("bufNum",id) + (col/255)*getPID("bufRange",id),creature = 0  }
		local xDev = getPID("xDeviation",id)
		local yDev = getPID("yDeviation",id)
		local child = spawnRelativeP(cArgs,id,1,getPID("childType",id), math.random(xDev* -1.0,xDev), math.random(yDev* -1.0,yDev)  )
		pause(  getPID("rate",id) + ((math.random(0,1000)/1000)*getPID("upperR",id)) + getPID("lowerR",id) )
	end
	pause(10)
	die()
end

function ai_MikroLifeSystem()
	initai()
	local props = {  rate = math.random(0,1000)/1000 * 3 ,grainThreshold = 90,predatorTreshold = 30,noPredatorSpawn = 5,childBuffer = 0}
	local id = getID()
	addP(props)
	while true do
		if math.random(0,100) <  getPID("grainThreshold",id) then
			local child = spawnRelative(id,1,"ai_MikroPreyGrainSynth",math.random(-100,100),math.random(-100,100))
			coroutine.yield()
			coroutine.yield()
			setPID("bufNum",getPID("childBuffer",id),child)
		end
		if math.random(0,100) <  getPID("predatorTreshold",id) then
			for i = 1, getPID("noPredatorSpawn",id), 1 do
				spawnRelative(id,1,"ai_Predator",math.random(-100,100),math.random(-100,100))
			end
		end
		pause(getP("rate"))
	end
	die()
end


function ai_RedGenerator()
  initai()
  changeMotionModel("Physics")
  local props = {AgentToSpawn = "ai_GravityGrain", spawnRate = 0.1,spawn = 1, threshold = 150}
  addP("AgentToSpawn","ai_GravityGrain")
  setP("Velocity",Vec2.new(10,0))
  setP("image",6)
  setP("dur",30)
  while getP("spawn") == true do
    local pos =  getP("pos")
    if getRedFromPoint(currentScreen(),pos) > getP("threshold") then
     spawnRelative(getID(),1,getP("AgentToSpawn"),0,0)
     pause(getP("spawnRate"))
   end
   coroutine.yield()
 end
end








