


(


//This file creates binary synthesizer files which the Agent Tool can invoke.
//To specify where to make the binary change the location of "dir" global e.g ~dir = "/Users/stephen/AgentToolStuff/Synths/"
~dir= SynthDef.synthDefDir;

//~dir= "/Users/stephen/AgentBinSynths/";

SynthDef.writeOnce("TestSynth",{
	|freq = 440|
	Out.ar(0,SinOsc.ar(freq)* 0.1);
},dir: ~dir);

SynthDef.writeOnce(\grain,{
	|speed=1,startpos= 0,amp =0.8,dur = 1,bufNum=0,pan=0,envShape = 1,sChannel = 16,sAmount =1 ,outAmp = 1,out = 0|
	var audio,env,shapes;
	audio=  PlayBuf.ar(bufNum.numChannels,bufNum, BufRateScale.kr(bufNum)*speed,1, BufFrames.kr(bufNum)* startpos,loop:1);
	shapes = [EnvGen.kr(Env.perc(0.01,dur-0.01)) ,EnvGen.kr(Env.sine(dur,1)) ];
	env = Select.kr(envShape,shapes);
	audio = Pan2.ar(  audio * env ,pan )*amp  ;
	Out.ar(0,audio * outAmp);
	Out.ar(sChannel,audio * sAmount);
},dir:~dir);

SynthDef.writeOnce(\grains,{
		|channels = 2,triggerRate = 1,bufNum = 0,rate = 1, centrePos =0,gsize = 0.25,pan = 0, amp = 0.2,interp = 4,bits = 23,maxDel = 4,delay = 0.1,decay = 0.5,dry = 0.5,wet = 0.5,tType = 0,sChannel = 16,sAmount =1 ,outAmp = 1,out = 0|
	var trigger,audio,t;
	t = [Impulse.ar(triggerRate),Dust2.ar(triggerRate)];
	trigger = Select.ar(tType,t);
	audio = TGrains.ar(2,trigger,bufNum,rate,centrePos,gsize,pan,amp,interp);
	audio = MantissaMask.ar(audio,bits);
	audio = ( CombL.ar(audio,maxDel,delay,decay) * wet )+ (audio * dry);
	Out.ar(out,audio * outAmp);
	Out.ar(sChannel,audio * sAmount);
},dir:~dir);

SynthDef.writeOnce(\bubble,{
	|speed=1,startpos= 0,amp =0.1,dur = 1,bufNum = 0|
	Out.ar([0,1],PlayBuf.ar(1,bufNum,speed,startPos: BufFrames.kr(bufNum) * startpos   )* EnvGen.kr(Env.sine(dur*0.9,1))  *amp   );
},dur:~dir);


SynthDef.writeOnce(\metamu,{
	|pos_x,pos_y,dur = 1,amp=0.02,r=0,g = 0,sChannel = 16,sAmount =1 ,outAmp = 1,out = 0|
	var partials = [1,2,3,5,6,10,12];
	var audio = SinOsc.ar((pos_y*0.2 )* (pos_y*0.2 ),0,[r,g ]/255);
	audio = audio*amp;
	Out.ar([0,1],audio *outAmp);
	Out.ar(sChannel,audio * sAmount);
},dir:~dir);

SynthDef.writeOnce(\test,{
	|pos_x = 300,amp = 0.05,pos_y |
	Out.ar([0,1],Pan2.ar(   Mix.ar(  SinOsc.ar(   (pos_y*0.2 )* (pos_y*0.2 )*[1,10,12,13],0,0.3  ) ), ((pos_x/900)*2)-1) *amp  );},dir:~dir);



SynthDef.writeOnce(\ringmodEffect,{
	| inputChannel=16,channelCount =2 , shape = 0,freq = 220,width = 0.5, amp=1,out1 = 0,out2 = 1|
	var audio, t;
	t = [SinOsc.ar(freq),Saw.ar(freq),Pulse.ar(freq,width),LFTri.ar(freq),LFNoise2.ar(freq),WhiteNoise.ar()];
	audio = In.ar(inputChannel,1) * Select.ar(shape,t);
	audio = audio *  amp;
	Out.ar([out1,out2],audio );
},dir:~dir);


SynthDef.writeOnce(\player,{
	|rate = 1,bufNum = 0, a = 0.01, d = 0.01,s = 1,r = 0.01,gate = 1 ,out = 0, pan = 0,start = 0 ,loop = 0,dur = 10,amp = 0.5|
	var audio, env;
	audio = PlayBuf.ar(1,bufNum, rate,1,start * BufFrames.kr(bufNum),loop);
	env = EnvGen.kr(Env.adsr(dur*a,dur*d,s,dur*r ),gate,doneAction:2);
	Out.ar(out,Pan2.ar(amp*audio*env,pan  ));

},dir:~dir);



SynthDef.writeOnce(\playerMorpher,{
	|rate = 1,bufANum = 0,bufBNum = 1,bufCNum = 2, a = 0.01, d = 0.01,s = 1,r = 0.01,gate = 1 ,out = 0, pan = 0,start = 0 ,loop = 1,dur = 10,amp = 0.5,chans = 1,bufAAmp = 0.3,bufBamp = 0.3, bufCAmp = 0.3|
	var audio, env;
	audio = PlayBuf.ar(1,bufANum, rate,1,start * BufFrames.kr(bufANum),loop) * bufAAmp;
	audio = audio + PlayBuf.ar(1,bufBNum, rate,1,start * BufFrames.kr(bufBNum),loop)*  bufBamp;
	audio = audio + PlayBuf.ar(1,bufCNum, rate,1,start * BufFrames.kr(bufCNum),loop) * bufCAmp;
	env = EnvGen.kr(Env.adsr(dur*a,dur*d,s,dur*r ),gate,doneAction:2);
	Out.ar(out,Pan2.ar(amp*audio*env,pan  ));

},dir:~dir);





SynthDef.writeOnce(\reverbController,{
	|outChannel = 0, mix = 0.5,room = 0.6, damp = 0.3,amp = 1|
	var audio;
	audio = In.ar(outChannel,2);
	audio = FreeVerb2.ar(audio[0],audio[1],mix,room,damp,amp);
	ReplaceOut.ar(outChannel,audio);
},dir:~dir);

SynthDef.writeOnce(\delayController,{
	|outChannel = 0, mix = 0.5, maxDelay = 2,delayTime = 0.1,decayTime = 1,amp = 1|
	var audio;
	audio = In.ar(outChannel,2);
	audio = audio*(1.0-mix) + CombL.ar(audio,maxDelay,delayTime,decayTime,amp) * mix;
	ReplaceOut.ar(outChannel,audio);
},dir:~dir);


SynthDef.writeOnce(\reverbPlayer,{
	|mix = 0.5,room = 0.8,rate = 1,bufNum = 0, a = 0.01, d = 0.01,s = 1,r = 0.01,gate = 1 ,out = 0, pan = 0,start = 0 ,loop = 1,dur = 10,amp = 0.5,chans = 1|
	var audio, env;
	audio = PlayBuf.ar(1,bufNum, rate,1,start * BufFrames.kr(bufNum),loop);
	audio = FreeVerb.ar(audio,mix,room) ;
	env = EnvGen.kr(Env.adsr(dur*a,dur*d,s,dur*r ),gate,doneAction:2);
	Out.ar(out,Pan2.ar(amp*audio*env,pan  ));

},dir:~dir);


SynthDef.writeOnce(\fiveOnePlayer,{
	|rate = 1,bufNum = 0, a = 0.01, d = 0.01,s = 1,r = 0.01,gate = 1 ,out = 0,start = 0 ,loop = 1,dur = 10,amp = 0.5,chans = 1, left = 0,right = 1, centre = 3,leftS = 4, rightS = 5,leftAmp = 0,rightAmp = 0,centreAmp = 0, leftSAmp= 0, rightSAmp = 0|
	var audio, env;
	audio = PlayBuf.ar(1,bufNum, rate,1,start * BufFrames.kr(bufNum),loop);
	env = EnvGen.kr(Env.adsr(dur*a,dur*d,s,dur*r ),gate,doneAction:2);
	Out.ar([left,right,centre,leftS,rightS],amp*audio*env * [leftAmp,rightAmp,centreAmp,leftSAmp,rightSAmp]);
},dir:~dir);


SynthDef.writeOnce(\fiveOneTester,{
	|freq = 440, a = 0.01, d = 0.01,s = 1,r = 0.01,gate = 1 ,out = 0,start = 0 ,loop = 1,dur = 10,amp = 0.5,chans = 1, left = 0,right = 1, centre = 3,leftS = 4, rightS = 5,leftAmp = 0,rightAmp = 0,centreAmp = 0, leftSAmp= 0, rightSAmp = 0|
	var audio, env;
	audio = SinOsc.ar(freq);
	env = EnvGen.kr(Env.adsr(dur*a,dur*d,s,dur*r ),gate,doneAction:2);
	Out.ar([left,right,centre,leftS,rightS],amp*audio*env * [leftAmp,rightAmp,centreAmp,leftSAmp,rightSAmp]);
},dir:~dir);


SynthDef.writeOnce(\fiveOneGrain,{
	|speed = 1, startpos = 0,bufNum = 0,envShape , a = 0.01, d = 0.01,s = 1,r = 0.01,gate = 1 ,out = 0,start = 0 ,loop = 1,dur = 1,amp = 0.5,chans = 1, left = 0,right = 1, centre = 3,leftS = 4, rightS = 5,leftAmp = 0,rightAmp = 0,centreAmp = 0, leftSAmp= 0, rightSAmp = 0|
	var audio, env,shapes;
	shapes = [EnvGen.kr(Env.perc(0.01,dur-0.01)) ,EnvGen.kr(Env.sine(dur,1)) ];
	env = Select.kr(envShape,shapes);
	audio = PlayBuf.ar(bufNum.numChannels,bufNum, BufRateScale.kr(bufNum)*speed,1, BufFrames.kr(bufNum)* startpos,loop:1);
	env = EnvGen.kr(Env.adsr(dur*a,dur*d,s,dur*r ),gate,doneAction:2);
	Out.ar([left,right,centre,leftS,rightS],amp*audio*env * [leftAmp,rightAmp,centreAmp,leftSAmp,rightSAmp]);
},dir:~dir);




)

